@@ -1204,7 +1204,7 @@ MENU_DestroyMenus(ScreenInfo *scr)
  * reset the menuing system
  */
 void
-MENU_Reset()
+MENU_Reset(void)
 {
     ActiveItem = NULL;
     ActiveMenu = NULL;
