@@ -41,6 +41,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #ifdef TIME_WITH_SYS_TIME
@@ -60,8 +63,7 @@
 #include <Xm/DisplayP.h>
 #include <Xm/ScreenP.h>
 
-#ifdef	HAVE_GETRLIMIT
-#include <sys/types.h>
+#ifdef HAVE_GETRLIMIT
 #include <sys/resource.h>
 #endif
 
@@ -148,11 +150,11 @@ sig_alarm(int nonsense)
 /*
  * figures out if there's another WM running
  */
-static XErrorHandler
+static void
 catch_redirect(Display *dpy, XErrorEvent *event)
 {
     fprintf(stderr, "mwm: Error: Another WM is running\n");
-    exit(1);
+    exit(1); /* we don't return (this determines function type) */
 }
 
 /*
