@@ -1,6 +1,6 @@
 /**
  *
- * $Id: resources.c,v 1.7 1998/10/12 06:11:11 gritton Exp $
+ * $Id: resources.c,v 1.9 2000/04/29 17:13:53 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -214,36 +214,6 @@ const char mwm_strings[] =
 "SmartPlacement\0"
 "\0";
 
-extern void _WmMultiClickTimeDefault(Widget w, int offset, XrmValue *val);
-extern void _WmFocusAutoRaiseDefault(Widget w, int offset, XrmValue *val);
-extern void _WmDefaultBorderWidth(Widget w, int offset, XrmValue *val);
-extern void _WmDefaultResizeBorderWidth(Widget widget, int offset, XrmValue *val);
-extern void _WmIconImageBDefault(Widget w, int offset, XrmValue *val);
-extern void _WmIconImageBSCDefault(Widget w, int offset, XrmValue *val);
-extern void _WmIconImageBSPDefault(Widget w, int offset, XrmValue *val);
-extern void _WmIconImageFDefault(Widget w, int offset, XrmValue *val);
-extern void _WmIconImageTSCDefault(Widget w, int offset, XrmValue *val);
-extern void _WmIconImageTSPDefault(Widget w, int offset, XrmValue *val);
-extern void _WmMatteBDefault(Widget w, int offset, XrmValue *val);
-extern void _WmMatteBSCDefault(Widget w, int offset, XrmValue *val);
-extern void _WmMatteBSPDefault(Widget w, int offset, XrmValue *val);
-extern void _WmMatteFDefault(Widget w, int offset, XrmValue *val);
-extern void _WmMatteTSCDefault(Widget w, int offset, XrmValue *val);
-extern void _WmMatteTSPDefault(Widget w, int offset, XrmValue *val);
-extern void _WmBackgroundDefault(Widget w, int offset, XrmValue *val);
-extern void _WmBackgroundPixmapDefault(Widget w, int offset, XrmValue *val);
-extern void _WmBottomShadowColorDefault(Widget w, int offset, XrmValue *val);
-extern void _WmBottomShadowPixmapDefault(Widget w, int offset, XrmValue *val);
-extern void _WmForegroundDefault(Widget w, int offset, XrmValue *val);
-extern void _WmTopShadowColorDefault(Widget w, int offset, XrmValue *val);
-extern void _WmTopShadowPixmapDefault(Widget w, int offset, XrmValue *val);
-extern void _WmABackgroundDefault(Widget w, int offset, XrmValue *val);
-extern void _WmAForegroundDefault(Widget w, int offset, XrmValue *val);
-extern void _WmABottomShadowColorDefault(Widget w, int offset, XrmValue *val);
-extern void _WmATopShadowColorDefault(Widget w, int offset, XrmValue *val);
-extern void _WmABackgroundPixmapDefault(Widget w, int offset, XrmValue *val);
-extern void _WmATopShadowPixmapDefault(Widget w, int offset, XrmValue *val);
-
 /*
  * Syntax:
  * "Mwm*resource_name: resource_value"
@@ -1591,7 +1561,7 @@ RES_GetClientDefaults(ScreenInfo *scr, MwmWindow *tmp_win,
 }
 
 void
-RES_Initialize()
+RES_Initialize(void)
 {
     XmRegisterConverters();
 
