@@ -1,6 +1,6 @@
 
 /* 
- * $Id: lookup.c,v 1.3 1999/05/28 20:43:20 danny Exp $
+ * $Id: lookup.c,v 1.4 2000/01/26 12:43:39 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -1399,7 +1399,7 @@ static Bool initialized = False;	/* easier to fill in at run time */
  *      Used for translation value or delimiter significance lookup.
  */
 static void
-initHexTable()
+initHexTable(void)
 {
     /*
      * We build the table at run time for several reasons:
