@@ -1,6 +1,7 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright (c) 1999-2000 by the LessTif Developers.
  *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
@@ -21,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Macros.h,v 1.8 2000/03/28 21:44:08 danny Exp $
+ * $Id: Macros.h,v 1.16 2000/06/11 17:58:39 danny Exp $
  */
 
 /*
@@ -31,10 +32,6 @@
 #ifndef _Xbae_Macros_h
 #define _Xbae_Macros_h
 
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
-
 #if XmVersion <= 1001
 #include <Xm/XmP.h>
 #else
@@ -43,6 +40,11 @@ extern "C" {
 #include <Xm/TextP.h>
 #include <Xm/PrimitiveP.h>
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * The third parameter is a string added for debugging purposes.
  */
@@ -54,8 +56,12 @@ extern "C" {
 	_XmDrawShadows(dpy, draw, tgc, bgc, x, y, w, h, sz, type);	\
     }
 #else
-#if 0
-/*	for BX */
+#if 1
+/*
+ * BX (Builder Xcessory - see http://www.ics.com) is linked with a 1.2
+ * version of Motif. Use of an Xbae widget which uses a Motif 2.* function
+ * will crash BX when it tries to use the function.
+ */
 #define DRAW_SHADOW(dpy, draw, dbg, tgc, bgc, sz, x, y, w, h, type)	\
     _XmDrawShadows(dpy, draw, tgc, bgc, x, y, w, h, sz, type)
 
@@ -137,9 +143,18 @@ extern "C" {
 				 mw->matrix.cell_highlight_thickness + \
 				 mw->matrix.text_shadow_thickness)
 
-#define SOME_ROW_HEIGHT(mw, row)	((mw->matrix.row_heights[row] * \
-				  FONT_HEIGHT(mw)) + \
-				 ((int)TEXT_HEIGHT_OFFSET(mw) * 2))
+#ifdef	HAVE_ROW_HEIGHTS
+#define SOME_ROW_HEIGHT(mw, row)	mw->matrix.row_heights[row]
+/* this is the original ...
+ * #define ROW_HEIGHT(mw) \
+ *	(int)((TEXT_HEIGHT_OFFSET(mw) * 2) + TEXT_HEIGHT(mw))
+ * #define TEXT_HEIGHT(mw)		(Max(FONT_HEIGHT(mw), LABEL_HEIGHT(mw)))
+ * #define FONT_HEIGHT(mw)		(mw->matrix.font_height)
+ * #define LABEL_HEIGHT(mw)	(mw->matrix.label_font_height)
+ */
+#else
+#define SOME_ROW_HEIGHT(mw, row)	ROW_HEIGHT(mw)
+#endif
 
 #define COLUMN_WIDTH(mw, col)	((mw->matrix.column_widths[col] * \
 				  FONT_WIDTH(mw)) + \
@@ -288,10 +303,9 @@ extern "C" {
 #define TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw) (FIXED_COLUMN_LABEL_OFFSET(mw)+\
 						VISIBLE_WIDTH(mw))
 
-#define FIXED_ROW_HEIGHT(mw)	((int)mw->matrix.fixed_rows * ROW_HEIGHT(mw))
+#define	FIXED_ROW_HEIGHT(mw)	xbaeMatrixFixedRowHeight(mw)
 
-#define TRAILING_FIXED_ROW_HEIGHT(mw) ((int)mw->matrix.trailing_fixed_rows * \
-				       ROW_HEIGHT(mw))
+#define TRAILING_FIXED_ROW_HEIGHT(mw) xbaeMatrixTrailingFixedRowHeight(mw)
 
 #define ROW_LABEL_OFFSET(mw)	(COLUMN_LABEL_HEIGHT(mw) + \
 				 HORIZ_SB_OFFSET(mw) + \
@@ -537,9 +551,6 @@ extern "C" {
 #define CreateColumnPositions(mw) \
 	(int *)XtMalloc((mw->matrix.columns+1) * sizeof(int))
 
-#define YtoRow(mw, y)			((y) / ROW_HEIGHT(mw))
-
-
 /*
  * Evaluates to 1 if two Rectangles overlap, 0 if no overlap
  */
@@ -585,7 +596,7 @@ extern "C" {
 #endif
 
 #ifdef NEED_24BIT_VISUAL
-#define GC_PARENT_WINDOW(w)	XtWindow(get_shell_ancestor((Widget)w))
+#define GC_PARENT_WINDOW(w)	XtWindow(_XbaeGetShellAncestor((Widget)w))
 #else
 #define GC_PARENT_WINDOW(w)	RootWindowOfScreen(XtScreen(w))
 #endif
@@ -643,7 +654,7 @@ extern "C" {
 #define IN_GRID_ROW_MODE(mw)	(mw->matrix.grid_type & GRID_MODE_ROW)
 #define IN_GRID_COLUMN_MODE(mw)	(mw->matrix.grid_type & GRID_MODE_COLUMN)
 
-#if defined(__cplusplus) || defined(c_plusplus)
+#ifdef __cplusplus
 }
 #endif
 
