@@ -1,6 +1,8 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright (c) 1999-2000 by the LessTif Developers.
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -22,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: MatrixP.h,v 1.10 2000/03/22 21:45:49 danny Exp $
+ * $Id: MatrixP.h,v 1.15 2000/06/11 17:58:39 danny Exp $
  */
 
 /*
@@ -32,10 +34,6 @@
 #ifndef _Xbae_MatrixP_h
 #define _Xbae_MatrixP_h
 
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
-
 #if XmVersion <= 1001
 #	include <Xm/XmP.h>
 #else
@@ -44,6 +42,10 @@ extern "C" {
 
 #include <Xbae/Matrix.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * A few definitions we like to use, but those with R4 won't have.
  * From Xfuncproto.h in R5.
@@ -443,8 +445,16 @@ typedef struct _XbaeMatrixRec {
 
 int xbaeCellTotalHeight(XbaeMatrixWidget mw);
 
+/*
+ * Macro replacements
+ */
+extern Dimension xbaeMatrixFixedRowHeight(XbaeMatrixWidget mw);
+extern int YtoRow(XbaeMatrixWidget mw, Position y);
+extern Dimension xbaeMatrixTrailingFixedRowHeight(XbaeMatrixWidget);
+extern Widget _XbaeGetShellAncestor(Widget w);
+
 #undef P
-#if defined(__cplusplus) || defined(c_plusplus)
+#ifdef __cplusplus
 }
 #endif
 
