@@ -1,6 +1,7 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright (c) 1999-2000 by the LessTif Developers.
  *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
@@ -23,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Public.c,v 1.7 2000/01/03 22:33:03 danny Exp $
+ * $Id: Public.c,v 1.13 2000/06/11 17:58:50 danny Exp $
  */
 
 /*
@@ -49,11 +50,7 @@
  * Public interface to set_cell method
  */
 void
-XbaeMatrixSetCell(w, row, column, value)
-Widget w;
-int row;
-int column;
-const String value;
+XbaeMatrixSetCell(Widget w, int row, int column, const String value)
 {
     xbaeObjectLock(w);
     /*
@@ -75,9 +72,7 @@ const String value;
  * Public interface to edit_cell method
  */
 void
-XbaeMatrixEditCell(w, row, column)
-Widget w;
-int row, column;
+XbaeMatrixEditCell(Widget w, int row, int column)
 {
     xbaeObjectLock(w);
     /*
@@ -100,9 +95,7 @@ int row, column;
  * Public interface to select_cell method
  */
 void
-XbaeMatrixSelectCell(w, row, column)
-Widget w;
-int row, column;
+XbaeMatrixSelectCell( Widget w, int row, int column)
 {
     xbaeObjectLock(w);
     /*
@@ -123,9 +116,7 @@ int row, column;
  * Public interface to select_row method
  */
 void
-XbaeMatrixSelectRow(w, row)
-Widget w;
-int row;
+XbaeMatrixSelectRow(Widget w, int row)
 {
     xbaeObjectLock(w);
     /*
@@ -146,9 +137,7 @@ int row;
  * Public interface to select_column method
  */
 void
-XbaeMatrixSelectColumn(w, column)
-Widget w;
-int column;
+XbaeMatrixSelectColumn(Widget w, int column)
 {
     xbaeObjectLock(w);
     /*
@@ -169,8 +158,7 @@ int column;
  * Public interface to deselect_all method
  */
 void
-XbaeMatrixDeselectAll(w)
-Widget w;
+XbaeMatrixDeselectAll(Widget w)
 {
     xbaeObjectLock(w);
     /*
@@ -191,8 +179,7 @@ Widget w;
  * Public interface to select_all method
  */
 void
-XbaeMatrixSelectAll(w)
-Widget w;
+XbaeMatrixSelectAll(Widget w)
 {
     xbaeObjectLock(w);
     /*
@@ -213,10 +200,7 @@ Widget w;
  * Public interface to deselect_cell method
  */
 void
-XbaeMatrixDeselectCell(w, row, column)
-Widget w;
-int row;
-int column;
+XbaeMatrixDeselectCell(Widget w, int row, int column)
 {
     xbaeObjectLock(w);
     /*
@@ -237,9 +221,7 @@ int column;
  * Public interface to deselect_row method
  */
 void
-XbaeMatrixDeselectRow(w, row)
-Widget w;
-int row;
+XbaeMatrixDeselectRow(Widget w, int row)
 {
     xbaeObjectLock(w);
     /*
@@ -260,9 +242,7 @@ int row;
  * Public interface to deselect_column method
  */
 void
-XbaeMatrixDeselectColumn(w, column)
-Widget w;
-int column;
+XbaeMatrixDeselectColumn(Widget w, int column)
 {
     xbaeObjectLock(w);
     /*
@@ -283,9 +263,7 @@ int column;
  * Public interface to get_cell method
  */
 String
-XbaeMatrixGetCell(w, row, column)
-Widget w;
-int row, column;
+XbaeMatrixGetCell(Widget w, int row, int column)
 {
     String	s;
 
@@ -310,13 +288,7 @@ int row, column;
  * Public interface to commit_edit method
  */
 Boolean
-#if NeedFunctionPrototypes
 XbaeMatrixCommitEdit(Widget w, Boolean unmap)
-#else
-XbaeMatrixCommitEdit(w, unmap)
-Widget w;
-Boolean unmap;
-#endif
 {
     Boolean b;
 
@@ -341,13 +313,7 @@ Boolean unmap;
  * Public interface to cancel_edit method
  */
 void
-#if NeedFunctionPrototypes
 XbaeMatrixCancelEdit(Widget w, Boolean unmap)
-#else
-XbaeMatrixCancelEdit(w, unmap)
-Widget w;
-Boolean unmap;
-#endif
 {
     xbaeObjectLock(w);
     /*
@@ -368,13 +334,12 @@ Boolean unmap;
  * Public interface to add_rows method
  */
 void
-XbaeMatrixAddRows(w, position, rows, labels, colors, num_rows)
-Widget w;
-int position;
-String *rows;
-String *labels;
-Pixel *colors;
-int num_rows;
+XbaeMatrixAddRows(Widget w,
+		int position,
+		String *rows,
+		String *labels,
+		Pixel *colors,
+		int num_rows)
 {
     xbaeObjectLock(w);
     /*
@@ -395,10 +360,7 @@ int num_rows;
  * Public interface to delete_rows method
  */
 void
-XbaeMatrixDeleteRows(w, position, num_rows)
-Widget w;
-int position;
-int num_rows;
+XbaeMatrixDeleteRows(Widget w, int position, int num_rows)
 {
     xbaeObjectLock(w);
     /*
@@ -419,18 +381,16 @@ int num_rows;
  * Public interface to add_columns method
  */
 void
-XbaeMatrixAddColumns(w, position, columns, labels, widths, max_lengths,
-		     alignments, label_alignments, colors, num_columns)
-Widget w;
-int position;
-String *columns;
-String *labels;
-short *widths;
-int *max_lengths;
-unsigned char *alignments;
-unsigned char *label_alignments;
-Pixel *colors;
-int num_columns;
+XbaeMatrixAddColumns(Widget w,
+		int position,
+		String *columns,
+		String *labels,
+		short *widths,
+		int *max_lengths,
+		unsigned char *alignments,
+		unsigned char *label_alignments,
+		Pixel *colors,
+		int num_columns)
 {
     xbaeObjectLock(w);
     /*
@@ -452,10 +412,7 @@ int num_columns;
  * Public interface to delete_columns method
  */
 void
-XbaeMatrixDeleteColumns(w, position, num_columns)
-Widget w;
-int position;
-int num_columns;
+XbaeMatrixDeleteColumns(Widget w, int position, int num_columns)
 {
     xbaeObjectLock(w);
     /*
@@ -476,11 +433,7 @@ int num_columns;
  * Public interface to set_row_colors method
  */
 void
-XbaeMatrixSetRowColors(w, position, colors, num_colors)
-Widget w;
-int position;
-Pixel *colors;
-int num_colors;
+XbaeMatrixSetRowColors(Widget w, int position, Pixel *colors, int num_colors)
 {
     xbaeObjectLock(w);
     /*
@@ -501,11 +454,7 @@ int num_colors;
  * Public interface to set_column_colors method
  */
 void
-XbaeMatrixSetColumnColors(w, position, colors, num_colors)
-Widget w;
-int position;
-Pixel *colors;
-int num_colors;
+XbaeMatrixSetColumnColors(Widget w, int position, Pixel *colors, int num_colors)
 {
     xbaeObjectLock(w);
     /*
@@ -526,11 +475,7 @@ int num_colors;
  * Public interface to set_cell_color method
  */
 void
-XbaeMatrixSetCellColor(w, row, column, color)
-Widget w;
-int row;
-int column;
-Pixel color;
+XbaeMatrixSetCellColor(Widget w, int row, int column, Pixel color)
 {
     xbaeObjectLock(w);
     /*
@@ -551,11 +496,10 @@ Pixel color;
  * Public interface to set_row_colors method
  */
 void
-XbaeMatrixSetRowBackgrounds(w, position, colors, num_colors)
-Widget w;
-int position;
-Pixel *colors;
-int num_colors;
+XbaeMatrixSetRowBackgrounds(Widget w,
+			int position,
+			Pixel *colors,
+			int num_colors)
 {
     xbaeObjectLock(w);
     /*
@@ -576,11 +520,10 @@ int num_colors;
  * Public interface to set_column_colors method
  */
 void
-XbaeMatrixSetColumnBackgrounds(w, position, colors, num_colors)
-Widget w;
-int position;
-Pixel *colors;
-int num_colors;
+XbaeMatrixSetColumnBackgrounds(Widget w,
+				int position,
+				Pixel *colors,
+				int num_colors)
 {
     xbaeObjectLock(w);
     /*
@@ -601,11 +544,10 @@ int num_colors;
  * Public interface to set_cell_color method
  */
 void
-XbaeMatrixSetCellBackground(w, row, column, color)
-Widget w;
-int row;
-int column;
-Pixel color;
+XbaeMatrixSetCellBackground(Widget w,
+				int row,
+				int column,
+				Pixel color)
 {
     xbaeObjectLock(w);
     /*
@@ -627,11 +569,10 @@ Pixel color;
  * Return True on success, False on failure.
  */
 int
-XbaeMatrixGetEventRowColumn(w, event, row, column)
-Widget w;
-XEvent * event;
-int *row;
-int *column;
+XbaeMatrixGetEventRowColumn(Widget w,
+			XEvent * event,
+			int *row,
+			int *column)
 {
     XbaeMatrixWidget mw;
     int x, y;
@@ -666,11 +607,10 @@ int *column;
  * Public interface for xbaeEventToXY()
  */
 Boolean
-XbaeMatrixEventToXY(w, event, x, y)
-Widget w;
-XEvent *event;  
-int *x;
-int *y;
+XbaeMatrixEventToXY(Widget w,
+		XEvent *event,
+		int *x,
+		int *y)
 {
     XbaeMatrixWidget mw;
     CellType cell;
@@ -697,12 +637,11 @@ int *y;
  * (philipa@parallax.co.uk)
  */
 Boolean
-XbaeMatrixRowColToXY(w, row, column, x, y)
-Widget w;
-int row;
-int column;
-int *x;
-int *y;
+XbaeMatrixRowColToXY(Widget w,
+		int row,
+		int column,
+		int *x,
+		int *y)
 {
     XbaeMatrixWidget mw;
 
@@ -725,10 +664,9 @@ int *y;
  * Set the row & column to -1 if bad widget.  Maybe the program will core. :)
  */
 void
-XbaeMatrixGetCurrentCell(w, row, column)
-Widget w;
-int *row;
-int *column;
+XbaeMatrixGetCurrentCell(Widget w,
+		int *row,
+		int *column)
 {
     XbaeMatrixWidget mw;
 
@@ -749,8 +687,7 @@ int *column;
  * that it is really needed.
  */
 void
-XbaeMatrixRefresh(w)
-Widget w;
+XbaeMatrixRefresh(Widget w)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget)w;
     int x, y;
@@ -857,10 +794,7 @@ Widget w;
  * Public interface for redrawing one cell
  */
 void
-XbaeMatrixRefreshCell(w, row, column)
-Widget w;
-int row;
-int column;
+XbaeMatrixRefreshCell(Widget w, int row, int column)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget)w;
 
@@ -884,9 +818,7 @@ int column;
  * Redraw an entire column
  */
 void
-XbaeMatrixRefreshColumn(w, column)
-Widget w;
-int column;
+XbaeMatrixRefreshColumn(Widget w, int column)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget)w;
     int row;
@@ -926,9 +858,7 @@ int column;
  * Redraw an entire row
  */
 void
-XbaeMatrixRefreshRow(w, row)
-Widget w;
-int row;
+XbaeMatrixRefreshRow(Widget w, int row)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget)w;
     int column;
@@ -974,8 +904,7 @@ int row;
  *      - Cloned from the local "xbaeAdjustTopRow(mw)" routine.
  */
 int
-XbaeMatrixVisibleRows(w)
-Widget w;
+XbaeMatrixVisibleRows(Widget w)
 {
     XbaeMatrixWidget    matrix = (XbaeMatrixWidget) w;
 
@@ -1013,8 +942,7 @@ Widget w;
  *      - Cloned from the local "xbaeAdjustTopRow(mw)" routine.
  */
 int
-XbaeMatrixVisibleColumns (w)
-Widget w;
+XbaeMatrixVisibleColumns(Widget w)
 {
     XbaeMatrixWidget    matrix = (XbaeMatrixWidget)w;
 
@@ -1034,10 +962,7 @@ Widget w;
  * Get per-cell user data
  */
 XtPointer
-XbaeMatrixGetCellUserData(w, row, column)
-Widget w;
-int row;
-int column;
+XbaeMatrixGetCellUserData(Widget w, int row, int column)
 {
     XbaeMatrixWidget mw;
     XtPointer data;
@@ -1065,11 +990,7 @@ int column;
  * Set per-cell user data
  */
 void
-XbaeMatrixSetCellUserData(w, row, column, data)
-Widget w;
-int row;
-int column;
-XtPointer data;
+XbaeMatrixSetCellUserData(Widget w, int row, int column, XtPointer data)
 {
     XbaeMatrixWidget mw;
 
@@ -1104,9 +1025,7 @@ XtPointer data;
  * Get per-row user data
  */
 XtPointer
-XbaeMatrixGetRowUserData(w, row)
-Widget w;
-int row;
+XbaeMatrixGetRowUserData(Widget w, int row)
 {
     XbaeMatrixWidget mw;
     XtPointer data;
@@ -1135,10 +1054,7 @@ int row;
  * Set per-row user data
  */
 void
-XbaeMatrixSetRowUserData(w, row, data)
-Widget w;
-int row;
-XtPointer data;
+XbaeMatrixSetRowUserData(Widget w, int row, XtPointer data)
 {
     XbaeMatrixWidget mw;
 
@@ -1168,9 +1084,7 @@ XtPointer data;
  * Get per-column user data
  */
 XtPointer
-XbaeMatrixGetColumnUserData(w, column)
-Widget w;
-int column;
+XbaeMatrixGetColumnUserData(Widget w, int column)
 {
     XbaeMatrixWidget mw;
     XtPointer data;
@@ -1198,10 +1112,7 @@ int column;
  * Set per-column user data
  */
 void
-XbaeMatrixSetColumnUserData(w, column, data)
-Widget w;
-int column;
-XtPointer data;
+XbaeMatrixSetColumnUserData(Widget w, int column, XtPointer data)
 {
     XbaeMatrixWidget mw;
 
@@ -1231,11 +1142,10 @@ XtPointer data;
  * Set per-cell widget
  */
 void
-XbaeMatrixSetCellWidget(w, row, column, widget)
-	Widget w;
-	int row;
-	int column;
-	Widget widget;
+XbaeMatrixSetCellWidget(Widget w,
+			int row,
+			int column,
+			Widget widget)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget) w;
 
@@ -1345,9 +1255,7 @@ XbaeMatrixGetCellWidget(Widget w, int row, int column)
 }
 
 Boolean
-XbaeMatrixIsRowSelected(w, row)
-Widget w;
-int row;
+XbaeMatrixIsRowSelected(Widget w, int row)
 {
     int col;
     XbaeMatrixWidget mw;
@@ -1391,9 +1299,7 @@ int row;
 }
 
 Boolean
-XbaeMatrixIsColumnSelected(w, col)
-Widget w;
-int col;
+XbaeMatrixIsColumnSelected(Widget w, int col)
 {
     int row;
     XbaeMatrixWidget mw;
@@ -1437,10 +1343,7 @@ int col;
 }
 
 Boolean
-XbaeMatrixIsCellSelected(w, row, column)
-Widget w;
-int row;
-int column;
+XbaeMatrixIsCellSelected(Widget w, int row, int column)
 {
     XbaeMatrixWidget mw;
 
@@ -1481,8 +1384,7 @@ int column;
 }
 
 int
-XbaeMatrixFirstSelectedRow(w)
-Widget w;
+XbaeMatrixFirstSelectedRow(Widget w)
 {
     int i;
     XbaeMatrixWidget mw;
@@ -1516,8 +1418,7 @@ Widget w;
 }
 
 int
-XbaeMatrixFirstSelectedColumn(w)
-Widget w;
+XbaeMatrixFirstSelectedColumn(Widget w)
 {
     int i;
     XbaeMatrixWidget mw;
@@ -1551,10 +1452,7 @@ Widget w;
 }
 
 void
-XbaeMatrixFirstSelectedCell(w, row, column)
-Widget w;
-int *row;
-int *column;
+XbaeMatrixFirstSelectedCell(Widget w, int *row, int *column)
 {
     int i, j;
     XbaeMatrixWidget mw;
@@ -1591,8 +1489,7 @@ int *column;
 }
 
 int
-XbaeMatrixGetNumSelected(w)
-Widget w;
+XbaeMatrixGetNumSelected(Widget w)
 {
     XbaeMatrixWidget mw;
     int i;
@@ -1612,8 +1509,7 @@ Widget w;
 
 
 int
-XbaeMatrixNumColumns(w)
-Widget w;
+XbaeMatrixNumColumns(Widget w)
 {
     XbaeMatrixWidget mw;
 
@@ -1631,8 +1527,7 @@ Widget w;
 
 
 int
-XbaeMatrixNumRows(w)
-Widget w;
+XbaeMatrixNumRows(Widget w)
 {
     XbaeMatrixWidget mw;
 
@@ -1650,8 +1545,7 @@ Widget w;
 
 #if XmVersion >= 1002
 void
-XbaeMatrixUnhighlightAll(w)
-Widget w;
+XbaeMatrixUnhighlightAll(Widget w)
 {
     XbaeMatrixWidget mw;
     int row, column;
@@ -1690,10 +1584,7 @@ Widget w;
 
 
 void
-XbaeMatrixHighlightCell(w, row, column)
-Widget w;
-int row;
-int column;
+XbaeMatrixHighlightCell(Widget w, int row, int column)
 {
     XbaeMatrixWidget mw;
 
@@ -1757,16 +1648,25 @@ int column;
 	     */
 	    xbaeRowColToXY(mw, row, column, &x, &y);
 
+#ifdef	HAVE_ROW_HEIGHTS
 	    DRAW_HIGHLIGHT(XtDisplay(mw), win, 
-			   mw->manager.highlight_GC,
-			   x + mw->matrix.cell_shadow_thickness,
-			   y + mw->matrix.cell_shadow_thickness,
-			   COLUMN_WIDTH(mw, column) -
-			   (2 * mw->matrix.cell_shadow_thickness),
-			   ROW_HEIGHT(mw) -
-			   (2 * mw->matrix.cell_shadow_thickness),
-			   mw->matrix.cell_highlight_thickness,
-			   LineSolid);
+		mw->manager.highlight_GC,
+		x + mw->matrix.cell_shadow_thickness,
+		y + mw->matrix.cell_shadow_thickness,
+		COLUMN_WIDTH(mw, column) - (2 * mw->matrix.cell_shadow_thickness),
+		mw->matrix.row_heights[row] - (2 * mw->matrix.cell_shadow_thickness),
+		mw->matrix.cell_highlight_thickness,
+		LineSolid);
+#else
+	    DRAW_HIGHLIGHT(XtDisplay(mw), win, 
+		mw->manager.highlight_GC,
+		x + mw->matrix.cell_shadow_thickness,
+		y + mw->matrix.cell_shadow_thickness,
+		COLUMN_WIDTH(mw, column) - (2 * mw->matrix.cell_shadow_thickness),
+		ROW_HEIGHT(mw) - (2 * mw->matrix.cell_shadow_thickness),
+		mw->matrix.cell_highlight_thickness,
+		LineSolid);
+#endif
 	}
     }
     mw->matrix.highlight_location = HighlightNone;
@@ -1775,10 +1675,7 @@ int column;
 
 
 void
-XbaeMatrixUnhighlightCell(w, row, column)
-Widget w;
-int row;
-int column;
+XbaeMatrixUnhighlightCell(Widget w, int row, int column)
 {
     XbaeMatrixWidget mw;
 
@@ -1819,9 +1716,7 @@ int column;
 
 
 void
-XbaeMatrixHighlightRow(w, row)
-Widget w;
-int row;
+XbaeMatrixHighlightRow(Widget w, int row)
 {
     XbaeMatrixWidget mw;
     int j, lc, rc;
@@ -1887,9 +1782,7 @@ int row;
 
 
 void
-XbaeMatrixUnhighlightRow(w, row)
-Widget w;
-int row;
+XbaeMatrixUnhighlightRow(Widget w, int row)
 {
     XbaeMatrixWidget mw;
     int column, lc, rc;
@@ -1946,9 +1839,7 @@ int row;
 
 
 void
-XbaeMatrixHighlightColumn(w, column)
-Widget w;
-int column;
+XbaeMatrixHighlightColumn(Widget w, int column)
 {
     XbaeMatrixWidget mw;
     int row, tr, br;
@@ -2007,9 +1898,7 @@ int column;
 
 
 void
-XbaeMatrixUnhighlightColumn(w, column)
-Widget w;
-int column;
+XbaeMatrixUnhighlightColumn(Widget w, int column)
 {
     XbaeMatrixWidget mw;
     int row, tr, br;
@@ -2064,8 +1953,7 @@ int column;
 #endif
 
 void
-XbaeMatrixDisableRedisplay(w)
-Widget w;
+XbaeMatrixDisableRedisplay(Widget w)
 {
     XbaeMatrixWidget mw;
 
@@ -2082,13 +1970,7 @@ Widget w;
 
 
 void
-#if NeedFunctionPrototypes
 XbaeMatrixEnableRedisplay(Widget w, Boolean redisplay)
-#else
-XbaeMatrixEnableRedisplay(w, redisplay)
-Widget w;
-Boolean redisplay;
-#endif
 {
     XbaeMatrixWidget mw;
 
@@ -2120,10 +2002,7 @@ Boolean redisplay;
  * Public interface for xbaeMakeCellVisible()
  */
 void
-XbaeMatrixMakeCellVisible(w, row, column)
-Widget w;
-int row;
-int column;
+XbaeMatrixMakeCellVisible(Widget w, int row, int column)
 {
     XbaeMatrixWidget mw;
 
@@ -2143,9 +2022,7 @@ int column;
  * Public interface for xbaeIsRowVisible()
  */
 Boolean
-XbaeMatrixIsRowVisible(w, row)
-Widget w;
-int row;
+XbaeMatrixIsRowVisible(Widget w, int row)
 {
     XbaeMatrixWidget mw;
     Boolean r;
@@ -2167,9 +2044,7 @@ int row;
  * Public interface for xbaeIsColumnVisible()
  */
 Boolean
-XbaeMatrixIsColumnVisible(w, col)
-Widget w;
-int col;
+XbaeMatrixIsColumnVisible(Widget w, int col)
 {
     XbaeMatrixWidget mw;
     Boolean r;
@@ -2191,10 +2066,7 @@ int col;
  * Public interface for xbaeIsCellVisible()
  */
 Boolean
-XbaeMatrixIsCellVisible(w, row, col)
-Widget w;
-int row;
-int col;
+XbaeMatrixIsCellVisible(Widget w, int row, int col)
 {
     XbaeMatrixWidget mw;
     Boolean r;
@@ -2219,12 +2091,9 @@ int col;
  *
  */
 void
-XbaeMatrixVisibleCells(w, top_row, bottom_row, left_column, right_column)
-Widget w;
-int *top_row;
-int *bottom_row;
-int *left_column;
-int *right_column;
+XbaeMatrixVisibleCells(Widget w,
+			int *top_row, int *bottom_row,
+			int *left_column, int *right_column)
 {
     XbaeMatrixWidget mw;
 
@@ -2246,9 +2115,7 @@ int *right_column;
  * Get the label of the column passed here.
  */
 String
-XbaeMatrixGetColumnLabel(w, column)
-Widget w;
-int column;
+XbaeMatrixGetColumnLabel(Widget w, int column)
 {
     XbaeMatrixWidget mw;
     String r;
@@ -2276,9 +2143,7 @@ int column;
  * Get the label of the row passed here.
  */
 String
-XbaeMatrixGetRowLabel(w, row)
-Widget w;
-int row;
+XbaeMatrixGetRowLabel(Widget w, int row)
 {
     XbaeMatrixWidget mw;
     String r;
@@ -2303,10 +2168,7 @@ int row;
 }
 
 void
-XbaeMatrixSetColumnLabel(w, column, value)
-Widget w;
-int column;
-String value;
+XbaeMatrixSetColumnLabel(Widget w, int column, String value)
 {
     XbaeMatrixWidget mw;
     ColumnLabelLines lines;
@@ -2381,10 +2243,7 @@ String value;
 }
 
 void
-XbaeMatrixSetRowLabel(w, row, value)
-Widget w;
-int row;
-String value;
+XbaeMatrixSetRowLabel(Widget w, int row, String value)
 {
     XbaeMatrixWidget mw;
 
@@ -2431,11 +2290,85 @@ String value;
 }
 
 Widget
-XbaeCreateMatrix(parent, name, args, ac) 
-Widget parent;
-String name;
-ArgList args;
-Cardinal ac;
+XbaeCreateMatrix(Widget parent, String name, ArgList args, Cardinal ac)
 { 
     return XtCreateWidget(name, xbaeMatrixWidgetClass, parent, args, ac); 
 } 
+
+#ifndef	XtNwrongParameters
+#define	XtNwrongParameters	"wrongParameters"
+#endif
+
+#ifndef	XtCXtToolkitError
+#define	XtCXtToolkitError	"XtToolkitError"
+#endif
+
+void
+XbaeMatrixSetColumnWidth(Widget w, int column, int width)
+{
+#ifdef	HAVE_ROW_HEIGHTS
+	XbaeMatrixWidget	mw = (XbaeMatrixWidget) w;
+
+	if (mw && mw->matrix.column_widths && column >= 0 && column <= mw->matrix.columns) {
+		if (width < 0)
+			mw->matrix.column_widths[column] = 5;	/* The default */
+		else
+			mw->matrix.column_widths[column] = width;
+		XbaeMatrixRefresh(w);
+	} else {
+		XtAppWarningMsg(XtDisplayToApplicationContext(XtDisplay(w)),
+			XtNwrongParameters, "xbaeMatrixNoSuchColumn", XtCXtToolkitError,
+			"XbaeMatrix doesn't have this column",
+			(String *)NULL, (Cardinal *)NULL);
+	}
+#endif
+}
+
+int
+XbaeMatrixGetColumnWidth(Widget w, int column)
+{
+#ifdef	HAVE_ROW_HEIGHTS
+	XbaeMatrixWidget	mw = (XbaeMatrixWidget) w;
+
+	if (mw && mw->matrix.column_widths && column >= 0 && column <= mw->matrix.columns)
+		return mw->matrix.column_widths[column];
+	return 0;
+#else
+        return 0;
+#endif
+}
+
+void
+XbaeMatrixSetRowHeight(Widget w, int row, int height)
+{
+#ifdef	HAVE_ROW_HEIGHTS
+	XbaeMatrixWidget	mw = (XbaeMatrixWidget) w;
+
+	if (mw && mw->matrix.row_heights && row >= 0 && row <= mw->matrix.rows) {
+		if (height < 0)
+			mw->matrix.row_heights[row] = ROW_HEIGHT(mw);
+		else
+			mw->matrix.row_heights[row] = height;
+		XbaeMatrixRefresh(w);
+	} else {
+		XtAppWarningMsg(XtDisplayToApplicationContext(XtDisplay(w)),
+			XtNwrongParameters, "xbaeMatrixNoSuchRow", XtCXtToolkitError,
+			"XbaeMatrix doesn't have this row",
+			(String *)NULL, (Cardinal *)NULL);
+	}
+#endif
+}
+
+int
+XbaeMatrixGetRowHeight(Widget w, int row)
+{
+#ifdef	HAVE_ROW_HEIGHTS
+	XbaeMatrixWidget	mw = (XbaeMatrixWidget) w;
+
+	if (mw && mw->matrix.row_heights && row >= 0 && row <= mw->matrix.rows)
+		return mw->matrix.row_heights[row];
+	return 0;
+#else
+        return 0;
+#endif
+}
