@@ -1,6 +1,8 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright (c) 1999-2000 by the LessTif Developers.
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -22,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Utils.c,v 1.3 2000/01/03 22:33:02 danny Exp $
+ * $Id: Utils.c,v 1.19 2000/06/21 21:27:31 danny Exp $
  */
 
 /*
@@ -39,26 +41,40 @@
 #include <Xbae/Utils.h>
 #include <Xbae/Actions.h>
 #include <Xm/ScrollBar.h>
+
 /*
  * Return the top and bottom-most visible non-fixed row
  */
 void
-xbaeGetVisibleRows(mw, top_row, bottom_row)
-XbaeMatrixWidget mw;
-int *top_row, *bottom_row;
+xbaeGetVisibleRows(XbaeMatrixWidget mw, int *top_row, int *bottom_row)
 {
+#ifdef	HAVE_ROW_HEIGHTS
+    int		i;
+    Dimension	h;
+
+    *top_row = VERT_ORIGIN(mw) + mw->matrix.fixed_rows;
+    h = 0;
+    for (i=VERT_ORIGIN(mw); i<mw->matrix.rows && h < VISIBLE_HEIGHT(mw); i++)
+	h += mw->matrix.row_heights[i];
+    *bottom_row = i;
+
+#ifdef	XBAE_VERBOSE
+    fprintf(stderr, "xbaeGetVisibleRows -> top %d bottom %d\n", *top_row, *bottom_row);
+#endif
+
+    SANITY_CHECK_ROW(mw, *bottom_row);
+#else
     *top_row = VERT_ORIGIN(mw) + mw->matrix.fixed_rows;
     *bottom_row = *top_row + (VISIBLE_HEIGHT(mw) - 1) / ROW_HEIGHT(mw);
     SANITY_CHECK_ROW(mw, *bottom_row);
+#endif
 }
 
 /*
  * Return the left and right-most visible non-fixed column
  */
 void
-xbaeGetVisibleColumns(mw, left_column, right_column)
-XbaeMatrixWidget mw;
-int *left_column, *right_column;
+xbaeGetVisibleColumns(XbaeMatrixWidget mw, int *left_column, int *right_column)
 {
     *left_column = xbaeXtoCol(mw, FIXED_COLUMN_WIDTH(mw) + HORIZ_ORIGIN(mw));
     *right_column = xbaeXtoCol(mw, FIXED_COLUMN_WIDTH(mw) + HORIZ_ORIGIN(mw) +
@@ -70,9 +86,9 @@ int *left_column, *right_column;
  * the visible non-fixed cells
  */
 void
-xbaeGetVisibleCells(mw, top_row, bottom_row, left_column, right_column)
-XbaeMatrixWidget mw;
-int *top_row, *bottom_row, *left_column, *right_column;
+xbaeGetVisibleCells(XbaeMatrixWidget mw,
+	int *top_row, int *bottom_row,
+	int *left_column, int *right_column)
 {
     xbaeGetVisibleRows(mw, top_row, bottom_row);
     xbaeGetVisibleColumns(mw, left_column, right_column);
@@ -85,8 +101,7 @@ int *top_row, *bottom_row, *left_column, *right_column;
  * If we can't make leftColumn the left column, make it as close as possible.
  */
 void
-xbaeAdjustLeftColumn(mw)
-XbaeMatrixWidget mw;
+xbaeAdjustLeftColumn(XbaeMatrixWidget mw)
 {
     int y;
     int i;
@@ -138,10 +153,13 @@ XbaeMatrixWidget mw;
  * If we can't make topRow the top row, make it as close as possible.
  */
 void
-xbaeAdjustTopRow(mw)
-XbaeMatrixWidget mw;
+xbaeAdjustTopRow(XbaeMatrixWidget mw)
 {
+#ifdef	HAVE_ROW_HEIGHTS
+    int rows_visible = VISIBLE_HEIGHT(mw) / SOME_ROW_HEIGHT(mw, mw->matrix.top_row);
+#else
     int rows_visible = VISIBLE_HEIGHT(mw) / ROW_HEIGHT(mw);
+#endif
 
     /*
      * If we have less than one full row visible, then count it as a full row
@@ -169,9 +187,7 @@ XbaeMatrixWidget mw;
  * Does not check if the cell is actually visible before clearing it.
  */
 void
-xbaeClearCell(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeClearCell(XbaeMatrixWidget mw, int row, int column)
 {
     int x, y;
     Boolean fixed = IS_FIXED(mw, row, column);
@@ -192,17 +208,16 @@ int row, column;
 	 (y < (int)ROW_LABEL_OFFSET(mw))))
 	return;
 #endif
+
     XClearArea(XtDisplay(mw), win, x, y, COLUMN_WIDTH(mw, column),
-	       ROW_HEIGHT(mw), fixed);
+	       SOME_ROW_HEIGHT(mw, row), fixed);
 }
 
 /*
  * Return True if a row is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsRowVisible(mw, row)
-XbaeMatrixWidget mw;
-int row;
+xbaeIsRowVisible(XbaeMatrixWidget mw, int row)
 {
     /*
      * If we are not in a fixed row or trailing fixed row,
@@ -216,13 +231,13 @@ int row;
 	if (row >= VERT_ORIGIN(mw))
 	{
 	    double height = ((double)ClipChild(mw)->core.height /
-			     ROW_HEIGHT(mw)) + VERT_ORIGIN(mw);
+			     SOME_ROW_HEIGHT(mw, row)) + VERT_ORIGIN(mw);
 	    if (row < height)
 		return True;
 	    
 	    if ((int)ClipChild(mw)->core.height >
 		(int)TEXT_HEIGHT_OFFSET(mw) &&
-		(int)ClipChild(mw)->core.height < ROW_HEIGHT(mw) &&
+		(int)ClipChild(mw)->core.height < SOME_ROW_HEIGHT(mw, row) &&
 		row == VERT_ORIGIN(mw))
 
 		return True;
@@ -238,9 +253,7 @@ int row;
  * Return True if a column is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsColumnVisible(mw, column)
-XbaeMatrixWidget mw;
-int column;
+xbaeIsColumnVisible(XbaeMatrixWidget mw, int column)
 {
     /*
      * If we are not in a fixed column, see if we are on the screen
@@ -273,9 +286,7 @@ int column;
  * Return True if a cell is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsCellVisible(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeIsCellVisible(XbaeMatrixWidget mw, int row, int column)
 {
     return xbaeIsRowVisible(mw, row) && xbaeIsColumnVisible(mw, column);
 }
@@ -284,11 +295,9 @@ int row, column;
  * Scroll a row so it is visible on the screen.
  */
 void
-xbaeMakeRowVisible(mw, row)
-XbaeMatrixWidget mw;
-int row;
+xbaeMakeRowVisible(XbaeMatrixWidget mw, int row)
 {
-    int rows_visible;
+    int rows_visible, rel_row;
     int value, slider_size, increment, page_increment, vert_value;
 
     /*
@@ -302,8 +311,8 @@ int row;
      * Calculate the number of rows visible. If less than one full
      * row is visible, use one full row.
      */
-    row -= mw->matrix.fixed_rows;
-    rows_visible = VISIBLE_HEIGHT(mw) / ROW_HEIGHT(mw);
+    rel_row = row - mw->matrix.fixed_rows;
+    rows_visible = VISIBLE_HEIGHT(mw) / SOME_ROW_HEIGHT(mw, row);
     if (rows_visible == 0)
 	rows_visible = 1;
 
@@ -311,10 +320,10 @@ int row;
      * Figure out the new value of the VSB to scroll this cell
      * onto the screen (the VSB uses row coordinates instead of pixels)
      */
-    if (row < VERT_ORIGIN(mw))
-	vert_value = row;
-    else if (row >= rows_visible + VERT_ORIGIN(mw))
-	vert_value = row - rows_visible + 1;
+    if (rel_row < VERT_ORIGIN(mw))
+	vert_value = rel_row;
+    else if (rel_row >= rows_visible + VERT_ORIGIN(mw))
+	vert_value = rel_row - rows_visible + 1;
     else
 	vert_value = VERT_ORIGIN(mw);
 
@@ -335,9 +344,7 @@ int row;
  * Scroll a column so it is visible on the screen.
  */
 void
-xbaeMakeColumnVisible(mw, column)
-XbaeMatrixWidget mw;
-int column;
+xbaeMakeColumnVisible(XbaeMatrixWidget mw, int column)
 {
     int value, slider_size, increment, page_increment, x, horiz_value;
 
@@ -363,13 +370,13 @@ int column;
     else if (x + COLUMN_WIDTH(mw, column) >
 	     VISIBLE_WIDTH(mw) + HORIZ_ORIGIN(mw))
     {
-	int off = (x + COLUMN_WIDTH(mw, column)) - (VISIBLE_WIDTH(mw) +
+	int offs = (x + COLUMN_WIDTH(mw, column)) - (VISIBLE_WIDTH(mw) +
 						    HORIZ_ORIGIN(mw));
 
-	if (x - off < HORIZ_ORIGIN(mw))
+	if (x - offs < HORIZ_ORIGIN(mw))
 	    horiz_value = x;
 	else
-	    horiz_value = HORIZ_ORIGIN(mw) + off;
+	    horiz_value = HORIZ_ORIGIN(mw) + offs;
     }
     else
 	horiz_value = HORIZ_ORIGIN(mw);
@@ -391,9 +398,7 @@ int column;
  * Scrolls a fixed or non-fixed cell so it is visible on the screen.
  */
 void
-xbaeMakeCellVisible(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeMakeCellVisible(XbaeMatrixWidget mw, int row, int column)
 {
     if (!xbaeIsRowVisible(mw, row))
 	xbaeMakeRowVisible(mw, row);
@@ -406,9 +411,7 @@ int row, column;
  * drawing non-fixed column labels and fixed rows.
  */
 void
-xbaeSetClipMask(mw, clip_reason)
-XbaeMatrixWidget mw;
-unsigned int clip_reason;
+xbaeSetClipMask(XbaeMatrixWidget mw, unsigned int clip_reason)
 {
     XRectangle r[2];
     int n = 1;
@@ -488,13 +491,11 @@ unsigned int clip_reason;
 		       0, 0, r, n, Unsorted);
 }
 
-
 /*
  * Get the total pixel width of the non-fixed cell area
  */
 void
-xbaeGetCellTotalWidth(mw)
-XbaeMatrixWidget mw;
+xbaeGetCellTotalWidth(XbaeMatrixWidget mw)
 {
     int i, columns;
 
@@ -512,8 +513,7 @@ XbaeMatrixWidget mw;
  * Cache the pixel position of each column
  */
 void
-xbaeGetColumnPositions(mw)
-XbaeMatrixWidget mw;
+xbaeGetColumnPositions(XbaeMatrixWidget mw)
 {
     int i, x;
 
@@ -524,20 +524,24 @@ XbaeMatrixWidget mw;
 }
 
 void
-#if NeedFunctionPrototypes
+xbaeGetRowPositions(XbaeMatrixWidget mw)
+{
+    int i, y;
+
+    for (i = 0, y = 0;
+	 i < mw->matrix.rows;
+	 y += SOME_ROW_HEIGHT(mw, i), i++)
+	ROW_POSITION(mw, i) = y;
+}
+
+void
 xbaeComputeSize(XbaeMatrixWidget mw, Boolean compute_width,
 		Boolean compute_height)
-#else
-xbaeComputeSize(mw, compute_width, compute_height)
-XbaeMatrixWidget mw;
-Boolean compute_width;
-Boolean compute_height;
-#endif
 {
     unsigned long full_width = NON_FIXED_TOTAL_WIDTH(mw) +
 	FIXED_COLUMN_WIDTH(mw) + TRAILING_FIXED_COLUMN_WIDTH(mw) +
 	ROW_LABEL_WIDTH(mw) + 2 * mw->manager.shadow_thickness;
-    unsigned long full_height = CELL_TOTAL_HEIGHT(mw) +
+    unsigned long full_height = xbaeCellTotalHeight(mw) +
 	FIXED_ROW_HEIGHT(mw) + TRAILING_FIXED_ROW_HEIGHT(mw) +
 	COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness;
     unsigned long width, height;
@@ -566,14 +570,27 @@ Boolean compute_height;
      * Otherwise, if the compute_height flag is set, then we are full height.
      * Otherwise we keep whatever height we are.
      */
-    if (mw->matrix.visible_rows)
+    if (mw->matrix.visible_rows) {
+#ifdef HAVE_ROW_HEIGHTS
+	int	i;
+	height = TRAILING_FIXED_ROW_HEIGHT(mw) + FIXED_ROW_HEIGHT(mw) +
+	    COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness;
+
+	for (i=0; i<mw->matrix.visible_rows; i++)
+		if (i < mw->matrix.rows)
+			height += SOME_ROW_HEIGHT(mw, i);
+		else
+			height += ROW_HEIGHT(mw);
+#else
 	height = mw->matrix.visible_rows * ROW_HEIGHT(mw) +
 	    TRAILING_FIXED_ROW_HEIGHT(mw) + FIXED_ROW_HEIGHT(mw) +
 	    COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness;
-    else if (compute_height)
+#endif
+    } else if (compute_height) {
 	height = full_height;
-    else
+    } else {
 	height = mw->core.height;
+    }
 
     /*
      * Store our calculated size.
@@ -612,14 +629,18 @@ Boolean compute_height;
      */
     mw->matrix.desired_width = mw->core.width;
     mw->matrix.desired_height = mw->core.height;
+
+#ifdef	XBAE_VERBOSE
+    fprintf(stderr, "xbaeComputeSize -> w %d h %d\n",
+	mw->matrix.desired_width, mw->matrix.desired_height);
+#endif
 }
 
 /*
  * Return the length of the longest row label
  */
 short
-xbaeMaxRowLabel(mw)
-XbaeMatrixWidget mw;
+xbaeMaxRowLabel(XbaeMatrixWidget mw)
 {
     int i;
     short max = 0, len;
@@ -637,9 +658,7 @@ XbaeMatrixWidget mw;
 }
 
 void
-xbaeParseColumnLabel(label, lines)
-String label;
-ColumnLabelLines lines;
+xbaeParseColumnLabel(String label, ColumnLabelLines lines)
 {
     char *nl;
 
@@ -685,6 +704,8 @@ ColumnLabelLines lines;
     }
 }
 
+/* FIX ME : this function contains some as yet unfixed ROW_HEIGHT behaviour.
+   Danny 23/5/2000 */
 /*
  * Convert an x/y pixel position to the row/column cell position it picks.
  * 'cell' specifies whether the x/y coord is relative to the fixed cells
@@ -697,11 +718,7 @@ ColumnLabelLines lines;
  * may be useful nevertheless.
  */
 Boolean
-xbaeXYToRowCol(mw, x, y, row, column, cell)
-XbaeMatrixWidget mw;
-int *x, *y;
-int *row, *column;
-CellType cell;
+xbaeXYToRowCol(XbaeMatrixWidget mw, int *x, int *y, int *row, int *column, CellType cell)
 {
     Rectangle rect;
     unsigned int inBox = CLIP_NONE;
@@ -995,7 +1012,7 @@ CellType cell;
 		*y += row_height;
 	    }
 	    else
-		*row = YtoRow(mw, *y) +
+		*row = xbaeYtoRow(mw, *y) +
 		    ((((CLIP_FIXED_COLUMNS & inBox) ||
 		       (CLIP_TRAILING_FIXED_COLUMNS & inBox))) &&
 		     !((CLIP_FIXED_ROWS & inBox) ||
@@ -1102,10 +1119,10 @@ CellType cell;
 	if (CLIP_ROW_LABELS & inBox)
 	{
 	    *y -= row_label_offset;
-	    *row = YtoRow(mw, *y);
+	    *row = xbaeYtoRow(mw, *y);
 	    *row += (CLIP_FIXED_ROWS & inBox ? 0 : mw->matrix.top_row);
 	    *row += ((CLIP_TRAILING_FIXED_ROWS & inBox) ?
-		     (CELL_TOTAL_HEIGHT(mw) - VISIBLE_HEIGHT(mw)) /
+		     (xbaeCellTotalHeight(mw) - VISIBLE_HEIGHT(mw)) /
 		     row_height - mw->matrix.top_row: 0);
 	    	    
 	    *column = -1;
@@ -1205,7 +1222,7 @@ CellType cell;
 	/*
 	 * Convert the new point to a row/column position
 	 */
-	*row = YtoRow(mw, *y + mw->matrix.first_row_offset) + VERT_ORIGIN(mw);
+	*row = xbaeYtoRow(mw, *y + mw->matrix.first_row_offset) + VERT_ORIGIN(mw);
 	*column = xbaeXtoCol(mw, *x + HORIZ_ORIGIN(mw));
 
 	/*
@@ -1336,6 +1353,39 @@ int x;
     return i - 1;
 }
 
+int
+xbaeYtoRow(XbaeMatrixWidget mw, int y)
+{
+#ifdef	HAVE_ROW_HEIGHTS
+	int	i, pos;
+
+	for (i=0, pos=0; i<mw->matrix.rows; i++) {
+		pos += mw->matrix.row_heights[i];
+		if (y <= pos)
+			return i;
+	}
+	return mw->matrix.rows;
+#else
+	return y / ROW_HEIGHT(mw);
+#endif
+
+#if 0
+    int i;
+
+    for (i = 0; i < mw->matrix.rows; i++)
+	if (ROW_POSITION(mw, i) > y)
+	    return i - 1;
+    /*
+     * I have seen cases where this function returned mw->matrix.columns
+     * causing a crash as array bounds are read - AL
+     */
+    if (i > mw->matrix.rows)
+	return mw->matrix.rows - 1;
+
+    return i - 1;
+#endif
+}
+
 /*
  * Convert a pixel position to the trailing
  * fixed column it is contained in
@@ -1363,13 +1413,90 @@ int x;
  * fixed cells, or the clip window for non-fixed).
  */
 void
-xbaeRowColToXY(mw, row, column, x, y)
-XbaeMatrixWidget mw;
-int row;
-int column;
-int *x;
-int *y;
+xbaeRowColToXY(XbaeMatrixWidget mw, int row, int column, int *x, int *y)
 {
+#ifdef	HAVE_ROW_HEIGHTS
+    int		i;
+
+    /*
+     * If we are in a fixed cell, calculate x/y relative to Matrixs
+     * window (take into account labels etc)
+     */
+    if (IS_FIXED(mw, row, column))
+    {
+	/*
+	 * Ignore horiz_origin if we are in a fixed column
+	 */
+	if (IS_LEADING_FIXED_COLUMN(mw, column))
+	{
+	    if (IS_FIXED_ROW(mw, row))
+		*x = COLUMN_LABEL_OFFSET(mw) + COLUMN_POSITION(mw, column);
+	    else
+		*x = COLUMN_POSITION(mw, column);	/* LeftClip */
+	}
+	else if (IS_TRAILING_FIXED_COLUMN(mw, column))
+	{
+	    int m;
+	    if (IS_FIXED_ROW(mw, row))
+		*x = TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw);
+	    else
+		*x = 0;					/* RightClip */
+	    for (m = TRAILING_HORIZ_ORIGIN(mw); m < column; m++)
+		*x += COLUMN_WIDTH(mw, m);
+	}
+	else if (IS_FIXED_ROW(mw, row))
+	    *x = (COLUMN_POSITION(mw, column) -
+		  COLUMN_POSITION(mw, mw->matrix.fixed_columns)) -
+		HORIZ_ORIGIN(mw);
+	else
+	    *x = COLUMN_LABEL_OFFSET(mw) +
+		COLUMN_POSITION(mw, column) - HORIZ_ORIGIN(mw);
+
+	/*
+	 * Ignore vert_origin if we are in a fixed row
+	 */
+	if (IS_LEADING_FIXED_ROW(mw, row)) {
+	    if (IS_FIXED_COLUMN(mw, column)) {
+		*y = ROW_LABEL_OFFSET(mw);
+	    } else {
+		*y = 0;		/* TopClip */
+	    }
+
+	    for (i=0; i<row; i++)
+		*y += mw->matrix.row_heights[i];
+	} else if (IS_TRAILING_FIXED_ROW(mw, row)) {
+	    int m;
+	    if (IS_FIXED_COLUMN(mw, column))
+		*y = TRAILING_FIXED_ROW_LABEL_OFFSET(mw);
+	    else
+		*y = 0;				/* BottomClip */
+	    for (m = TRAILING_VERT_ORIGIN(mw); m < row; m++)
+		*y += ROW_HEIGHT(mw);
+	} else if (IS_FIXED_COLUMN(mw, column)) {
+	    *y = 0;
+	    for (i=mw->matrix.fixed_rows + VERT_ORIGIN(mw); i<row; i++)
+		*y += mw->matrix.row_heights[i];
+	} else {
+	    *y = ROW_LABEL_OFFSET(mw);
+	    for (i=VERT_ORIGIN(mw); i<row; i++)
+		*y += mw->matrix.row_heights[i];
+	}
+    }
+
+    /*
+     * If we are not fixed we must account for fixed rows/columns
+     * and scrolling origins.
+     */
+    else {
+	*x = (COLUMN_POSITION(mw, column) -
+	      COLUMN_POSITION(mw, mw->matrix.fixed_columns)) -
+	    HORIZ_ORIGIN(mw);
+	*y = 0;
+	for (i=mw->matrix.fixed_rows + VERT_ORIGIN(mw); i<row; i++)
+	    *y += mw->matrix.row_heights[i];
+	*y -= mw->matrix.first_row_offset;
+    }
+#else
     /*
      * If we are in a fixed cell, calculate x/y relative to Matrixs
      * window (take into account labels etc)
@@ -1446,6 +1573,7 @@ int *y;
 			       VERT_ORIGIN(mw));
 	*y -= mw->matrix.first_row_offset;
     }
+#endif
 }
 
 #define FIXED_NONE	0
@@ -1465,10 +1593,7 @@ int *y;
  * or one of the extra clips which handle the fixed row/col cells.
  */
 Window
-xbaeGetCellWindow(mw, w, row, column)
-XbaeMatrixWidget mw;
-Widget *w;
-int row, column;
+xbaeGetCellWindow(XbaeMatrixWidget mw, Widget *w, int row, int column)
 {
     int posn;
     Window win;
@@ -1534,17 +1659,8 @@ int row, column;
 }
 
 void
-xbaeCalcVertFill(mw, win, x, y, row, column, ax, ay, width, height)
-XbaeMatrixWidget mw;
-Window win;
-int x;
-int y;
-int row;
-int column;
-int *ax;
-int *ay;
-int *width;
-int *height;
+xbaeCalcVertFill(XbaeMatrixWidget mw, Window win, int x, int y, int row, int column,
+	int *ax, int *ay, int *width, int *height)
 {
     *ax = x;
     *width = COLUMN_WIDTH(mw, column);
@@ -1624,20 +1740,11 @@ int *height;
 }
     
 void
-xbaeCalcHorizFill(mw, win, x, y, row, column, ax, ay, width, height)
-XbaeMatrixWidget mw;
-Window win;
-int x;
-int y;
-int row;
-int column;
-int *ax;
-int *ay;
-int *width;
-int *height;
+xbaeCalcHorizFill(XbaeMatrixWidget mw, Window win, int x, int y, int row, int column,
+	int *ax, int *ay, int *width, int *height)
 {
     *ay = y;
-    *height = ROW_HEIGHT(mw);
+    *height = SOME_ROW_HEIGHT(mw, row);
 
     if (win == XtWindow(TopClip(mw)))
     {
@@ -1733,3 +1840,34 @@ xbaeObjectUnlock(Widget w)
                 XtAppUnlock(XtWidgetToApplicationContext(w));
 #endif
 }
+
+/* The following 2 functions are here to provide information about the
+   version of this library - since we may often build it as a shared/dynamic
+   library it's important to have this builtin */
+const char *
+XbaeGetVersionTxt(void)
+{
+	return XbaeVersionTxt;
+}
+
+int
+XbaeGetVersionNum(void)
+{
+	return XbaeVersion;
+}
+
+/* The following 2 functions contain/return info about the
+   M*tif/LessTif version this library was built with.
+   Building against one toolkit and linking on runtime against
+   another is likely to fail! */
+const char *
+XbaeGetXmVersionTxt(void)
+{
+        return XmVERSION_STRING;
+}
+
+int
+XbaeGetXmVersionNum(void)
+{
+        return XmVersion;
+}
