@@ -1,5 +1,5 @@
 /*
-   $Id: Stroke.c,v 1.1 1998/06/03 21:55:08 rwscott Exp $
+   $Id: Stroke.c,v 1.4 2000/01/25 11:54:25 amai Exp $
 
    libStroke is a library to enable any X application to use Strokes.
    Copyright (C) 1997  Richard Scott <rwscott@magi.com>
@@ -19,7 +19,7 @@
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-
+#include "../config.h"
 #include <stdio.h>
 
 #include <X11/IntrinsicP.h>
@@ -35,7 +35,7 @@ typedef struct
 AppResources_t, *AppResourcesPtr;
 
 static char DefaultHelp[] = "\n\
-Strokes are neat, strokes are cool, try man strokes.\n\
+Strokes are neat, strokes are cool, try man Strokes.\n\
 If it does not turn up anything you are out of luck.\n\
 ";
 
@@ -574,7 +574,7 @@ InputAction(Widget W, XEvent *event)
 
 /* ***** */
 
-void 
+static void 
 _StrokeInitialize(Widget W)
 {
 	StrokeStatePtr NewState;
