@@ -1,6 +1,6 @@
 /**
  *
- * $Id: IconG.c,v 1.2 1997/10/30 03:09:21 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/IconG.c,v 1.7 2000/02/15 23:32:52 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: IconG.c,v 1.2 1997/10/30 03:09:21 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/IconG.c,v 1.7 2000/02/15 23:32:52 danny Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -41,14 +41,20 @@ static const char rcsid[] = "$Id: IconG.c,v 1.2 1997/10/30 03:09:21 rwscott Exp
 #include <X11/ShellP.h>
 
 #include <XmI/DebugUtil.h>
+#include <XmI/PixConvI.h>
 
 #ifndef XmUNSPECIFIED
 #define XmUNSPECIFIED (~0)
 #endif
 
+#ifndef IG_Font
+#define IG_Font(w) \
+       (((XmIconGadget)(w))->icong.cache->render_table)
+#endif
+
 /* Forward Declarations */
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -94,6 +100,8 @@ static void get_values_posthook(Widget new_w, ArgList args, Cardinal *num_args);
 static Cardinal get_sec_res_data(WidgetClass wc,
 				 XmSecondaryResourceData **data);
 
+static void export_label_string(Widget w, int offset, XtArgVal *value);
+
 
 #if 0
 static void Arm(Widget w, XEvent *event,
@@ -134,39 +142,60 @@ int _XmIconCacheCompare(XtPointer A, XtPointer B);
 static XtResource cache_resources[] =
 {
     {
-	XmNmultiClick, XmCMultiClick, XmRMultiClick,
-	sizeof(unsigned char), Offset(multiClick),
-	XmRImmediate, (XtPointer)((unsigned char)XmUNSPECIFIED)
+	XmNfontList, XmCFontList, XmRFontList,
+	sizeof(XmFontList), Offset(render_table),
+	XmRImmediate, (XtPointer)NULL
     },
     {
-	XmNdefaultButtonShadowThickness, XmCDefaultButtonShadowThickness, XmRHorizontalDimension,
-	sizeof(Dimension), Offset(default_button_shadow_thickness),
-	XtRImmediate, (XtPointer)0
+	XmNrenderTable, XmCRenderTable, XmRRenderTable,
+	sizeof(XmRenderTable), Offset(render_table),
+	XmRImmediate, (XtPointer)NULL
     },
     {
-	XmNfillOnArm, XmCFillOnArm, XmRBoolean,
-	sizeof(Boolean), Offset(fill_on_arm),
-	XmRImmediate, (XtPointer)True
+        XmNbackground, XmCBackground, XmRPixel,
+        sizeof(Pixel), Offset(background),
+        XmRCallProc, (XtPointer)_XmBackgroundColorDefault
     },
     {
-	XmNarmColor, XmCArmColor, XmRPixel,
-	sizeof(Pixel), Offset(arm_color),
-	XmRCallProc, (XtPointer)_XmSelectColorDefault
+        XmNforeground, XmCForeground, XmRPixel,
+        sizeof(Pixel), Offset(foreground),
+        XmRCallProc, (XtPointer)_XmForegroundColorDefault
+    },
+    {   
+	XmNtopShadowColor, XmCTopShadowColor, XmRPixel,
+	sizeof(Pixel), Offset(top_shadow_color),
+	XmRCallProc, (XtPointer)_XmTopShadowColorDefault
     },
     {
-	XmNarmPixmap, XmCArmPixmap, XmRGadgetPixmap,
-	sizeof(Pixmap), Offset(arm_pixmap),
-	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
-    }
-};
-
-static XmSyntheticResource cache_syn_resources[] =
-{
+	XmNbottomShadowColor, XmCBottomShadowColor, XmRPixel,
+	sizeof(Pixel), Offset(bottom_shadow_color), 
+	XmRCallProc, (XtPointer)_XmBottomShadowColorDefault
+    },
     {
-	XmNdefaultButtonShadowThickness,
-	sizeof(Dimension), Offset(default_button_shadow_thickness),
-	_XmFromHorizontalPixels, _XmToHorizontalPixels
-    }
+	XmNhighlightColor, XmCHighlightColor, XmRPixel,
+	sizeof(Pixel), Offset(highlight_color),
+	XmRCallProc, (XtPointer)_XmHighlightColorDefault
+    },
+    {
+        XmNbackgroundPixmap, XmCPixmap, XmRXmBackgroundPixmap,
+        sizeof(Pixmap), Offset(background_pixmap),
+        XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
+    },
+    {
+	XmNtopShadowPixmap, XmCTopShadowPixmap, XmRTopShadowPixmap,
+	sizeof(Pixmap), Offset(top_shadow_pixmap),
+	XmRCallProc, (XtPointer)_XmTopShadowPixmapDefault
+    },
+    {
+        XmNbottomShadowPixmap, XmCBottomShadowPixmap, XmRBottomShadowPixmap,
+        sizeof(Pixmap), Offset(bottom_shadow_pixmap),
+        XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
+    },
+    {
+        XmNhighlightPixmap, XmCHighlightPixmap, XmRHighlightPixmap,
+        sizeof(Pixmap), Offset(highlight_pixmap),
+        XmRCallProc, (XtPointer)_XmHighlightPixmapDefault
+    },
 };
 
 /* *INDENT-OFF* */
@@ -208,8 +237,8 @@ XmIconGCacheObjClassRec xmIconGCacheObjClassRec = {
     },
     /* XmExtObject part */
     {
-        /* syn_resources      */ cache_syn_resources,
-        /* num_syn_resources  */ XtNumber(cache_syn_resources),
+        /* syn_resources      */ NULL,
+        /* num_syn_resources  */ 0,
         /* extension          */ NULL
     },
     /* IconGCacheObj part */
@@ -219,87 +248,67 @@ XmIconGCacheObjClassRec xmIconGCacheObjClassRec = {
 };
 
 #undef Offset
-#define Offset(field) XtOffsetOf(XmIconGadgetRec, icon.field)
+#define Offset(field) XtOffsetOf(XmIconGadgetRec, icong.field)
 
 /* Resources for the icon class */
 static XtResource resources[] = {
-    {
-	XmNdetail, XmCDetail, XmRXmStringTable,
-	sizeof(XmStringTable), Offset(detail),
-	XmRStringTable, (XtPointer)NULL
-    },
-    {
-	XmNdetailCount, XmCDetailCount, XmRInt,
-	sizeof(int), Offset(detailCount),
-	XmRImmediate, (XtPointer)0
-    },
-    {
-	XmNfontList, XmCFontList, XmRFontList,
-	sizeof(XmFontList), Offset(font),
-	XmRImmediate, (XtPointer)NULL
-    },
     {
 	XmNlabelString, XmCXmString, XmRXmString,
-	sizeof(XmString), Offset(labelString),
+	sizeof(XmString), Offset(label_string),
 	XmRImmediate, (XtPointer)NULL
     },
     {
 	XmNlargeIconMask, XmCIconMask, XmRGadgetPixmap,
-	sizeof(Pixmap), Offset(largeIconMask),
+	sizeof(Pixmap), Offset(large_icon_mask),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
     },
     {
 	XmNlargeIconPixmap, XmCIconPixmap, XmRGadgetPixmap,
-	sizeof(Pixmap), Offset(largeIconPixmap),
+	sizeof(Pixmap), Offset(large_icon_pixmap),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
     },
-    {
-	XmNrenderTable, XmCRenderTable, XmRRenderTable,
-	sizeof(XmRenderTable), Offset(renderTable),
-	XmRImmediate, (XtPointer)NULL
-    },
     {
 	XmNsmallIconMask, XmCIconMask, XmRGadgetPixmap,
-	sizeof(Pixmap), Offset(smallIconMask),
+	sizeof(Pixmap), Offset(small_icon_mask),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
     },
     {
 	XmNsmallIconPixmap, XmCIconPixmap, XmRGadgetPixmap,
-	sizeof(Pixmap), Offset(smallIconPixmap),
+	sizeof(Pixmap), Offset(small_icon_pixmap),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
     },
     {
 	XmNviewType, XmCViewType, XmRViewType,
-	sizeof(unsigned char), Offset(viewType),
+	sizeof(unsigned char), Offset(view_type),
 	XmRImmediate, (XtPointer)XmLARGE_ICON
     },
     {
 	XmNvisualEmphasis, XmCVisualEmphasis, XmRVisualEmphasis,
-	sizeof(unsigned char), Offset(visualEmphasis),
+	sizeof(unsigned char), Offset(visual_emphasis),
 	XmRImmediate, (XtPointer)XmNOT_SELECTED
     },
     {
-	XmNshadowThickness, XmCShadowThickness, XmRHorizontalDimension,
-	sizeof(Dimension), XtOffsetOf(XmIconGadgetRec, gadget.shadow_thickness),
-	XmRImmediate, (XtPointer)2
+	XmNdetail, XmCDetail, XmRStringTable,
+	sizeof(XmStringTable), Offset(detail),
+	XmRImmediate, (XtPointer)NULL
     },
     {
-	XmNtraversalOn, XmCTraversalOn, XmRBoolean,
-	sizeof(Boolean), XtOffsetOf(XmIconGadgetRec, gadget.traversal_on),
-	XmRImmediate, (XtPointer)True
+	XmNdetailCount, XmCDetailCount, XmRCardinal,
+	sizeof(Cardinal), Offset(detail_count),
+	XmRImmediate, (XtPointer)0
+    },
+    {   /* This is interesting */
+	".cache", ".Cache", XmRPointer,
+	sizeof(XtPointer), Offset(cache),
+	XmRImmediate, (XtPointer)NULL
     },
-    {
-	XmNhighlightThickness, XmCHighlightThickness, XmRHorizontalDimension,
-	sizeof(Dimension), XtOffsetOf(XmIconGadgetRec,gadget.highlight_thickness),
-	XmRImmediate, (XtPointer)2
-    }
 };
 
 static XmSyntheticResource syn_resources[] = {
     {
 	XmNlabelString,
-	sizeof(XmString), Offset(labelString),
-	_XmExportLabelString, NULL
+	sizeof(XmString), Offset(label_string),
+	export_label_string, NULL
     }
 };
 
@@ -407,14 +416,6 @@ XmIconGadgetClassRec xmIconGadgetClassRec = {
 
 WidgetClass xmIconGadgetClass = (WidgetClass)&xmIconGadgetClassRec;
 
-/* 
- * Some #defines to make the code below more readable
- */
-#if 0
-#define IN_MENU(w) (LabG_MenuType(w) == XmMENU_POPUP || \
-                    LabG_MenuType(w) == XmMENU_PULLDOWN)
-#endif
-
 /******************************* GADGET PART *********************************/
 static void
 secondary_object_create(Widget request, Widget new_w,
@@ -479,7 +480,7 @@ _XmIconCacheCompare(XtPointer A, XtPointer B)
 
 /******************************* GADGET PART *********************************/
 static void
-class_initialize()
+class_initialize(void)
 {
 #if 0
     XtResourceList combined, labels;
@@ -610,13 +611,13 @@ Dimension TextHeight, TextWidth;
 unsigned int IconHeight, IconWidth;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmCalcIconGDimensions\n"));
-    if (_XmStringIsXmString((XmString)IG_Label(w)))
+    if (_XmStringIsXmString((XmString)IG_LabelString(w)))
     {
-	IG_Label(w) = _XmStringCreate((XmString)IG_Label(w));
+	IG_LabelString(w) = (XmString)_XmStringCreate((XmString)IG_LabelString(w));
     }
 
     _XmStringExtent(IG_Font(w),
-		    IG_Label(w),
+		    (_XmString) IG_LabelString(w),
 		    &TextWidth,
 		    &TextHeight);
     DEBUGOUT(XdbDebug(__FILE__, w, "\t Text width %d height %d\n",
@@ -677,8 +678,8 @@ initialize(Widget request, Widget new_w,
 
     /* If the label was not initialized with the resource labelString set,
        use its name -- the follow _XmString code comes from MegaButton */
-    if (IG_Label(new_w) == (_XmString)XmUNSPECIFIED ||
-	IG_Label(new_w) == (_XmString)0)
+    if (IG_LabelString(new_w) == (XmString)XmUNSPECIFIED ||
+	IG_LabelString(new_w) == (XmString)0)
     {				/* Shouldn't be necessary but is */
 	XmString xmstring;
 
@@ -687,12 +688,12 @@ initialize(Widget request, Widget new_w,
 					   XmNlabelString,
 					   XtName(new_w));
 
-	IG_Label(new_w) = _XmStringCreate(xmstring);
+	IG_LabelString(new_w) = (XmString) _XmStringCreate(xmstring);
     }
 
-    if (_XmStringIsXmString((XmString)IG_Label(new_w)))
+    if (_XmStringIsXmString((XmString)IG_LabelString(new_w)))
     {
-	IG_Label(new_w) = _XmStringCreate((XmString)IG_Label(new_w));
+	IG_LabelString(new_w) = (XmString) _XmStringCreate((XmString)IG_LabelString(new_w));
     }
 
     if (IG_LargeIconPixmap(new_w) != XmUNSPECIFIED_PIXMAP &&
@@ -1282,7 +1283,7 @@ expose(Widget w, XEvent *event, Region region)
     }
 
     _XmStringExtent(IG_Font(w),
-		    IG_Label(w),
+		    (_XmString) IG_LabelString(w),
 		    &TextWidth,
 		    &TextHeight);
 
@@ -1310,7 +1311,7 @@ expose(Widget w, XEvent *event, Region region)
     _XmStringDraw(XtDisplayOfObject(w),
 		  XtWindowOfObject(w),
 		  IG_Font(w),
-		  IG_Label(w),
+		  (_XmString) IG_LabelString(w),
 		  myGC,
 		  TextX,
 		  TextY,
@@ -1716,16 +1717,13 @@ ButtonUp(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	return;
 #endif
 
-    _XmDrawShadows(XtDisplay(w),
+    _XmClearBorder(XtDisplay(w),
 		   XtWindow(w),
-		   XmParentTopShadowGC(w),
-		   XmParentBottomShadowGC(w),
 		   LabG_Highlight(w),
 		   LabG_Highlight(w),
 		   XtWidth(w) - 2 * LabG_Highlight(w),
 		   XtHeight(w) - 2 * LabG_Highlight(w),
-		   LabG_Shadow(w),
-		   XmNO_LINE);
+		   LabG_Shadow(w));
 
     if (!LabG_SkipCallback(w) && IG_ActivateCallback(w))
     {
@@ -1959,6 +1957,16 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
 #endif
 }
 
+static void
+export_label_string(Widget w, int offset, XtArgVal *value)
+{
+    _XmString str = *(_XmString *)(((char *)w) + offset);
+
+    *value = str
+        ? (XtArgVal)_XmStringCreateExternal(IG_Font(w), str)
+        : (XtArgVal)NULL;
+}
+
 void
 _XmClearIconCompatibility(Widget pbg)
 {
