@@ -1,7 +1,7 @@
 #define	LARGE_PAGE
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.26 1999/12/29 18:24:44 danny Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.27 2000/01/25 23:56:27 amai Exp $
  *
  * Copyright (C) 1997-1998 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.26 1999/12/29 18:24:44 danny Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Notebook.c,v 1.27 2000/01/25 23:56:27 amai Exp $";
 
 
 /*
@@ -87,7 +87,7 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Noteboo
 
 /* Forward Declarations */
 
-static void ClassInitialize();
+static void ClassInitialize(void);
 
 static void ClassPartInitialize(WidgetClass w_class);
 
@@ -506,7 +506,7 @@ XmNotebookClassRec xmNotebookClassRec = {
 WidgetClass xmNotebookWidgetClass = (WidgetClass)&xmNotebookClassRec;
 
 static void
-ClassInitialize()
+ClassInitialize(void)
 {
     _XmNotebookCoreClassExtRec.record_type = XmQmotif;
 
