@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.12 2000/02/15 23:32:53 danny Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.13 2000/04/24 06:04:07 danny Exp $
  *
  * Copyright (C) 1995-1998 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.12 2000/02/15 23:32:53 danny Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.13 2000/04/24 06:04:07 danny Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -1063,6 +1063,7 @@ insert_child(Widget w)
 		XmNseparatorType,	XmSHADOW_ETCHED_IN,
 		XmNorientation,		(PW_Orientation(pw) == XmVERTICAL)
 						? XmHORIZONTAL : XmVERTICAL,
+		XmNnavigationType,	XmNONE,
 					NULL);
 
     PWC_Separator(PWC_Separator(w)) = w;
@@ -1084,6 +1085,8 @@ insert_child(Widget w)
 		  pw);
 
     PW_RecursivelyCalled(pw) = False;
+
+    XtVaSetValues(w, XmNnavigationType, XmTAB_GROUP, NULL);
 }
 
 static void
