@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Trait.c,v 1.8 1998/09/24 15:37:40 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Trait.c,v 1.10 2000/01/26 12:44:00 amai Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Trait.c,v 1.8 1998/09/24 15:37:40 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Trait.c,v 1.10 2000/01/26 12:44:00 amai Exp $";
 
 /*
  * This file contains a poor man's implementation of traits.
@@ -85,6 +85,7 @@ XrmQuark	XmQTaccessTextual = NULLQUARK,
 		XmQTmenuSystem = NULLQUARK,
 		XmQTnavigator = NULLQUARK,
 		XmQTscrollFrame = NULLQUARK,
+		XmQTspecifyLayoutDirection = NULLQUARK,
 		XmQTspecifyRenderTable = NULLQUARK,
 		XmQTtakesDefault = NULLQUARK,
 		XmQTtransfer = NULLQUARK;
@@ -300,7 +301,7 @@ XmeTraitSet(XtPointer obj, XrmQuark trait, XtPointer rec)
 }
 
 void
-_XmInitTraits()
+_XmInitTraits(void)
 {
 	if (XmQTaccessTextual) {
 	    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmInitTraits() 2nd time\n"));
@@ -320,6 +321,7 @@ _XmInitTraits()
 	XmQTmenuSystem = XrmStringToQuark("XmQTmenuSystem");
 	XmQTnavigator = XrmStringToQuark("XmQTnavigator");
 	XmQTscrollFrame = XrmStringToQuark("XmQTscrollFrame");
+	XmQTspecifyLayoutDirection = XrmStringToQuark("XmQTspecifyLayoutDirection");
 	XmQTspecifyRenderTable = XrmStringToQuark("XmQTspecifyRenderTable");
 	XmQTtakesDefault = XrmStringToQuark("XmQTtakesDefault");
 	XmQTtransfer = XrmStringToQuark("XmQTtransfer");
