@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ArrowB.c,v 1.24 1998/10/10 07:40:44 gritton Exp $
+ * $Id: ArrowB.c,v 1.33 2000/03/17 17:48:46 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,13 +22,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ArrowB.c,v 1.24 1998/10/10 07:40:44 gritton Exp $";
+static const char rcsid[] = "$Id: ArrowB.c,v 1.33 2000/03/17 17:48:46 danny Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
 #include <Xm/ArrowBP.h>
+#include <Xm/RepType.h>
 #include <Xm/TransltnsP.h>
 
 #include <XmI/DebugUtil.h>
@@ -45,14 +46,23 @@ static const char rcsid[] = "$Id: ArrowB.c,v 1.24 1998/10/10 07:40:44 gritton Ex
 void _XmArrowB_TraitAddCallback(Widget, XtCallbackProc, XtPointer, Boolean);
 
 static XmActivatableTraitRec _XmArrowBTraitRec = {
-        /* version      */      0,
-        /* cb           */      _XmArrowB_TraitAddCallback
+	/* version      */      0,
+	/* cb           */      _XmArrowB_TraitAddCallback
 };
 #endif
 
+#if XmVERSION > 1
+#define PUSH_GC(w)			(XtSensitive(w) \
+					 ? AB_ArrowGC(w) : AB_InsensitiveGC(w))
+#define DETAIL_SHADOW_THICKNESS(w)	AB_DetailShadowThickness(w)
+#else
+#define PUSH_GC(w)			NULL
+#define DETAIL_SHADOW_THICKNESS(w)	Xm3D_ENHANCE_PIXEL
+#endif
+
 /* Forward Declarations */
 #if XmVERSION > 1
-static void class_initialize();
+static void class_initialize(void);
 #endif
 static void class_part_initialize(WidgetClass w_class);
 
@@ -102,9 +112,26 @@ static XtResource resources[] =
 	XmNdisarmCallback, XmCCallback, XmRCallback,
 	sizeof(XtCallbackList), Offset(disarm_callback),
 	XmRPointer, (XtPointer)NULL
+    },
+#if XmVERSION > 1
+    {
+	XmNdetailShadowThickness, XmCShadowThickness, XmRHorizontalDimension,
+	sizeof(Dimension), Offset(detail_shadow_thickness),
+	XmRImmediate, (XtPointer)2 /* FIX ME */
     }
+#endif
 };
 
+#if XmVERSION > 1
+static XmSyntheticResource syn_resources[] =
+{
+    {
+	XmNdetailShadowThickness,
+	sizeof(Dimension), Offset(detail_shadow_thickness),
+	_XmFromHorizontalPixels, _XmToHorizontalPixels
+    }
+};
+#endif
 
 static void Activate(Widget w, XEvent *event,
 		     String *params, Cardinal *num_params);
@@ -147,7 +174,7 @@ XmArrowButtonClassRec xmArrowButtonClassRec = {
     /* Core class part */
     {
 	/* superclass            */ (WidgetClass) &xmPrimitiveClassRec,
-        /* class_name            */ "XmArrowButton",
+	/* class_name            */ "XmArrowButton",
 	/* widget_size           */ sizeof(XmArrowButtonRec),
 #if XmVERSION > 1
 	/* class_initialize      */ class_initialize,
@@ -176,7 +203,7 @@ XmArrowButtonClassRec xmArrowButtonClassRec = {
 	/* set_values_almost     */ XtInheritSetValuesAlmost,
 	/* get_values_hook       */ NULL,
 	/* accept_focus          */ NULL,
-	/* version               */ XtVersion,
+	/* version               */ XtVersionDontCheck,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ _XmArrowB_defaultTranslations,
 	/* query_geometry        */ NULL,
@@ -186,12 +213,16 @@ XmArrowButtonClassRec xmArrowButtonClassRec = {
     /* Primitive Class part */
     {
 	/* border_highlight      */ XmInheritBorderHighlight,
-        /* border_unhighlight    */ XmInheritBorderUnhighlight,
-        /* translations          */ XtInheritTranslations,
-        /* arm_and_activate_proc */ ArmAndActivate,
-        /* synthetic resources   */ NULL,
+	/* border_unhighlight    */ XmInheritBorderUnhighlight,
+	/* translations          */ XtInheritTranslations,
+	/* arm_and_activate_proc */ ArmAndActivate,
+#if XmVERSION > 1
+	/* synthetic resources   */ syn_resources,
+#else
+	/* synthetic resources   */ NULL,
+#endif
 	/* num syn res           */ 0,
-        /* extension             */ (XtPointer)NULL
+	/* extension             */ (XtPointer)NULL
     },
     /* ArrowButton Class part */
     {
@@ -205,7 +236,7 @@ WidgetClass xmArrowButtonWidgetClass = (WidgetClass)&xmArrowButtonClassRec;
 
 #if XmVERSION > 1
 static void
-class_initialize()
+class_initialize(void)
 {
     if (! XmeTraitSet(xmArrowButtonWidgetClass, XmQTactivatable,
 		&_XmArrowBTraitRec)) {
@@ -259,11 +290,8 @@ CreateInsensitiveGC(Widget w)
     values.background = XtBackground(w);
     values.fill_style = FillStippled;
     values.ts_x_origin = values.ts_y_origin = 0;
-    values.stipple = XmGetPixmapByDepth(XtScreen(w),
-					XmEVEN_STIPPLE_IMAGE,
-					_XmWhitePixelOfObject(w),
-					_XmBlackPixelOfObject(w),
-					1);
+    values.stipple =
+	XmGetPixmapByDepth(XtScreen(w), "50_foreground", 1, 0, 1);
 
     AB_InsensitiveGC(w) = XtGetGC(w, mask, &values);
 }
@@ -272,6 +300,14 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRArrowDirection),
+			     AB_Direction(new_w), new_w))
+	AB_Direction(new_w) = XmARROW_UP;
+
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRMultiClick),
+			     AB_MultiClick(new_w), new_w))
+	AB_MultiClick(new_w) = XmMULTICLICK_KEEP;
+
     if (XtWidth(request) == (Dimension)0)
     {
 	XtWidth(new_w) += 15;
@@ -295,6 +331,7 @@ destroy(Widget w)
     if (AB_Timer(w) != 0)
     {
 	XtRemoveTimeOut(AB_Timer(w));
+	AB_Timer(w) = 0;
     }
 
     XtReleaseGC(w, AB_ArrowGC(w));
@@ -307,14 +344,26 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed = False;
 
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRArrowDirection),
+			     AB_Direction(new_w), new_w))
+	AB_Direction(new_w) = AB_Direction(old);
+
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRMultiClick),
+			     AB_MultiClick(new_w), new_w))
+	AB_MultiClick(new_w) = AB_MultiClick(old);
+
     DEBUGOUT(XdbDebug(__FILE__, new_w, "ArrowB set_values\n"));
 
-    if (AB_Direction(old) != AB_Direction(new_w))
+    if (AB_Direction(old) != AB_Direction(new_w) ||
+#if XmVERSION > 1
+	AB_DetailShadowThickness(old) != AB_DetailShadowThickness(new_w) ||
+#endif
+	XtSensitive(old) != XtSensitive(new_w))
     {
 	refresh_needed = True;
     }
-    if (Prim_Foreground(old) != Prim_Foreground(new_w) || 
-    	XtBackground(old) != XtBackground(new_w))
+    if (Prim_Foreground(old) != Prim_Foreground(new_w) ||
+	XtBackground(old) != XtBackground(new_w))
     {
 	XtReleaseGC(old, AB_ArrowGC(old));
 	XtReleaseGC(old, AB_InsensitiveGC(old));
@@ -329,18 +378,9 @@ set_values(Widget old, Widget request, Widget new_w,
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-    Boolean State;
-    int x, y;
-    int xsize;
-    int ysize;
-    int margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w) + 1;
+    int margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
     GC myGC;
 
-    x = margin;
-    y = margin;
-    xsize = XtWidth(w) - margin * 2;
-    ysize = XtHeight(w) - margin * 2;
-
     if (XtSensitive(w))
     {
 	myGC = AB_ArrowGC(w);
@@ -352,7 +392,7 @@ expose(Widget w, XEvent *event, Region region)
 
     if (Prim_Highlighted(w))
     {
-	    (*PrimC_BorderHighlight(XtClass(w))) (w);
+	(*PrimC_BorderHighlight(XtClass(w))) (w);
     }
     else
     {
@@ -370,37 +410,30 @@ expose(Widget w, XEvent *event, Region region)
 		   Prim_ShadowThickness(w),
 		   XmSHADOW_OUT);
 
-    State = AB_Armed(w);
-    if (State)
+    if (AB_Armed(w))
     {
 	_XmDrawArrow(XtDisplay(w),
 		     XtWindow(w),
-		     Prim_TopShadowGC(w),
 		     Prim_BottomShadowGC(w),
+		     Prim_TopShadowGC(w),
 		     myGC,
-		     x, y,
-		     xsize, ysize,
-	/* Per tegla@katalin.csoma.elte.hu, M*tif ignores
-	   this
-	   Prim_ShadowThickness(w),
-	 */
-		     2,
+		     margin, margin,
+		     XtWidth(w) - (margin << 1),
+		     XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
 		     AB_Direction(w));
     }
     else
     {
 	_XmDrawArrow(XtDisplay(w),
 		     XtWindow(w),
-		     Prim_BottomShadowGC(w),
 		     Prim_TopShadowGC(w),
+		     Prim_BottomShadowGC(w),
 		     myGC,
-		     x, y,
-		     xsize, ysize,
-	/* Per tegla@katalin.csoma.elte.hu, M*tif ignores
-	   this
-	   Prim_ShadowThickness(w),
-	 */
-		     2,
+		     margin, margin,
+		     XtWidth(w) - (margin << 1),
+		     XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
 		     AB_Direction(w));
     }
 }
@@ -436,8 +469,8 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 static void
 Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+    int margin;
     XmArrowButtonCallbackStruct cbs;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Arm\n"));
 
@@ -451,7 +484,20 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	    AB_ArmTimeStamp(w) = event->xbutton.time;
 	}
 
-	(*exp) (w, event, (Region)NULL);
+	if (XtIsRealized(w))
+	{
+	    margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
+	    _XmDrawArrow(XtDisplay(w),
+			 XtWindow(w),
+			 Prim_BottomShadowGC(w),
+			 Prim_TopShadowGC(w),
+			 PUSH_GC(w),
+			 margin, margin,
+			 XtWidth(w) - (margin << 1),
+			 XtHeight(w) - (margin << 1),
+			 DETAIL_SHADOW_THICKNESS(w),
+			 AB_Direction(w));
+	}
 
 	if (AB_ArmCallback(w))
 	{
@@ -471,16 +517,28 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 Activate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+    int margin;
     XmArrowButtonCallbackStruct cbs;
     XButtonEvent *ev = (XButtonEvent *)event;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Activate 000\n"));
 
-    (*exp) (w, event, NULL);
-
-    XFlush(XtDisplay(w));
+    if (XtIsRealized(w))
+    {
+	margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
+	_XmDrawArrow(XtDisplay(w),
+		     XtWindow(w),
+		     Prim_TopShadowGC(w),
+		     Prim_BottomShadowGC(w),
+		     PUSH_GC(w),
+		     margin, margin,
+		     XtWidth(w) - (margin << 1),
+		     XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
+		     AB_Direction(w));
+    }
 
+    AB_Armed(w) = False;
     AB_ClickCount(w) = 1;
 
     if ((ev->x >= 0 && ev->x < XtWidth(w)) &&
@@ -505,15 +563,28 @@ Activate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+    int margin;
     XmArrowButtonCallbackStruct cbs;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "Disarm\n"));
 
     if (AB_Armed(w))
     {
 	AB_Armed(w) = False;
-	(*exp) (w, event, NULL);
+	if (XtIsRealized(w))
+	{
+	    margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
+	    _XmDrawArrow(XtDisplay(w),
+			 XtWindow(w),
+			 Prim_TopShadowGC(w),
+			 Prim_BottomShadowGC(w),
+			 PUSH_GC(w),
+			 margin, margin,
+			 XtWidth(w) - (margin << 1),
+			 XtHeight(w) - (margin << 1),
+			 DETAIL_SHADOW_THICKNESS(w),
+			 AB_Direction(w));
+	}
     }
 
     if (AB_DisarmCallback(w))
@@ -533,16 +604,42 @@ Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 ArmTimeout(XtPointer data, XtIntervalId *id)
 {
+    int margin;
     Widget w = (Widget)data;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
- 
+
     AB_Timer(w) = 0;
 
     if (XtIsRealized(w))
     {
-	(*exp) (w, NULL, NULL);
+	margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
+	if (AB_Armed(w))
+	{
+	    _XmDrawArrow(XtDisplay(w),
+			 XtWindow(w),
+			 Prim_BottomShadowGC(w),
+			 Prim_TopShadowGC(w),
+			 PUSH_GC(w),
+			 margin, margin,
+			 XtWidth(w) - (margin << 1),
+			 XtHeight(w) - (margin << 1),
+			 DETAIL_SHADOW_THICKNESS(w),
+			 AB_Direction(w));
+	}
+	else
+	{
+	    _XmDrawArrow(XtDisplay(w),
+			 XtWindow(w),
+			 Prim_TopShadowGC(w),
+			 Prim_BottomShadowGC(w),
+			 PUSH_GC(w),
+			 margin, margin,
+			 XtWidth(w) - (margin << 1),
+			 XtHeight(w) - (margin << 1),
+			 DETAIL_SHADOW_THICKNESS(w),
+			 AB_Direction(w));
+	}
 	XFlush(XtDisplay(w));
     }
 }
@@ -551,13 +648,12 @@ static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmArrowButtonCallbackStruct cbs;
- 
+
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
 
     /* Arm, Activate, and Disarm now, but draw the disarmed state later */
 
     Arm(w, event, params, num_params);
-    AB_Armed(w) = False;
     if (AB_ActivateCallback(w))
     {
 	XFlush(XtDisplay(w));
@@ -568,6 +664,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 			   AB_ActivateCallback(w),
 			   (XtPointer)&cbs);
     }
+    AB_Armed(w) = False;
     if (AB_DisarmCallback(w))
     {
 	XFlush(XtDisplay(w));
@@ -582,6 +679,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     if (AB_Timer(w) != 0)
     {
 	XtRemoveTimeOut(AB_Timer(w));
+	AB_Timer(w) = 0;
     }
 
     AB_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
@@ -591,9 +689,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    int margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w) + 1;
-    int xsize = XtWidth(w) - margin * 2;
-    int ysize = XtHeight(w) - margin * 2;
+    int margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
 
     DEBUGOUT(XdbDebug(__FILE__, w, "EnterWindow\n"));
 
@@ -603,16 +699,13 @@ EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	_XmDrawArrow(XtDisplay(w),
 		     XtWindow(w),
-		     Prim_TopShadowGC(w),
 		     Prim_BottomShadowGC(w),
-		     AB_ArrowGC(w),
+		     Prim_TopShadowGC(w),
+		     PUSH_GC(w),
 		     margin, margin,
-		     xsize, ysize,
-	/* Per tegla@katalin.csoma.elte.hu, M*tif ignores
-	   this
-	   Prim_ShadowThickness(w),
-	 */
-		     2,
+		     XtWidth(w) - (margin << 1),
+		     XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
 		     AB_Direction(w));
     }
 }
@@ -620,9 +713,7 @@ EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    int margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w) + 1;
-    int xsize = XtWidth(w) - margin * 2;
-    int ysize = XtHeight(w) - margin * 2;
+    int margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
 
     DEBUGOUT(XdbDebug(__FILE__, w, "LeaveWindow\n"));
 
@@ -632,16 +723,13 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	_XmDrawArrow(XtDisplay(w),
 		     XtWindow(w),
-		     Prim_BottomShadowGC(w),
 		     Prim_TopShadowGC(w),
-		     AB_ArrowGC(w),
+		     Prim_BottomShadowGC(w),
+		     PUSH_GC(w),
 		     margin, margin,
-		     xsize, ysize,
-	/* Per tegla@katalin.csoma.elte.hu, M*tif ignores
-	   this
-	   Prim_ShadowThickness(w),
-	 */
-		     2,
+		     XtWidth(w) - (margin << 1),
+		     XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
 		     AB_Direction(w));
     }
 }
@@ -660,9 +748,9 @@ MultiArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 MultiActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+    int margin;
     XButtonEvent *ev = (XButtonEvent *)event;
     XmPushButtonCallbackStruct cbs;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "PushB: MultiClick\n"));
 
@@ -681,18 +769,25 @@ MultiActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	AB_Armed(w) = False;
 
-	(*exp) (w, event, NULL);
+	if (XtIsRealized(w))
+	{
+	    margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
+	    _XmDrawArrow(XtDisplay(w),
+			 XtWindow(w),
+			 Prim_TopShadowGC(w),
+			 Prim_BottomShadowGC(w),
+			 PUSH_GC(w),
+			 margin, margin,
+			 XtWidth(w) - (margin << 1),
+			 XtHeight(w) - (margin << 1),
+			 DETAIL_SHADOW_THICKNESS(w),
+			 AB_Direction(w));
+	}
 
 	if (ev->type == KeyPress || ev->type == KeyRelease ||
 	    ((ev->x >= 0 && ev->x < XtWidth(w)) &&
 	     (ev->y >= 0 && ev->y < XtHeight(w))))
 	{
-	    if (AB_MultiClick(w) == XmMULTICLICK_DISCARD &&
-		AB_ClickCount(w) > 1)
-	    {
-		return;
-	    }
-
 	    if (AB_ActivateCallback(w))
 	    {
 		cbs.reason = XmCR_ACTIVATE;
@@ -720,13 +815,13 @@ XmCreateArrowButton(Widget parent, char *name, Arg *arglist, Cardinal argcount)
 
 #if XmVERSION > 1
 void _XmArrowB_TraitAddCallback(Widget w,
-                        XtCallbackProc cb,
-                        XtPointer cbp,
-                        Boolean set)
-{   
-        if (set)
-                XtAddCallback(w, XmNactivateCallback, cb, cbp);
-        else
-                XtRemoveCallback(w, XmNactivateCallback, cb, cbp);
+				XtCallbackProc cb,
+				XtPointer cbp,
+				Boolean set)
+{
+	if (set)
+	    XtAddCallback(w, XmNactivateCallback, cb, cbp);
+	else
+	    XtRemoveCallback(w, XmNactivateCallback, cb, cbp);
 }
 #endif
