@@ -1,6 +1,6 @@
 /**
  *
- * $Id: BulletinBoard.c,v 1.38 1998/10/23 17:27:39 rwscott Exp $
+ * $Id: BulletinBoard.c,v 1.50 2000/01/10 13:35:08 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: BulletinBoard.c,v 1.38 1998/10/23 17:27:39 rwscott Exp $";
+static const char rcsid[] = "$Id: BulletinBoard.c,v 1.50 2000/01/10 13:35:08 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -44,7 +44,7 @@ static const char rcsid[] = "$Id: BulletinBoard.c,v 1.38 1998/10/23 17:27:39 rws
 
 /* Forward Declarations */
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -55,8 +55,10 @@ static void destroy(Widget w);
 
 static void resize(Widget w);
 
+/*
 static void realize(Widget w, XtValueMask *value_mask,
 		    XSetWindowAttributes *attributes);
+*/
 
 static XtGeometryResult query_geometry(Widget w,
 				       XtWidgetGeometry *proposed,
@@ -220,12 +222,13 @@ static XmSyntheticResource syn_resources[] =
 };
 
 static char _XmBulletinB_mapTranslations[] =
-    "<Map>:           BulletinBoardMap()\n\
-     <Unmap>:         BulletinBoardMap()";
+	"";
 
 static XtActionsRec actions[] =
 {
-    {"BulletinBoardMap", _XmBulletinBoardMap}
+    /* {"Return", _XmReturn}, */
+    {"BulletinBoardReturn", (XtActionProc)_XmBulletinBoardReturn},
+    {"BulletinBoardCancel", (XtActionProc)_XmBulletinBoardCancel},
 };
 
 static XtTranslations mapTrans;
@@ -290,7 +293,7 @@ XmBulletinBoardClassRec xmBulletinBoardClassRec = {
 	/* class_inited          */ False,
 	/* initialize            */ initialize,
 	/* initialize_hook       */ NULL,
-	/* realize               */ realize,
+	/* realize               */ XtInheritRealize /*realize*/,
 	/* actions               */ actions,
 	/* num_actions           */ XtNumber(actions),
 	/* resources             */ resources,
@@ -357,7 +360,7 @@ XmBulletinBoardClassRec xmBulletinBoardClassRec = {
 WidgetClass xmBulletinBoardWidgetClass = (WidgetClass)&xmBulletinBoardClassRec;
 
 static void
-class_initialize()
+class_initialize(void)
 {
     /* Motif does not have this method */
 #if 0
@@ -461,8 +464,11 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 	BB_TextFontList(bb) = XmFontListCopy(BB_TextFontList(new_w));
     }
 
-    XtAugmentTranslations(new_w, mapTrans);
-    XtAugmentTranslations(new_w, defTrans);
+    if (((XmBulletinBoardWidgetClass)XtClass(new_w))->bulletin_board_class.always_install_accelerators)
+    {
+	XtAugmentTranslations(new_w, mapTrans);
+	XtAugmentTranslations(new_w, defTrans);
+    }
     /* rws 30 Aug 1998
        We seem to loose a few hundred bytes every time we parse a translation
        table, so like mapTrans lets parse it in class_initialize and use the
@@ -521,8 +527,7 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 	{
 	    char *p;
 
-	    if (XmStringGetLtoR(BB_DialogTitle(new_w),
-				XmFONTLIST_DEFAULT_TAG, &p))
+	    if ((p = _XmStringGetTextConcat(BB_DialogTitle(new_w))))
 	    {
 		XtVaSetValues(XtParent(new_w), XtNtitle, p, NULL);
 		XtFree(p);
@@ -569,6 +574,9 @@ destroy(Widget w)
 {
   Widget ve;
 
+    DEBUGOUT(XdbDebug("RWS", w,"%s:destroy(%d)\n",
+    	__FILE__, __LINE__
+    	));
     XmFontListFree(BB_LabelFontList(w));
     XmFontListFree(BB_ButtonFontList(w));
     XmFontListFree(BB_TextFontList(w));
@@ -591,9 +599,22 @@ set_values(Widget old,
     XmBulletinBoardWidget nw = (XmBulletinBoardWidget)new_w;
     Boolean need_refresh = False;
 
-    BB_InSetValues(new_w) = True;
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s(%d):set_values - %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(old), XtY(old),
+		      XtWidth(old), XtHeight(old),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "SetValues\n"));
+    BB_InSetValues(new_w) = True;
 
     if (BB_DefaultButton(ow) != BB_DefaultButton(nw))
     {
@@ -623,9 +644,7 @@ set_values(Widget old,
 	{
 	    char *p;
 
-	    if (XmStringGetLtoR(BB_DialogTitle(new_w),
-				XmFONTLIST_DEFAULT_TAG,
-				&p))
+	    if ((p = _XmStringGetTextConcat(BB_DialogTitle(new_w))))
 	    {
 		XtVaSetValues(XtParent(new_w), XtNtitle, p, NULL);
 		XtFree(p);
@@ -635,9 +654,13 @@ set_values(Widget old,
 	    XmStringFree(BB_DialogTitle(old));
 	}
 
+#if 0
 	/*
 	 * If we're a dialog, refuse to be placed away from 0,0
 	 */
+	/* rws 23 Jan 1999
+	   Na, let DialogShell handle the geometry request that Xt will make.
+	 */
 	if (XtX(new_w) != 0)
 	{
 	    XtX(new_w) = 0;
@@ -649,6 +672,7 @@ set_values(Widget old,
 	    XtY(new_w) = 0;
 	    need_refresh = True;
 	}
+#endif
     }
 
     if (XmIsDialogShell(XtParent(new_w)))
@@ -887,6 +911,7 @@ handle_geometry_manager(Widget w,
 
     BB_OldWidth(bb) = XtWidth(bb);
     BB_OldHeight(bb) = XtHeight(bb);
+    BB_OldShadowThickness(bb) = MGR_ShadowThickness(bb);
 
     return res;
 }
@@ -1156,6 +1181,7 @@ constraint_set_values(Widget old, Widget request, Widget new_w,
 }
 #endif
 
+#if 0
 static void
 handle_realize(Widget w, XmGeoCreateProc mat_make)
 {
@@ -1224,7 +1250,9 @@ handle_realize(Widget w, XmGeoCreateProc mat_make)
     BB_OldHeight(w) = XtHeight(w);
     BB_OldShadowThickness(w) = MGR_ShadowThickness(w);
 }
+#endif
 
+#if 0
 static void
 realize(Widget w,
 	XtValueMask *value_mask,
@@ -1284,6 +1312,7 @@ realize(Widget w,
     BB_OldShadowThickness(w) = MGR_ShadowThickness(w);
 #endif
 }
+#endif
 
 static void
 expose(Widget w, XEvent *event, Region region)
@@ -1453,7 +1482,7 @@ _XmBB_CreateButtonG(Widget bb, XmString l_string, char *name)
     Arg args[2];
     Cardinal n = 0;
 
-    if (l_string)
+    if (l_string && l_string != (XmString)XmUNSPECIFIED)
     {
 	XtSetArg(args[n], XmNlabelString, l_string); n++;
     }
@@ -1474,7 +1503,7 @@ _XmBB_CreateLabelG(Widget bb, XmString l_string, char *name)
     Arg args[2];
     Cardinal n = 0;
 
-    if (l_string)
+    if (l_string && l_string != (XmString)XmUNSPECIFIED)
     {
 	XtSetArg(args[n], XmNlabelString, l_string); n++;
     }
@@ -1619,32 +1648,25 @@ _XmBulletinBoardFocusMoved(Widget wid, XtPointer client_data, XtPointer data)
     }
 }
 
-
 void
-_XmBulletinBoardMap(Widget w, XEvent *event,
-		    String *params, Cardinal *numParams)
+_XmBulletinBoardUnmap(Widget w)
 {
-    XMapEvent *mev = (XMapEvent *)event;
-    XUnmapEvent *uev = (XUnmapEvent *) event;
     XmAnyCallbackStruct cb;
 
-    cb.event = event;
-    if (mev->type == MapNotify)
-    {
-	DEBUGOUT(XdbDebug(__FILE__, w, "Map\n"));
+    cb.event = NULL;
+    cb.reason = XmCR_UNMAP;
 
-	cb.reason = XmCR_MAP;
+    XtCallCallbackList(w, BB_UnmapCallback(w), (XtPointer)&cb);
+}
 
-	XtCallCallbackList(w, BB_MapCallback(w), (XtPointer)&cb);
-    }
-    else if (uev->type == UnmapNotify)
-    {
-	DEBUGOUT(XdbDebug(__FILE__, w, "Unmap\n"));
+void
+_XmBulletinBoardMap(Widget w)
+{
+    XmAnyCallbackStruct cb;
 
-	cb.reason = XmCR_UNMAP;
-
-	XtCallCallbackList(w, BB_UnmapCallback(w), (XtPointer)&cb);
-    }
+    cb.event = NULL;
+    cb.reason = XmCR_MAP;
+    XtCallCallbackList(w, BB_MapCallback(w), (XtPointer)&cb);
 }
 
 static void
