@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DragC.c,v 1.26 2000/02/13 12:43:25 rwscott Exp $
+ * $Id: DragC.c,v 1.27 2000/04/18 01:58:02 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -40,7 +40,7 @@
  * provided "as is" without express or implied warranty.
  */
 
-static const char rcsid[] = "$Id: DragC.c,v 1.26 2000/02/13 12:43:25 rwscott Exp $";
+static const char rcsid[] = "$Id: DragC.c,v 1.27 2000/04/18 01:58:02 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -537,6 +537,9 @@ send_drag_message(Widget dc, Window win, XmDragDropCallbackStruct * cbs,
     DEBUGOUT(XdbDebug("DRAGSOURCE", dc, "%s:send_drag_message(%d) - %s to %p\n",
     	__FILE__, __LINE__,
     	XdbDragAndDropMessageType2String(cbs->any.reason), win));
+    DEBUGOUT(XdbDebug("DRAGSINK", dc, "%s:send_drag_message(%d) - %s to %p\n",
+    	__FILE__, __LINE__,
+    	XdbDragAndDropMessageType2String(cbs->any.reason), win));
 
     msg = XmInternAtom(XtDisplay(dc), _XA_MOTIF_DRAG_AND_DROP_MESSAGE, False);
 
@@ -1162,6 +1165,8 @@ external_notify(Widget w, XtPointer cd, XtPointer cbs)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "%s:external_notify(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSINK", w, "%s:external_notify(%d)\n",
+    	__FILE__, __LINE__));
 
     switch (cb->any.reason)
     {
@@ -1898,9 +1903,9 @@ drag_main_loop(XtPointer data, XtIntervalId *id)
 	    case EnterNotify:
 	    case LeaveNotify:
 		DEBUGOUT(XdbDebug(__FILE__, dc,
-				  "Type: %d Window: %08x swindow: %08x "
+				  "Type: %s Window: %08x swindow: %08x "
 				  "root: %08x\n",
-				  event.type, event.xbutton.subwindow,
+				  XdbEventType2String(event.type), event.xbutton.subwindow,
 				  event.xbutton.window, event.xbutton.root));
 
 		/* so that the event gets delivered to the right widget (us) */
@@ -2803,8 +2808,12 @@ XmDragStart(Widget widget,
     Arg arg[1];
     ArgList merged;
 
-    DEBUGOUT(XdbDebug(__FILE__, widget, "XmDragStart()\n"));
-    DEBUGOUT(XdbDebug("DRAGSOURCE", widget, "XmDragStart() - %s\n",
+    DEBUGOUT(XdbDebug(__FILE__, widget, "%s:XmDragStart(%d) - %s\n",
+    	__FILE__, __LINE__,
+    	XdbDragType2String(Display_DragInitiatorProtocolStyle(disp))));
+    DEBUGOUT(XdbPrintArgList(__FILE__, widget, arglist, argcount, False));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", widget, "%s:XmDragStart(%d) - %s\n",
+    	__FILE__, __LINE__,
     	XdbDragType2String(Display_DragInitiatorProtocolStyle(disp))));
     DEBUGOUT(XdbPrintArgList("DRAGSOURCE", widget, arglist, argcount, False));
 
