@@ -1,6 +1,6 @@
 /** -*- Mode: C; tab-width: 8; indent-tabs-mode: nil -*-
  *
- * $Id: DropSMgr.c,v 1.27 2000/02/13 12:43:26 rwscott Exp $
+ * $Id: DropSMgr.c,v 1.28 2000/04/18 01:58:05 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DropSMgr.c,v 1.27 2000/02/13 12:43:26 rwscott Exp $";
+static const char rcsid[] = "$Id: DropSMgr.c,v 1.28 2000/04/18 01:58:05 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -430,6 +430,7 @@ initialize(Widget request, Widget new_w,
 
     if (DS_TreeUpdateProc(new_w) == NULL)
 	DS_TreeUpdateProc(new_w) = updateTree;
+
 }
 
 static void
@@ -470,8 +471,9 @@ updateReceiverInfo(Widget w, XtPointer cd, XEvent *ev, Boolean *ctd)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "%s:updateReceiverInfo(%d)\n",
     	__FILE__, __LINE__));
-    DEBUGOUT(XdbDebug("DRAGSINK", w, "%s:updateReceiverInfo(%d)\n",
-    	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSINK", w, "%s:updateReceiverInfo(%d) - %s\n",
+    	__FILE__, __LINE__,
+    	XdbEventType2String(ev->xany.type)));
 #if 1
     /* rws 8 Feb 2000
        I have no idea if this belongs here or not, but it has to be
@@ -501,6 +503,141 @@ updateReceiverInfo(Widget w, XtPointer cd, XEvent *ev, Boolean *ctd)
 #endif
 }
 
+static void
+SwapMessageData(XmDndMessage *msg)
+{
+    if (msg->byte_order != _XmByteOrder())
+    {
+	SWAP2BYTES(msg->flags);
+	SWAP4BYTES(msg->time);
+	switch (msg->reason & DND_CLEAR_EVENT_TYPE)
+	{
+	case XmTOP_LEVEL_ENTER:
+	case XmTOP_LEVEL_LEAVE:
+	    SWAP4BYTES(msg->data.top.property);
+	    SWAP4BYTES(msg->data.top.src_window);
+	    break;
+	case XmDROP_SITE_ENTER:
+	case XmDRAG_MOTION:
+	case XmDROP_START:
+	    SWAP2BYTES(msg->data.pot.x);
+	    SWAP2BYTES(msg->data.pot.y);
+	    SWAP4BYTES(msg->data.pot.property);
+	    SWAP4BYTES(msg->data.pot.src_window);
+	    break;
+	case XmDROP_SITE_LEAVE:
+	case XmDROP_FINISH:
+	case XmDRAG_DROP_FINISH:
+	case XmOPERATION_CHANGED:
+	    break;
+	default:
+	    _XmWarning(NULL, "Unexpected DnD message type >%i<\n    %s:SwapMessageData(%d)", 
+		msg->reason, __FILE__, __LINE__);
+	    break;
+	}
+    }
+}
+
+int
+_XmMessageTypeToReason(BYTE reason)
+{
+    switch (reason & DND_CLEAR_EVENT_TYPE)
+    {
+    case XmTOP_LEVEL_ENTER:
+	return(XmCR_TOP_LEVEL_ENTER);
+	break;
+    case XmTOP_LEVEL_LEAVE:
+	return(XmCR_TOP_LEVEL_LEAVE);
+	break;
+    case XmDROP_SITE_ENTER:
+	return(XmCR_DROP_SITE_ENTER);
+	break;
+    case XmDRAG_MOTION:
+	return(XmCR_DRAG_MOTION);
+	break;
+    case XmDROP_START:
+	return(XmCR_DROP_START);
+	break;
+    case XmDROP_SITE_LEAVE:
+	return(XmCR_DROP_SITE_LEAVE);
+	break;
+    case XmDROP_FINISH:
+	return(XmCR_DROP_FINISH);
+	break;
+    case XmDRAG_DROP_FINISH:
+	return(XmCR_DRAG_DROP_FINISH);
+	break;
+    case XmOPERATION_CHANGED:
+	return(XmCR_OPERATION_CHANGED);
+	break;
+    default:
+	_XmWarning(NULL, "Unexpected DnD message type >%i<\n    %s:_XmMessageTypeToReason(%d)", 
+	    reason, __FILE__, __LINE__);
+	return(reason);
+	break;
+    }
+}
+
+void
+_XmICCEventToICCCallback(XmDndMessage *msg, XmDragDropCallbackStruct *cbs)
+{
+    switch (msg->reason & DND_CLEAR_EVENT_TYPE)
+    {
+    case XmTOP_LEVEL_ENTER:
+	/*
+	cbs->tle.screen = 
+	cbs->tle.x =
+	cbs->tle.y =
+	cbs->tle.dragProtocolStyle =
+	*/
+	cbs->tle.window = msg->data.top.src_window;
+	cbs->tle.iccHandle = msg->data.top.property;
+	break;
+    case XmTOP_LEVEL_LEAVE:
+	/*
+	cbs->tll.screen = 
+	*/
+	cbs->tle.window = msg->data.top.src_window;
+	break;
+    case XmDROP_SITE_ENTER:
+	cbs->dse.operation = DND_GET_OPERATION(msg->flags);
+	cbs->dse.operations = DND_GET_OPERATIONS(msg->flags);
+	cbs->dse.dropSiteStatus = DND_GET_STATUS(msg->flags);
+	cbs->dse.x = msg->data.pot.x;
+	cbs->dse.y = msg->data.pot.y;
+	break;
+    case XmDRAG_MOTION:
+	cbs->dm.operation = DND_GET_OPERATION(msg->flags);
+	cbs->dm.operations = DND_GET_OPERATIONS(msg->flags);
+	cbs->dm.dropSiteStatus = DND_GET_STATUS(msg->flags);
+	cbs->dm.x = msg->data.pot.x;
+	cbs->dm.y = msg->data.pot.y;
+	break;
+    case XmDROP_START:
+	cbs->ds.operation = DND_GET_OPERATION(msg->flags);
+	cbs->ds.operations = DND_GET_OPERATIONS(msg->flags);
+	cbs->ds.dropSiteStatus = DND_GET_STATUS(msg->flags);
+	cbs->ds.dropAction = DND_GET_COMPLETION(msg->flags);
+	cbs->ds.x = msg->data.pot.x;
+	cbs->ds.y = msg->data.pot.y;
+	cbs->ds.iccHandle = msg->data.pot.property;
+	cbs->ds.window = msg->data.pot.src_window;
+	break;
+    case XmDROP_SITE_LEAVE:
+	/* nothing to do */
+	break;
+    case XmOPERATION_CHANGED:
+	cbs->oc.operation = DND_GET_OPERATION(msg->flags);
+	cbs->oc.operations = DND_GET_OPERATIONS(msg->flags);
+	cbs->oc.dropSiteStatus = DND_GET_STATUS(msg->flags);
+	break;
+    default:
+	_XmWarning(NULL, "Unexpected DnD message type >%i<\n    %s:_XmICCEventToICCCallback(%d)", 
+	    msg->reason, __FILE__, __LINE__);
+	break;
+    }
+}
+
 static void
 externalSourceHandler(Widget w, XtPointer cd, XEvent *ev, Boolean *ctd)
 {
@@ -508,6 +645,75 @@ externalSourceHandler(Widget w, XtPointer cd, XEvent *ev, Boolean *ctd)
     	__FILE__, __LINE__));
     DEBUGOUT(XdbDebug2("DRAGSINK", w, (Widget)cd, "%s:externalSourceHandler(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug0("DRAGSINK", w, "\t%s\n",
+    	XdbEventType2String(ev->xany.type)));
+
+    switch (ev->xany.type)
+    {
+    case ClientMessage:
+	{
+	Atom DndMessage;
+
+	    DndMessage = XmInternAtom(ev->xclient.display, _XA_MOTIF_DRAG_AND_DROP_MESSAGE, False);
+	    if (DndMessage == ev->xclient.message_type)
+	    {
+	    XmDndMessage *msg = (XmDndMessage *)&(ev->xclient.data.b[0]);
+	    XmDropSiteManagerObject dsm = (XmDropSiteManagerObject) cd;
+	    XmDisplay disp = (XmDisplay)XmGetXmDisplay(ev->xclient.display);
+	    XmDragDropCallbackStruct DnDcbs;
+	    XmDragTopLevelClientDataStruct TLCcbs;
+
+	    	SwapMessageData(msg);
+		DEBUGOUT(XdbDebug0("DRAGSINK", w, "\t%s from %s\n",
+		    XdbDragAndDropMessageType2String(msg->reason & DND_CLEAR_EVENT_TYPE),
+		    msg->reason & ~DND_CLEAR_EVENT_TYPE ? "Receiver" : "Initiator"));
+		DEBUGOUT(XdbDebug0("DRAGSINK", w, "\tDS_CurDragContext %s %p\n",
+		    DS_CurDragContext(dsm) ? XtName(DS_CurDragContext(dsm)) : "NULL", DS_CurDragContext(dsm)));
+		DEBUGOUT(XdbDebug0("DRAGSINK", w, "\tDisplay_ActiveDC %s %p\n",
+		    Display_ActiveDC(disp) ? XtName(Display_ActiveDC(disp)) : "NULL", Display_ActiveDC(disp)));
+		DEBUGOUT(XdbDebug0("DRAGSINK", w, "\tXmGetDragContext %s %p\n",
+		    XmGetDragContext((Widget)dsm, msg->time) ? XtName(XmGetDragContext((Widget)dsm, msg->time)) : "NULL", XmGetDragContext((Widget)dsm, msg->time)));
+
+		DnDcbs.any.reason = _XmMessageTypeToReason(msg->reason);
+		DnDcbs.any.event = ev;
+		DnDcbs.any.timeStamp = msg->time;
+		_XmICCEventToICCCallback(msg, &DnDcbs);
+		TLCcbs.shell = w;
+		TLCcbs.sourceIsExternal = True;
+		TLCcbs.window = ev->xclient.window;
+		TLCcbs.xOrigin = XtX(w);
+		TLCcbs.yOrigin = XtY(w);
+		TLCcbs.width = XtWidth(w);
+		TLCcbs.height = XtHeight(w);
+		DS_CurDragContext(dsm) = XmGetDragContext((Widget)dsm,
+						  DnDcbs.any.timeStamp);
+		DSMUpdate(dsm, &TLCcbs, &DnDcbs);
+	    }
+	    else
+	    {
+	    char *AtomName;
+
+		AtomName = XGetAtomName(ev->xclient.display, ev->xclient.message_type);
+		_XmWarning(w, "Unexpected ClientMessaget, type >%s<\n    %s:externalSourceHandler(%d)", 
+		    AtomName, __FILE__, __LINE__);
+		XtFree(AtomName);
+	    }
+	}
+	break;
+    case SelectionClear:
+	{
+	char *AtomName;
+
+	    AtomName = XGetAtomName(ev->xselectionclear.display, ev->xselectionclear.selection);
+	    _XmWarning(w, "Unexpected SelectionClear event, losing %s selection\n    %s:externalSourceHandler(%d)", 
+	    	AtomName, __FILE__, __LINE__);
+	    XtFree(AtomName);
+	}
+	break;
+    default:
+    	_XmWarning(w, "Unexpected event >%s<\n    %s:externalSourceHandler(%d)", XdbEventType2String(ev->xany.type), __FILE__, __LINE__);
+    	;
+    }
 #if 0
     Widget dc, disp;
     XmDropSiteManagerObject dsm;
@@ -1110,6 +1316,7 @@ __XmDSMUpdateDSM(XmDropSiteManagerObject dsm,
     XmDragDropCallbackStruct *cbs = (XmDragDropCallbackStruct *) callData;
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM UpdateDSM\n"));
+    DEBUGOUT(XdbDebug("DRAGSINK", (Widget)dsm, "__XmDSMUpdateDSM\n"));
 
     switch (cbs->any.reason)
     {
@@ -1424,7 +1631,7 @@ drag_motion(XmDropSiteManagerObject dsm, XmDragMotionClientDataStruct * cd,
     XmDragMotionCallbackStruct dm;
     Position x, y;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:drag_motion(%d)\n",
+    DEBUGOUT(XdbDebug("DRAGSINK", (Widget)dsm, "%s:drag_motion(%d)\n",
     	__FILE__, __LINE__));
 
     dpc.reason = XmCR_DROP_SITE_MOTION_MESSAGE;
@@ -1516,10 +1723,14 @@ __XmDSMProcessMotion(XmDropSiteManagerObject dsm,
     DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "__XmDSMProcessMotion - %s %+i%+i %+i%+i\n",
     	XdbDragAndDropMessageType2String(cbs->reason),
     	cbs->x, cbs->y, DS_RootX(dsm), DS_RootY(dsm)));
+    DEBUGOUT(XdbDebug("DRAGSINK", (Widget)dsm, "__XmDSMProcessMotion - %s %+i%+i %+i%+i\n",
+    	XdbDragAndDropMessageType2String(cbs->reason),
+    	cbs->x, cbs->y, DS_RootX(dsm), DS_RootY(dsm)));
 
     if (DS_CurDragContext(dsm) == NULL)
     {
-	_XmWarning((Widget)dsm, "Eeek!  Where'd that come from?\n");
+	_XmWarning((Widget)dsm, "Eeek!  Where'd that come from? %s(%d)\n",
+		__FILE__, __LINE__);
 	return;
     }
 
@@ -1558,7 +1769,7 @@ __XmDSMProcessMotion(XmDropSiteManagerObject dsm,
 	}
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, 
+    DEBUGOUT(XdbDebug("DRAGSINK", 
                       (Widget)dsm, "Dragging on %s\n", 
                       dsi ? XtName(dsi->dropSite) : "NULL"));
     drag_motion(dsm, cd, cbs, dsi, ps);
@@ -1586,12 +1797,16 @@ __XmDSMProcessDrop(XmDropSiteManagerObject dsm,
     Atom *itargs;
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "__XmDSMProcessDrop\n"));
+    DEBUGOUT(XdbDebug("DRAGSINK", (Widget)dsm, "__XmDSMProcessDrop %s %s\n",
+    	cbs->dropAction == XmDROP ? "XmDROP" : "not XmDROP",
+    	cbs->dropSiteStatus == XmDROP_SITE_VALID ? "XmDROP_SITE_VALID" : "not XmDROP_SITE_VALID"));
 
     dc = XmGetDragContext((Widget)dsm, cbs->timeStamp);
 
     if (dc == NULL)
     {
-	_XmWarning((Widget)dsm, "Eeek!  Where'd that come from?\n");
+	_XmWarning((Widget)dsm, "Eeek!  Where'd that come from? %s(%d)\n",
+		__FILE__, __LINE__);
 	return;
     }
 
@@ -1732,7 +1947,8 @@ __XmDSMOperationChanged(XmDropSiteManagerObject dsm,
 
     if (DS_CurDragContext(dsm) == NULL)
     {
-	_XmWarning((Widget)dsm, "Eeek!  Where'd that come from?\n");
+	_XmWarning((Widget)dsm, "Eeek!  Where'd that come from? %s(%d)\n",
+		__FILE__, __LINE__);
 	return;
     }
 
@@ -1862,13 +2078,29 @@ __XmDSMChangeRoot(XmDropSiteManagerObject dsm,
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "__XmDSMChangeRoot: %s\n",
 		      XdbDragAndDropMessageType2String(cbs->any.reason)));
+    DEBUGOUT(XdbDebug("DRAGSINK", (Widget)dsm, "__XmDSMChangeRoot: %s\n",
+		      XdbDragAndDropMessageType2String(cbs->any.reason)));
 
     DS_CurTime(dsm) = cbs->any.timeStamp;
 
     if (cbs->any.reason == XmCR_TOP_LEVEL_ENTER)
     {
-	DS_CurDragContext(dsm) = XmGetDragContext((Widget)dsm,
+	if (cd->sourceIsExternal)
+	{
+	    DS_CurDragContext(dsm) = XtVaCreateWidget("ExternalDragC", 
+	    	xmDragContextClass, XmGetXmDisplay(XtDisplay(dsm)),
+		XmNsourceIsExternal, True, 
+		XmNsourceWindow, cbs->tle.window,
+		XmNiccHandle, cbs->tle.iccHandle,
+		NULL);
+	    DC_CurrReceiverInfo(DS_CurDragContext(dsm)) = _XmAllocReceiverInfo((XmDragContext)DS_CurDragContext(dsm));
+	    (DC_CurrReceiverInfo(DS_CurDragContext(dsm)))->shell = cd->shell;
+	}
+	else
+	{
+	    DS_CurDragContext(dsm) = XmGetDragContext((Widget)dsm,
 						  cbs->any.timeStamp);
+	}
 
 	if (cd->shell)
 	{
