@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ImageCache.c,v 1.17 1999/07/31 08:44:19 danny Exp $
+ * $Id: ImageCache.c,v 1.18 2000/03/16 13:00:40 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ImageCache.c,v 1.17 1999/07/31 08:44:19 danny Exp $";
+static const char rcsid[] = "$Id: ImageCache.c,v 1.18 2000/03/16 13:00:40 rwscott Exp $";
 
 /*
  * --aldi 97/01/11: changed to use the generic hashtable module.
@@ -280,7 +280,7 @@ XmGetPixmapByDepth(Screen *screen, char *image_name,
     GC gc;
     XGCValues values;
 #ifdef NONSTANDARD_CONVERTERS
-    _LtXpmColorSymbol xpm_colour_symbols[2];
+    _LtXpmColorSymbol xpm_colour_symbols[3];
     _LtXpmAttributes xpm_attrib;
 #endif /* NONSTANDARD_CONVERTERS */
 
@@ -457,6 +457,9 @@ XmGetPixmapByDepth(Screen *screen, char *image_name,
     xpm_colour_symbols[1].name = "foreground";
     xpm_colour_symbols[1].value = 0;
     xpm_colour_symbols[1].pixel = foreground ;    
+    xpm_colour_symbols[2].name = NULL;
+    xpm_colour_symbols[2].value = "None";
+    xpm_colour_symbols[2].pixel = background ;
 
     xpm_attrib.colorsymbols = xpm_colour_symbols;
     xpm_attrib.numsymbols = XtNumber(xpm_colour_symbols);
