@@ -1,5 +1,5 @@
 /**
- * $Id: LTglob.c,v 1.6 1998/09/12 15:59:24 rwscott Exp $
+ * $Id: LTglob.c,v 1.10 2000/01/16 17:12:09 amai Exp $
  *
  * derived from GNU libc glob.c
  *
@@ -24,7 +24,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: LTglob.c,v 1.6 1998/09/12 15:59:24 rwscott Exp $";
+static const char rcsid[] = "$Id: LTglob.c,v 1.10 2000/01/16 17:12:09 amai Exp $";
 
 #if defined (_AIX) && !defined (__GNUC__)
 #pragma alloca
@@ -40,11 +40,11 @@ static const char rcsid[] = "$Id: LTglob.c,v 1.6 1998/09/12 15:59:24 rwscott Exp
 #include <stdio.h>
 
 
-#ifdef	STDC_HEADERS
+#ifdef STDC_HEADERS
 #include <stddef.h>
 #endif
 
-#ifdef	HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #ifndef POSIX
 #ifdef	_POSIX_VERSION
@@ -108,10 +108,16 @@ extern int errno;
 #endif
 
 extern void qsort();
-extern void abort(), exit();
 
 #endif /* Standard headers.  */
 
+#ifdef __EMX__   /* -> OS/2 drive letters support */
+#define  INCL_DOS
+#define  INCL_DOSMISC
+#define  INCL_ERRORS
+#include <os2.h>
+#endif
+
 #ifndef	HAVE_STRCOLL
 #define	strcoll	strcmp
 #endif
@@ -155,17 +161,38 @@ extern char *alloca(size_t size_to_allocate);
 
 #endif
 
+#if 0
+/* rws 5 Nov 1998
+   What is all this for??  IMO a link to a directory is still a directory.
+   We shouldn't care whether it is a link or not, therefore I think stat is
+   the correct call here, whether we have lstat or not.  If I'm wrong, write
+   a test and prove it by doing a FileSelectionSearch on a directory with
+   links to directories and comparing the results.
+ */
+
 #ifndef __GNU_LIBRARY__
-#define __lstat lstat
-#ifndef HAVE_LSTAT
-#define lstat stat
+#  define __lstat lstat
+#  ifndef HAVE_LSTAT
+#    define lstat stat
+#  endif
+#  ifdef STAT_MACROS_BROKEN
+#    undef S_ISDIR
+#  endif
+#  ifndef S_ISDIR
+#    define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
+#  endif
 #endif
+
+#else
+
+#define __lstat stat
 #ifdef STAT_MACROS_BROKEN
-#undef S_ISDIR
+#  undef S_ISDIR
 #endif
 #ifndef S_ISDIR
-#define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
+#  define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
 #endif
+
 #endif
 
 #ifndef	STDC_HEADERS
@@ -682,3 +709,44 @@ glob_in_dir(const char *pattern, const char *directory, int flags,
     }
     return GLOB_NOSPACE;
 }
+
+#ifdef __EMX__
+/* Under OS/2 we may need to deal with drive letters ...*/
+
+static char *GetDriveMap(void)
+{
+  /* returns a string containing a list of all drives: e.g. "C: D: E:" */
+
+  static char drivelist[256];   /* so we'll never have a memory leak beyond 256 bytes */
+  static char tmpstr[4];
+  ULONG  CurDrive;
+  ULONG  DriveMap;
+  int    dnum;
+  int    start = 3;
+ 
+  drivelist[0] = '\0';
+  tmpstr[1] = ':';
+  tmpstr[2] = ' ';
+  tmpstr[3] = '\0';
+/*
+  DosError(0);
+*/
+  DosQueryCurrentDisk(&CurDrive, &DriveMap);
+  DriveMap>>=start-1;
+  for (dnum = start; dnum <= 26; dnum++) {
+     if ((DriveMap & (ULONG)1)) {
+        tmpstr[0] = (char)dnum+'A'-1;
+        strcat(drivelist, tmpstr);
+   }
+      DriveMap>>=1; 
+  }
+ 
+  if (drivelist[strlen(drivelist)-1]== ' ')
+    drivelist[strlen(drivelist)-1] = '\0';
+
+/*
+  DosError(1);
+*/
+  return drivelist;
+}
+#endif
