@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Manager.c,v 1.57 1998/10/18 22:01:00 rwscott Exp $
+ * $Id: Manager.c,v 1.83 2000/03/05 12:38:29 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Manager.c,v 1.57 1998/10/18 22:01:00 rwscott Exp $";
+static const char rcsid[] = "$Id: Manager.c,v 1.83 2000/03/05 12:38:29 rwscott Exp $";
 
 #include <LTconfig.h>
 
@@ -58,9 +58,27 @@ static const char rcsid[] = "$Id: Manager.c,v 1.57 1998/10/18 22:01:00 rwscott E
 #include <Xm/RowColumnP.h>
 #include <Xm/DialogS.h>
 
+#if XmVERSION > 1
+/* 
+ * Traits  
+ */
+#include <Xm/TraitP.h>
+#include <Xm/LayoutT.h>
+#include <XmI/DirectionI.h>
+
+static XmDirection get_direction(Widget w);
+
+static XmSpecifyLayoutDirectionTraitRec _XmManagerLayoutTraitRec =
+{
+    0,				/* version */
+    get_direction		/* getValue */
+};
+
+#endif
+
 /* Forward Declarations */
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -75,11 +93,13 @@ static void destroy(Widget w);
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
 
-static XtGeometryResult geometry_manager(Widget w,
-					 XtWidgetGeometry *request,
-					 XtWidgetGeometry *reply);
+/*
+   static XtGeometryResult geometry_manager(Widget w,
+   XtWidgetGeometry *request,
+   XtWidgetGeometry *reply);
 
-static void changed_managed(Widget w);
+   static void changed_managed(Widget w);
+ */
 
 static void constraint_initialize(Widget request, Widget new_w,
 				  ArgList args, Cardinal *num_args);
@@ -88,6 +108,10 @@ static XmNavigability widget_navigable(Widget w);
 
 void _XmManagerInstallAccelerator(Widget m, Widget w, String s);
 
+static void _XmManagerExportX(Widget w, int offset, XtArgVal *value);
+
+static void _XmManagerExportY(Widget w, int offset, XtArgVal *value);
+
 /* action routine prototypes */
 void _XmGadgetButtonMotion(Widget w, XEvent *event,
 			   String *params, Cardinal *num_params);
@@ -149,6 +173,10 @@ void _XmGadgetHelp(Widget w, XEvent *event,
 void _XmGadgetKeyInput(Widget w, XEvent *event,
 		       String *params, Cardinal *num_params);
 
+void _XmGadgetHelp(Widget w, XEvent *event,
+		   String *params, Cardinal *num_params);
+
+
 /*
  * the event handler to handle pointer position related events for gadget
  * children
@@ -307,15 +335,22 @@ static XtResource resources[] =
 	XmRImmediate, (XtPointer)NULL
     }
 
+#if XmVERSION > 1
 #if LESSTIF_BACK_COMPAT
 #else
-#if XmVERSION > 1
-    ,{
+    ,
+    {
 	XmNpopupHandlerCallback, XmCCallback, XmRCallback,
 	sizeof(XtCallbackList), Offset(popup_handler_callback),
 	XtRPointer, (XtPointer)NULL
     }
 #endif
+    ,
+    {
+	XmNlayoutDirection, XmCLayoutDirection, XmRDirection,
+	sizeof(XmDirection), Offset(string_direction),
+	XmRImmediate, (XtPointer)((XmDirection) NULL)	/* dynamic how? */
+    }
 #endif
 };
 
@@ -327,13 +362,13 @@ static XmSyntheticResource syn_resources[] =
     {
 	XmNx,
 	sizeof(Position), COffset(x),
-	_XmFromHorizontalPixels, _XmToHorizontalPixels
+	_XmManagerExportX, _XmToHorizontalPixels
     },
     {
 	XmNy,
 	sizeof(Position),
 	COffset(y),
-	_XmFromVerticalPixels, _XmToVerticalPixels
+	_XmManagerExportY, _XmToVerticalPixels
     },
     {
 	XmNwidth,
@@ -356,6 +391,13 @@ static XmSyntheticResource syn_resources[] =
 	sizeof(Dimension), MOffset(shadow_thickness),
 	_XmFromHorizontalPixels, _XmToHorizontalPixels
     },
+#if XmVERSION > 1
+    {
+	XmNstringDirection,
+	sizeof(XmStringDirection), MOffset(string_direction),
+	_XmFromLayoutDirection, _XmToLayoutDirection
+    },
+#endif
 };
 /* *INDENT-OFF* */
 static XmBaseClassExtRec _XmManagerCoreClassExtRec = {
@@ -392,7 +434,7 @@ static CompositeClassExtensionRec managerCompositeExt =
     /* record_size    */  sizeof(CompositeClassExtensionRec),
     /* accepts_objects */ True,
 #if XtSpecificationRelease >= 6
-    /* allows_change_managed_set */ True
+    /* allows_change_managed_set */ False
 #endif
 };
 
@@ -442,8 +484,8 @@ XmManagerClassRec xmManagerClassRec = {
     },
     /* Composite class part */
     {
-	/* geometry manager */ geometry_manager,
-        /* change_managed   */ changed_managed,
+	/* geometry manager */ NULL /*geometry_manager*/,
+        /* change_managed   */ NULL /*changed_managed*/,
         /* insert_child     */ XtInheritInsertChild,
         /* delete_child     */ XtInheritDeleteChild,
         /* extension        */ (XtPointer)&managerCompositeExt,
@@ -472,10 +514,11 @@ XmManagerClassRec xmManagerClassRec = {
 /* *INDENT-ON* */
 
 
+
 WidgetClass xmManagerWidgetClass = (WidgetClass)&xmManagerClassRec;
 
 static void
-class_initialize()
+class_initialize(void)
 {
     _XmRegisterConverters();
     _XmRegisterPixmapConverters();
@@ -520,11 +563,17 @@ class_part_initialize(WidgetClass widget_class)
 	    ext->record_size = sizeof(CompositeClassExtensionRec);
 	    ext->accepts_objects = True;
 #if XtSpecificationRelease >= 6
-	    ext->allows_change_managed_set = True;
+	    ext->allows_change_managed_set = False;
 #endif
 	    mwc->composite_class.extension = (XtPointer)ext;
 	}
     }
+    /*
+       else
+       {
+       (*extptr)->accepts_objects = True;
+       }
+     */
 
 
     if (widget_class != xmManagerWidgetClass)
@@ -537,7 +586,7 @@ class_part_initialize(WidgetClass widget_class)
 	if (mce && !*mce && sce && *sce)
 	{
 	    *mce =
-	      (XmManagerClassExtRec *)XtCalloc(1, sizeof(XmManagerClassExtRec));
+		(XmManagerClassExtRec *)XtCalloc(1, sizeof(XmManagerClassExtRec));
 	    (*mce)->next_extension = NULL;
 	    (*mce)->record_type = NULLQUARK;
 	    (*mce)->version = XmManagerClassExtVersion;
@@ -567,7 +616,7 @@ class_part_initialize(WidgetClass widget_class)
     _XmInitializeSyntheticResources(mwc->manager_class.syn_resources,
 				    mwc->manager_class.num_syn_resources);
     _XmInitializeSyntheticResources(mwc->manager_class.syn_constraint_resources,
-				    mwc->manager_class.num_syn_constraint_resources);
+			      mwc->manager_class.num_syn_constraint_resources);
 
     if (widget_class != xmManagerWidgetClass)
     {
@@ -584,6 +633,16 @@ class_part_initialize(WidgetClass widget_class)
 			  msc->manager_class.syn_constraint_resources,
 			  msc->manager_class.num_syn_constraint_resources);
     }
+
+#if XmVERSION > 1
+    /* FIX ME: do I need to worry about subclasses */
+    /* initialize traits */
+    if (!XmeTraitSet(xmManagerWidgetClass, XmQTspecifyLayoutDirection,
+		     &_XmManagerLayoutTraitRec))
+    {
+	_XmWarning(NULL, "XmManager ClassInitialize: XmeTraitSet failed\n");
+    }
+#endif
 }
 
 static void
@@ -699,6 +758,7 @@ initialize(Widget request,
 	   Cardinal *num_args)
 {
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)new_w->core.widget_class;
+    /*XmManagerWidget   mw = (XmManagerWidget)new_w; */
 
     MGR_EventHandlerAdded(new_w) = False;
     MGR_HighlightedWidget(new_w) = NULL;
@@ -745,7 +805,7 @@ initialize(Widget request,
     {
 	XtOverrideTranslations(new_w,
 			       (XtTranslations)mwc->
-					manager_class.translations);
+			       manager_class.translations);
     }
 
     /*
@@ -764,13 +824,13 @@ initialize(Widget request,
      */
     if (_XmGetBGPixmapName() != NULL)
     {
-        CoreBackgroundPixmap(new_w) =
-                XmGetPixmapByDepth(XtScreen(new_w),
-                                            _XmGetBGPixmapName(),
-                                            MGR_Foreground(new_w),
-                                            XtBackground(new_w),
-                                            CoreDepth(new_w));
-        _XmClearBGPixmap();
+	CoreBackgroundPixmap(new_w) =
+	    XmGetPixmapByDepth(XtScreen(new_w),
+			       _XmGetBGPixmapName(),
+			       MGR_Foreground(new_w),
+			       XtBackground(new_w),
+			       CoreDepth(new_w));
+	_XmClearBGPixmap();
     }
 
     CreateTopShadowGC(new_w);
@@ -785,6 +845,9 @@ initialize(Widget request,
 static void
 destroy(Widget w)
 {
+    DEBUGOUT(XdbDebug("RWS", w, "%s:destroy(%d)\n",
+		      __FILE__, __LINE__
+	     ));
     XtReleaseGC(w, MGR_TopShadowGC(w));
     XtReleaseGC(w, MGR_BottomShadowGC(w));
     XtReleaseGC(w, MGR_HighlightGC(w));
@@ -804,29 +867,31 @@ destroy(Widget w)
 static void
 realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 {
-    Boolean wz = False, hz = False;
+    /* rws 13 Nov 1998
+       This causes a parse error with gcc on AIX???
+       Boolean wz = False, hz = False; */
 
     DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
-    	__FILE__, __LINE__,
-    	XtWidth(w), XtHeight(w)));
+		      __FILE__, __LINE__,
+		      XtWidth(w), XtHeight(w)));
 
     /*
      * MLM 960524 - if you get an XError: widget has zero width or height.
      */
     if (XtWidth(w) == 0)
     {
-	wz = True;
+	/* wz = True; */
 	XtWidth(w) = 1;
     }
     if (XtHeight(w) == 0)
     {
-	hz = True;
+	/* hz = True; */
 	XtHeight(w) = 1;
     }
 
-    *value_mask |= CWDontPropagate|CWBitGravity;
+    *value_mask |= CWDontPropagate | CWBitGravity;
     attributes->do_not_propagate_mask =
-	(KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask|
+	(KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask |
 	 PointerMotionMask);
     attributes->bit_gravity = NorthWestGravity;
 
@@ -835,15 +900,15 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 
     /* rws 7 Oct 1998
        drawingarea/test7 shows that this does not happen
-    if (wz)
-    {
-	XtWidth(w) = 0;
-    }
-    if (hz)
-    {
-	XtHeight(w) = 0;
-    }
-    */
+       if (wz)
+       {
+       XtWidth(w) = 0;
+       }
+       if (hz)
+       {
+       XtHeight(w) = 0;
+       }
+     */
 }
 
 static Boolean
@@ -857,22 +922,39 @@ set_values(Widget old,
 
     DEBUGOUT(XdbDebug(__FILE__, new_w, "SetValues()\n"));
 
+#if 0
+    if (XtWidth(new_w) != XtWidth(old) && XtWidth(new_w) == 0)
+    {
+	_XmWarning(new_w,
+	 "XmManager SetValues: won't set width of widget to 0 (previous %d)\n",
+		   XtWidth(old));
+	XtWidth(new_w) = XtWidth(old);
+    }
+    if (XtHeight(new_w) != XtHeight(old) && XtHeight(new_w) == 0)
+    {
+	_XmWarning(new_w,
+	"XmManager SetValues: won't set height of widget to 0 (previous %d)\n",
+		   XtHeight(old));
+	XtHeight(new_w) = XtHeight(old);
+    }
+#endif
+
     if (_XmGetBGPixmapName() != NULL)
     {
 	XSetWindowAttributes attr;
 
-        CoreBackgroundPixmap(new_w) =
-                XmGetPixmapByDepth(XtScreen(new_w),
-                                            _XmGetBGPixmapName(),
-                                            MGR_Foreground(new_w),
-                                            XtBackground(new_w),
-                                            CoreDepth(new_w));
+	CoreBackgroundPixmap(new_w) =
+	    XmGetPixmapByDepth(XtScreen(new_w),
+			       _XmGetBGPixmapName(),
+			       MGR_Foreground(new_w),
+			       XtBackground(new_w),
+			       CoreDepth(new_w));
 
 	attr.background_pixmap = CoreBackgroundPixmap(new_w);
 	XChangeWindowAttributes(XtDisplay(new_w), XtWindow(new_w),
-			        CWBackPixmap, &attr);
+				CWBackPixmap, &attr);
 
-        _XmClearBGPixmap();
+	_XmClearBGPixmap();
     }
 
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRUnitType),
@@ -936,6 +1018,7 @@ set_values(Widget old,
     return need_refresh;
 }
 
+#if 0
 static XtGeometryResult
 geometry_manager(Widget w,
 		 XtWidgetGeometry *request,
@@ -983,6 +1066,7 @@ changed_managed(Widget w)
 
     _XmNavigChangeManaged(w);
 }
+#endif
 
 /* constraint class methods */
 
@@ -1036,12 +1120,12 @@ _XmGadgetArm(Widget w,
 	_XmDispatchGadgetInput(g,
 			       event,
 			       XmARM_EVENT);
-	
+
 	MGR_SelectedGadget(w) = (XmGadget)g;
     }
     else
     {
-	if (_XmIsNavigable(w)) 
+	if (_XmIsNavigable(w))
 	{
 	    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 	}
@@ -1104,7 +1188,7 @@ _XmGadgetMultiArm(Widget w,
     }
     else
     {
-	if (_XmIsNavigable(w)) 
+	if (_XmIsNavigable(w))
 	{
 	    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 	}
@@ -1330,6 +1414,15 @@ _XmGadgetHelp(Widget w,
     }
 }
 
+void
+_XmManagerHelp(Widget w,
+	       XEvent *event,
+	       String *params,
+	       Cardinal *num_params)
+{
+    _XmGadgetHelp(w, event, params, num_params);
+}
+
 void
 _XmGadgetKeyInput(Widget w,
 		  XEvent *event,
@@ -1578,6 +1671,10 @@ _XmManagerEventHandler(Widget w,
 void
 _XmDestroyParentCallback(Widget w, XtPointer client, XtPointer call)
 {
+    DEBUGOUT(XdbDebug2("NEDIT", XtParent(w), w,
+		       "%s:_XmDestroyParentCallback(%d)\n",
+		       __FILE__, __LINE__));
+    XtDestroyWidget(XtParent(w));
 }
 
 void
@@ -1591,7 +1688,7 @@ _XmClearShadowType(Widget w,
 	if (XtWidth(w) >= old_width)
 	{
 	    XClearArea(XtDisplay(w), XtWindow(w),
-		       old_width - old_shadow_thickness - old_highlight_thickness,
+		    old_width - old_shadow_thickness - old_highlight_thickness,
 		       0, old_shadow_thickness,
 		       old_height - old_highlight_thickness, False);
 	}
@@ -1599,7 +1696,7 @@ _XmClearShadowType(Widget w,
 	{
 	    XClearArea(XtDisplay(w), XtWindow(w),
 		       0,
-		       old_height - old_shadow_thickness - old_highlight_thickness,
+		   old_height - old_shadow_thickness - old_highlight_thickness,
 		       old_width - old_highlight_thickness,
 		       old_shadow_thickness, False);
 	}
@@ -1647,6 +1744,11 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 		      "state %08x data %08x\n",
 		      event->type, event->xkey.keycode, event->xkey.state,
 		      client_data));
+    DEBUGOUT(XdbDebug("NMEM", w,
+		      "Got Accelerator key: type %d code %04x "
+		      "state %08x data %08x\n",
+		      event->type, event->xkey.keycode, event->xkey.state,
+		      client_data));
 
     if (!XmIsManager(w) ||
 	(event->type != KeyPress && event->type != KeyRelease))
@@ -1657,9 +1759,46 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
     }
 
     DEBUGOUT(XdbDebug(__FILE__, w,
-		      "Got Accelerator key: code: %08x state: %08x cd %08x %s\n",
-		      event->xkey.keycode, event->xkey.state, client_data,
-		      XtName(client_data)));
+		    "Got Accelerator key: code: %08x state: %08x cd %08x %s\n",
+		      event->xkey.keycode, event->xkey.state, comp,
+		      XtName(comp)));
+    DEBUGOUT(XdbDebug("NMEM", w,
+		    "Got Accelerator key: code: %08x state: %08x cd %08x %s\n",
+		      event->xkey.keycode, event->xkey.state, comp,
+		      XtName(comp)));
+
+    /* JAC added this test.  Occasionally in nedit the accelerators cause
+       segfaults.  The comp value passed has 0 for all fields in the
+       Widget.  e.g.:
+
+       (gdb) p *comp
+
+       $10 = {core = {self = 0x0, widget_class = 0x0, parent = 0x0,
+       xrm_name = 0, being_destroyed = 0 '\000', destroy_callbacks = 0x0,
+       constraints = 0x0, x = 0, y = 0, width = 0, height = 0,
+       border_width = 0, managed = 0 '\000', sensitive = 0 '\000',
+       ancestor_sensitive = 0 '\000', event_table = 0x0, tm = {
+       translations = 0x0, proc_table = 0x0, current_state = 0x0,
+       lastEventTime = 0}, accelerators = 0x0, border_pixel = 0,
+       border_pixmap = 0, popup_list = 0x0, num_popups = 0, name = 0x0,
+       screen = 0x0, colormap = 0, window = 0, depth = 0, background_pixel
+       = 0, background_pixmap = 0, visible = 0 '\000', mapped_when_managed
+       = 0 '\000'}} 
+
+       I suspect that "comp" may be the address of a widget that's been
+       destroyed, and for some reason we're still getting the callback,
+       but that's a WAG.
+     */
+    if (!comp->core.widget_class)
+    {
+	_XmWarning(NULL, "XmManager _XmAcceleratorHandler: bad news!!!\n");
+	DEBUGOUT(XdbDebug(__FILE__, w,
+			  "_XmAcceleratorHandler: called for w=%s"
+			  " with invalid comp, ignoring\n",
+			  XtName(w)));
+	return;
+
+    }
 
     /*
      * Insensitive widgets shouldn't be activated by accelerators or
@@ -1669,22 +1808,22 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
      * This test could go inside the following search loop but there's
      * no point in searching when we know that we can't do anything with
      * what we find.
-     *         --mu@trends.net, 1998.09.06
-     */
-    if(!XtIsSensitive(comp))
+     * --mu@trends.net, 1998.09.06 */
+    if (!XtIsSensitive(comp))
     {
-       DEBUGOUT(XdbDebug(__FILE__, w,
-                         "_XmAcceleratorHandler: %s is insensitive, ignoring",
-                         XtName(comp)));
-       return;
+	DEBUGOUT(XdbDebug(__FILE__, w,
+			"_XmAcceleratorHandler: %s is insensitive, ignoring\n",
+			  XtName(comp)));
+	return;
     }
 
     for (i = 0; i < MGR_NumKeyboardEntries(w); i++)
     {
-	if (MGR_KeyboardList(w)[i].eventType == event->type &&
+	if (/*MGR_KeyboardList(w)[i].eventType == event->type && */
 	    MGR_KeyboardList(w)[i].key == event->xkey.keycode &&
 	    MGR_KeyboardList(w)[i].modifiers == event->xkey.state &&
-	    MGR_KeyboardList(w)[i].component == client_data)
+	    MGR_KeyboardList(w)[i].component == comp &&
+	    XtIsManaged(comp))
 	{
 	    Cardinal num_params = 0;
 
@@ -1693,42 +1832,70 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 			      "code %08x state %08x name: %s:%s mnem: %d\n",
 			      event->xkey.keycode, event->xkey.state,
 			      comp
-				? comp->core.widget_class->core_class.class_name
-				: "(nil)",
+			      ? comp->core.widget_class->core_class.class_name
+			      : "(nil)",
+			      comp ? XtName(comp) : "(nil)",
+			      MGR_KeyboardList(w)[i].isMnemonic));
+	    DEBUGOUT(XdbDebug("NMEM", w,
+			      "MATCH!!!! "
+			      "code %08x state %08x name: %s:%s mnem: %d\n",
+			      event->xkey.keycode, event->xkey.state,
+			      comp
+			      ? comp->core.widget_class->core_class.class_name
+			      : "(nil)",
 			      comp ? XtName(comp) : "(nil)",
 			      MGR_KeyboardList(w)[i].isMnemonic));
 
 	    /* Make sure XmGetPostedFromWidget works: ? */
 
-	    if (XmIsPrimitive(MGR_KeyboardList(w)[i].component))
+	    if (!MGR_KeyboardList(w)[i].isMnemonic || RC_Type(XtParent(comp)) == XmMENU_BAR ||
+		(MGR_KeyboardList(w)[i].isMnemonic && XtParent(comp) == RC_PopupPosted(w))
+		)
 	    {
-		act = PrimC_ArmAndActivate(XtClass(comp));
+		DEBUGOUT(XdbDebug2(__FILE__, w, MGR_KeyboardList(w)[i].component,
+				   "menu showing\n"));
+		DEBUGOUT(XdbDebug2("NMEM", w, MGR_KeyboardList(w)[i].component,
+				   "menu showing\n"));
+		if (XmIsPrimitive(MGR_KeyboardList(w)[i].component))
+		{
+		    act = PrimC_ArmAndActivate(XtClass(comp));
 
-		(*act)(comp, event, NULL, &num_params);
-	    }
-	    else if (XmIsGadget(MGR_KeyboardList(w)[i].component))
-	    {
-		act = GC_ArmAndActivate(XtClass(comp));
+		    (*act) (comp, event, NULL, &num_params);
+		}
+		else if (XmIsGadget(MGR_KeyboardList(w)[i].component))
+		{
+		    act = GC_ArmAndActivate(XtClass(comp));
 
-		(*act)(comp, event, NULL, &num_params);
-	    }
-	    else if (XmIsRowColumn(w))
-	    {
-		act = RCClass_ArmAndActivate(XtClass(comp));
+		    (*act) (comp, event, NULL, &num_params);
+		}
+		else if (XmIsRowColumn(w))
+		{
+		    act = RCClass_ArmAndActivate(XtClass(comp));
 
-		(*act)(comp, event, NULL, &num_params);
+		    (*act) (comp, event, NULL, &num_params);
+		}
+		else
+		{
+		    DEBUGOUT(XdbDebug(__FILE__, MGR_KeyboardList(w)[i].component,
+				      "Not a widget/gadget\n"));
+
+		    XtCallActionProc(comp, "Activate", event, NULL, 0);
+		}
+		*cont = False;
+		break;
 	    }
 	    else
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, MGR_KeyboardList(w)[i].component,
-				  "Not a widget/gadget\n"));
-
-		XtCallActionProc(comp, "Activate", event, NULL, 0);
+		DEBUGOUT(XdbDebug2(__FILE__, w, MGR_KeyboardList(w)[i].component,
+				   "menu is not showing\n"));
+		DEBUGOUT(XdbDebug2("NMEM", w, MGR_KeyboardList(w)[i].component,
+				   "menu is not showing\n"));
 	    }
 
+	    /*
 	    *cont = False;
-
 	    break;
+	    */
 	}
     }
 }
@@ -1781,10 +1948,17 @@ AddKeyboardEntry(Widget m, Widget w, XmKeyboardData *item)
     MGR_NumKeyboardEntries(m)++;
 
     DEBUGOUT(XdbDebug(__FILE__, m,
-		      "GrabKey: key %04x Sym: %04x modifiers %08x\n",
-		      item->key, item->keysym, item->modifiers));
+		      "GrabKey: key %04x Sym: %04x modifiers %08x %s %s\n",
+		      item->key, item->keysym, item->modifiers,
+		      item->needGrab ? "Grabbing" : "Not Grabbing",
+		      item->isMnemonic ? "mnemonic" : "not mnemonic"));
+    DEBUGOUT(XdbDebug("NMEM", m,
+		      "GrabKey: key %04x Sym: %04x modifiers %08x %s %s\n",
+		      item->key, item->keysym, item->modifiers,
+		      item->needGrab ? "Grabbing" : "Not Grabbing",
+		      item->isMnemonic ? "mnemonic" : "not mnemonic"));
 
-    XtInsertEventHandler(m, KeyPressMask|KeyReleaseMask,
+    XtInsertEventHandler(m, item->eventType /*KeyPressMask|KeyReleaseMask */ ,
 			 False, _XmAcceleratorHandler,
 			 (XtPointer)w, XtListHead);
 
@@ -1827,7 +2001,7 @@ DeleteKeyboardEntry(Widget m, Widget w, Boolean isMnemonic)
 		bcopy(&MGR_KeyboardList(m)[i + 1],
 		      &MGR_KeyboardList(m)[i],
 		      sizeof(XmKeyboardData) *
-			(MGR_NumKeyboardEntries(m) - i - 1));
+		        (MGR_NumKeyboardEntries(m) - i - 1));
 
 		bzero(&MGR_KeyboardList(m)[MGR_NumKeyboardEntries(m) - 1],
 		      sizeof(XmKeyboardData));
@@ -1855,6 +2029,8 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
 
     DEBUGOUT(XdbDebug2(__FILE__, m, w,
 		       "_XmManagerInstallAccelerator(%s)\n", s));
+    DEBUGOUT(XdbDebug2("NMEM", m, w,
+		       "_XmManagerInstallAccelerator(%s)\n", s));
 
     /* Do we really have an accelerator ? */
     if (s == NULL || strlen(s) == 0)
@@ -1879,12 +2055,19 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
 
 	if (XtIsSubclass(sh, applicationShellWidgetClass) ||
 	    XtIsSubclass(sh, xmDialogShellWidgetClass) ||
+	    XtIsSubclass(sh, transientShellWidgetClass) ||
 	    XtIsSubclass(sh, topLevelShellWidgetClass))
 	{
 	    break;
 	}
     }
 
+    if (!XmIsManager(mw))
+    {
+	return;
+    }
+
+
     DEBUGOUT(XdbDebug(__FILE__, w,
 		      "_XmManagerInstallAccelerator found manager %s\n",
 		      XtName(mw)));
@@ -1900,8 +2083,31 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
     data.key = XKeysymToKeycode(XtDisplay(w), data.keysym);
     data.modifiers = modifiers;
     data.needGrab = True;
+    if (data.key == 0)
+    {
+#if 1
+    KeySym real_sym;
+    Modifiers real_mod;
 
-    AddKeyboardEntry(mw, w, &data);
+	_XmVirtualToActualKeysym(XtDisplay(w), data.keysym, &real_sym, &real_mod);
+	if (real_sym == NoSymbol && real_mod == (Modifiers)0)
+	{
+	    _XmWarning(m, "%s:_XmManagerInstallAccelerator(%d) - Could not convert >%s< to a KeyCode\n    %s -> %08x -> %i", 
+		__FILE__, __LINE__, s,
+		s, data.keysym, data.key);
+	}
+	else
+	{
+	    data.key = XKeysymToKeycode(XtDisplay(w), real_sym);
+	    data.modifiers = real_mod;
+	    AddKeyboardEntry(mw, w, &data);
+	}
+#endif
+    }
+    else
+    {
+	AddKeyboardEntry(mw, w, &data);
+    }
 }
 
 void
@@ -1915,6 +2121,9 @@ _XmManagerUninstallAccelerator(Widget m, Widget w)
 	sh = XtParent(mw);
 	if (XtIsSubclass(sh, applicationShellWidgetClass) ||
 	    XtIsSubclass(sh, xmDialogShellWidgetClass) ||
+	    XtIsSubclass(sh, transientShellWidgetClass) ||
+		XtIsSubclass(sh, transientShellWidgetClass) ||
+		XtIsSubclass(sh, transientShellWidgetClass) ||
 	    XtIsSubclass(sh, topLevelShellWidgetClass))
 	{
 	    break;
@@ -1942,31 +2151,40 @@ _XmManagerInstallMnemonic(Widget m, Widget w, KeySym mn)
     }
 
     DEBUGOUT(XdbDebug2(__FILE__, m, w, "_XmManagerInstallMnemonic(%c)\n", mn));
+    DEBUGOUT(XdbDebug2("NMEM", m, w, "_XmManagerInstallMnemonic(%c)\n", mn));
 
     /* If we're dealing with the menu bar, store the stuff in the highest
      * manager widget that you can find (with all the accelerators) */
     if (XmIsRowColumn(m) &&
-	(RC_Type(m) == XmMENU_BAR || RC_Type(m) == XmMENU_OPTION))
+	(RC_Type(m) == XmMENU_PULLDOWN || RC_Type(m) == XmMENU_BAR || RC_Type(m) == XmMENU_OPTION))
     {
 	Widget mw, sh;
 	XmModifierMaskSetReference mods =
-		_XmGetModifierMappingsForDisplay(XtDisplay(w));
+	_XmGetModifierMappingsForDisplay(XtDisplay(w));
 
-	need_grab = True;
-	modifiers |= mods[ALTModifier];
+	if (RC_Type(m) == XmMENU_PULLDOWN)
+	{
+	}
+	else
+	{
+	    need_grab = True;
+	    modifiers |= mods[ALTModifier];
+	}
 
 	for (mw = m; XtParent(mw); mw = sh)
 	{
 	    sh = XtParent(mw);
 	    if (XtIsSubclass(sh, applicationShellWidgetClass) ||
 		XtIsSubclass(sh, xmDialogShellWidgetClass) ||
-		XtIsSubclass(sh, topLevelShellWidgetClass))
+		XtIsSubclass(sh, topLevelShellWidgetClass) ||
+		(RC_Type(m) != XmMENU_BAR && XmIsRowColumn(mw) && RC_Type(mw) == XmMENU_BAR))
 	    {
 		break;
 	    }
 	}
 
 	DEBUGOUT(XdbDebug2(__FILE__, mw, w, "Store mnemonic info\n"));
+	DEBUGOUT(XdbDebug2("NMEM", mw, w, "Store mnemonic info\n"));
 	m = mw;
     }
 
@@ -1990,7 +2208,8 @@ _XmManagerUninstallMnemonic(Widget m, Widget w)
 {
     Widget mw, sh;
 
-    if (XmIsRowColumn(m) && RC_Type(m) == XmMENU_BAR)
+    if (XmIsRowColumn(m) &&
+	(RC_Type(m) == XmMENU_PULLDOWN || RC_Type(m) == XmMENU_BAR || RC_Type(m) == XmMENU_OPTION))
     {
 	/* Find the manager widget just underneath the shell */
 	for (mw = m; XtParent(mw); mw = sh)
@@ -1998,7 +2217,8 @@ _XmManagerUninstallMnemonic(Widget m, Widget w)
 	    sh = XtParent(mw);
 	    if (XtIsSubclass(sh, applicationShellWidgetClass) ||
 		XtIsSubclass(sh, xmDialogShellWidgetClass) ||
-		XtIsSubclass(sh, topLevelShellWidgetClass))
+		XtIsSubclass(sh, topLevelShellWidgetClass) ||
+		(RC_Type(m) != XmMENU_BAR && XmIsRowColumn(mw) && RC_Type(mw) == XmMENU_BAR))
 	    {
 		break;
 	    }
@@ -2045,42 +2265,95 @@ widget_navigable(Widget w)
 void
 _XmManagerHighlightPixmapDefault(Widget w, int offset, XrmValue *value)
 {
-	static Pixmap pixmap;  
+    static Pixmap pixmap;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerHighlightPixmapDefault\n"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerHighlightPixmapDefault\n"));
 
-	pixmap = XmUNSPECIFIED_PIXMAP;
+    pixmap = XmUNSPECIFIED_PIXMAP;
 
-	value->addr = (char *) &pixmap;
-	value->size = sizeof (Pixmap);
+    value->addr = (char *)&pixmap;
+    value->size = sizeof(Pixmap);
 
-	if (MGR_HighlightColor(w) == XtBackground(w))
-		pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
-			MGR_HighlightColor(w),
-			MGR_Foreground(w),
-			CoreDepth(w));
+    if (MGR_HighlightColor(w) == XtBackground(w))
+	pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
+				    MGR_HighlightColor(w),
+				    MGR_Foreground(w),
+				    CoreDepth(w));
 }
 
 void
 _XmManagerTopShadowPixmapDefault(Widget w, int offset, XrmValue *value)
 {
-	static Pixmap pixmap;
+    static Pixmap pixmap;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerTopShadowPixmapDefault\n"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerTopShadowPixmapDefault\n"));
 
-	pixmap = XmUNSPECIFIED_PIXMAP;
+    pixmap = XmUNSPECIFIED_PIXMAP;
 
-	value->addr = (char *) &pixmap;
-	value->size = sizeof (Pixmap);
+    value->addr = (char *)&pixmap;
+    value->size = sizeof(Pixmap);
 
-	if (MGR_TopShadowColor(w) == XtBackground(w))
-		pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
-			MGR_TopShadowColor(w),
-			MGR_Foreground(w),
-			CoreDepth(w));
-	else if (DefaultDepthOfScreen(XtScreen(w)) == 1)
-		pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
-			MGR_TopShadowColor(w),
-			XtBackground(w),
-			CoreDepth(w));
+    if (MGR_TopShadowColor(w) == XtBackground(w))
+	pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
+				    MGR_TopShadowColor(w),
+				    MGR_Foreground(w),
+				    CoreDepth(w));
+    else if (DefaultDepthOfScreen(XtScreen(w)) == 1)
+	pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
+				    MGR_TopShadowColor(w),
+				    XtBackground(w),
+				    CoreDepth(w));
 }
+
+static void
+_XmManagerExportX(Widget widget, int offset, XtArgVal *value)
+{
+    unsigned char unitType = _XmGetUnitType(widget);
+    int converted_value;
+
+    if (XtIsManaged(widget) && XtParent(widget) && XmIsVendorShell(XtParent(widget)))
+    {
+	*value = XtX(XtParent(widget));
+    }
+    converted_value = XmConvertUnits(widget,
+				     XmHORIZONTAL,
+				     XmPIXELS,
+				     *value,
+				     unitType);
+
+    *value = converted_value;
+}
+
+static void
+_XmManagerExportY(Widget widget, int offset, XtArgVal *value)
+{
+    unsigned char unitType = _XmGetUnitType(widget);
+    int converted_value;
+
+    if (XtIsManaged(widget) && XtParent(widget) && XmIsVendorShell(XtParent(widget)))
+    {
+	*value = XtY(XtParent(widget));
+    }
+    converted_value = XmConvertUnits(widget,
+				     XmVERTICAL,
+				     XmPIXELS,
+				     *value,
+				     unitType);
+
+    *value = converted_value;
+}
+
+#if XmVERSION > 1
+static XmDirection 
+get_direction(Widget w)
+{
+    /* 
+       This macro obtains the layout direction resource even
+       though it looks like the string direction resource. 
+       In 2.0 layout direction replaced string direction
+       but the identifier in the internal data structure stayed
+       the same.  Everyone still with me. 8)
+     */
+    return MGR_StringDirection(w);
+}
+#endif
