@@ -1,6 +1,6 @@
 /**
  *
- * $Id: RepType.c,v 1.8 1998/10/16 04:49:28 gritton Exp $
+ * $Id: RepType.c,v 1.11 2000/01/21 16:04:22 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -21,11 +21,12 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  **/
-static const char rcsid[] = "$Id: RepType.c,v 1.8 1998/10/16 04:49:28 gritton Exp $";
+static const char rcsid[] = "$Id: RepType.c,v 1.11 2000/01/21 16:04:22 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
 
+#include <Xm/Xm.h>
 #include <Xm/XmP.h>
 #include <Xm/RepType.h>
 
@@ -246,13 +247,13 @@ __XmCvtRepToString(Display *display,
     {
 	String params[2];
 	Cardinal numParams = 2;
-	char ASCIIValue[10];
+	char ASCIIValue[4];
 
     error:
 	value = entry->value_names[0];
-	sprintf(ASCIIValue, "%i", (int)from->addr);
-	params[0] = (String)ASCIIValue;
-	params[1] = (String)from->addr;
+	sprintf(ASCIIValue, "%i", *(unsigned char *)from->addr);
+	params[0] = ASCIIValue;
+	params[1] = entry->rep_type_name;
 	XtAppWarningMsg(XtDisplayToApplicationContext(display),
 			"conversionError", entry->rep_type_name,
 			"XtToolkitError",
@@ -350,7 +351,7 @@ XmRepTypeAddReverse(XmRepTypeId rep_type_id)
 	entry->reverse_installed = True;
 
 	Args[0].address_mode = XtImmediate;
-	Args[0].address_id = (XtPointer)(int)entry->rep_type_id;
+	Args[0].address_id = (XtPointer)(long)entry->rep_type_id;
 	Args[0].size = sizeof(XmRepTypeId);
 
 	XtSetTypeConverter(entry->rep_type_name, XmRString,
@@ -444,7 +445,7 @@ XmRepTypeGetRecord(XmRepTypeId rep_type_id)
 }
 
 XmRepTypeList
-XmRepTypeGetRegistered()
+XmRepTypeGetRegistered(void)
 {
     int i, strlens, num_values;
     char *cp;
@@ -496,7 +497,7 @@ XmRepTypeGetRegistered()
 }
 
 void
-XmRepTypeInstallTearOffModelConverter()
+XmRepTypeInstallTearOffModelConverter(void)
 {
     static char *tear_off_models[] =
     {
@@ -604,7 +605,8 @@ XmRepTypeValidValue(XmRepTypeId rep_type_id,
 
     if (enable_default_warning)
 	_XmWarning(enable_default_warning,
-		   "XmRepTypeValidValue: invalid type.\n");
+		   "illegal value (%d) for rep type %s",
+		   test_value, entry->rep_type_name);
 
     return False;
 }
