@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ResConvert.c,v 1.57 2000/03/02 21:01:08 amai Exp $
+ * $Id: ResConvert.c,v 1.58 2000/05/23 22:05:15 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ResConvert.c,v 1.57 2000/03/02 21:01:08 amai Exp $";
+static const char rcsid[] = "$Id: ResConvert.c,v 1.58 2000/05/23 22:05:15 danny Exp $";
 
 #include <LTconfig.h>
 
@@ -281,6 +281,11 @@ static char *unit_types[] =
     "100th_font_units"
 };
 
+/*
+ * These are the values of XmNdialogType for XmMessageBox
+ *	This is the value type XmRDialogType.
+ *	The class of this resource is XmCDialogType.
+ */
 static char *dialog_types[] =
 {
     "dialog_template",
@@ -292,6 +297,19 @@ static char *dialog_types[] =
     "dialog_working"
 };
 
+/*
+ * These are the values of XmNdialogType for XmSelectionBox
+ *	This is the value type XmRSelectionType.
+ *	The class of this resource is (also) XmCDialogType
+ */
+static char *selection_dialog_types[] = {
+	"dialog_work_area",
+	"dialog_prompt",
+	"dialog_selection",
+	"dialog_command",
+	"dialog_file_selection",
+};
+
 static char *dialog_styles[] =
 {
     "dialog_modeless",
@@ -692,7 +710,7 @@ _XmRegisterConverters(void)
     REPTYPE_REGISTER(XmRDragReceiverProtocolStyle, protocol_styles);
     REPTYPE_REGISTER(XmRUnitType, unit_types);
     REPTYPE_REGISTER(XmRDialogType, dialog_types);
-    REPTYPE_REGISTER(XmRSelectionType, dialog_types);
+    REPTYPE_REGISTER(XmRSelectionType, selection_dialog_types);
     REPTYPE_REGISTER(XmRDialogStyle, dialog_styles);
     REPTYPE_REGISTER(XmRSeparatorType, separator_types);
     REPTYPE_REGISTER(XmRResizePolicy, resize_policies);
