@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Scale.c,v 1.38 2000/01/21 16:04:22 amai Exp $
+ * $Id: Scale.c,v 1.39 2000/05/30 19:53:12 hennessy Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Scale.c,v 1.38 2000/01/21 16:04:22 amai Exp $";
+static const char rcsid[] = "$Id: Scale.c,v 1.39 2000/05/30 19:53:12 hennessy Exp $";
 
 #include <LTconfig.h>
 #include <stdlib.h>
@@ -722,8 +722,12 @@ set_values(Widget old, Widget request, Widget new_w,
 	Scale_ScaleHeight(new_w) != Scale_ScaleHeight(old))
     {
 	argc = 0;
-	XtSetArg(argl[argc], XmNwidth, Scale_ScaleWidth(new_w)); argc++;
-	XtSetArg(argl[argc], XmNheight, Scale_ScaleHeight(new_w)); argc++;
+	if (Scale_ScaleWidth(new_w) != Scale_ScaleWidth(old) ) {
+	    XtSetArg(argl[argc], XmNwidth, Scale_ScaleWidth(new_w)); argc++;
+	}
+	if ( Scale_ScaleHeight(new_w) != Scale_ScaleHeight(old) ) {
+	    XtSetArg(argl[argc], XmNheight, Scale_ScaleHeight(new_w)); argc++;
+	}
 
 	XtSetValues(sb, argl, argc);
        
