@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Simple.c,v 1.15 1998/10/04 07:23:16 gritton Exp $
+ * $Id: Simple.c,v 1.22 1999/12/01 22:19:21 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Simple.c,v 1.15 1998/10/04 07:23:16 gritton Exp $";
+static const char rcsid[] = "$Id: Simple.c,v 1.22 1999/12/01 22:19:21 rwscott Exp $";
 
 #include <stdio.h>
 
@@ -41,6 +41,11 @@ static const char rcsid[] = "$Id: Simple.c,v 1.15 1998/10/04 07:23:16 gritton Ex
 
 #include <XmI/DebugUtil.h>
 
+/* For the prototype police */
+extern Widget _XmCreateSimpleGadget(char *name, Widget parent, int def_type,
+				    XmSimpleMenu data, int which,
+				    int call_data, ArgList args, int argc);
+
 /*
  * I should have done this in the first place.  The old implementation wouldn't
  * catch resources specified outside of the create call.
@@ -217,13 +222,13 @@ _XmCreateSimpleGadget(char *name, Widget parent, int def_type,
 	if (bclass == xmToggleButtonGadgetClass)
 	{
 	    XtAddCallback(ret, XmNvalueChangedCallback,
-			  data->callback, (XtPointer)call_data);
+			  data->callback, (XtPointer)(long)call_data);
 	}
 	else if (bclass == xmPushButtonGadgetClass ||
 		 bclass == xmCascadeButtonGadgetClass)
 	{
 	    XtAddCallback(ret, XmNactivateCallback,
-			  data->callback, (XtPointer)call_data);
+			  data->callback, (XtPointer)(long)call_data);
 	}
     }
 
@@ -243,10 +248,12 @@ XmCreateSimpleCheckBox(Widget parent,
     ArgList combined;
     XmSimpleMenuRec data;
 
+#ifdef	SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     bzero((void *)&data, sizeof(XmSimpleMenuRec));
 
@@ -297,10 +304,12 @@ XmCreateSimpleMenuBar(Widget parent,
     ArgList combined;
     XmSimpleMenuRec data;
 
+#ifdef	SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     bzero((void *)&data, sizeof(XmSimpleMenuRec));
 
@@ -350,12 +359,15 @@ XmCreateSimpleOptionMenu(Widget parent,
     XmSimpleMenuRec data;
     int scnt = 0;
     int bcnt = 0;
+    int lcnt = 0;
 
 
+#ifdef	SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     bzero((void *)&data, sizeof(XmSimpleMenuRec));
 
@@ -399,20 +411,35 @@ XmCreateSimpleOptionMenu(Widget parent,
 		scnt++;
 		break;
 
+	    case XmTITLE:
+		sprintf(buf, "label_%d", lcnt);
+		lcnt++;
+		break;
+
 	    case XmCASCADEBUTTON:
 	    case XmPUSHBUTTON:
 	    default:
-		sprintf(buf, "button_%d", i);
+		sprintf(buf, "button_%d", bcnt);
 		bcnt++;
 		break;
 	    }
-	    _XmCreateSimpleGadget(buf, RC_OptionSubMenu(rc),
-				  data.button_type[i],
-				  &data, i, bcnt - 1, arglist, argcount);
+	    {
+	    Widget tmp;
+
+		tmp = _XmCreateSimpleGadget(buf, RC_OptionSubMenu(rc),
+				      data.button_type[i],
+				      &data, i, bcnt - 1, arglist, argcount);
+		if (data.button_type[i] == XmPUSHBUTTON && data.button_set == bcnt - 1)
+		{
+		    XtVaSetValues(rc,
+			    XmNmenuHistory, tmp,
+			    NULL);
+		}
+	    }
 	}
 	else
 	{
-	    sprintf(buf, "button_%d", i);
+	    sprintf(buf, "button_%d", bcnt);
 	    bcnt++;
 	    if (RC_Type(rc) == XmMENU_BAR)
 	    {
@@ -432,8 +459,18 @@ XmCreateSimpleOptionMenu(Widget parent,
     /* set the default button */
     if (data.button_set >= 0 && data.button_set < data.count)
     {
-	XtVaSetValues(button, XmNlabelString,
+	/* rws 12 Aug 1999
+	   Don't set the label if the value is NULL, just pick it up from
+	   the resource database by default.
+	   Xquote Options->Updates...
+	 */
+	/*
+	if (data.label_string[data.button_set])
+	{
+	    XtVaSetValues(button, XmNlabelString,
 		      data.label_string[data.button_set], NULL);
+	}
+	*/
     }
 
     return rc;
@@ -453,11 +490,12 @@ XmCreateSimplePopupMenu(Widget parent,
     int tcnt = 0;
     int bcnt = 0;
 
-
+#ifdef	SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     bzero((void *)&data, sizeof(XmSimpleMenuRec));
 
@@ -525,10 +563,12 @@ XmCreateSimplePulldownMenu(Widget parent,
     int tcnt = 0;
     int bcnt = 0;
 
+#ifdef	SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     bzero((void *)&data, sizeof(XmSimpleMenuRec));
 
@@ -544,6 +584,13 @@ XmCreateSimplePulldownMenu(Widget parent,
 		  XmNchildren, &wlist,
 		  NULL);
 
+#if 0
+    /* rws 25 Jan 1999
+       vasimple/test5 shows that this is not correct.  This would only work
+       if the menu that this menu is being posted from is also a simple type
+       menu.  In this test it is not, and therefore will not find a button
+       of the correct name.
+     */
     sprintf(buf, "button_%d", data.post_from_button);
     for (i = 0; i < num_children; i++)
     {
@@ -567,6 +614,36 @@ XmCreateSimplePulldownMenu(Widget parent,
 	/* add the new pulldown menu to the cascade button */
 	XtVaSetValues(wlist[i], XmNsubMenuId, rc, NULL);
     }
+#else
+    {
+    int count = 0;
+
+	for (i = 0; i < num_children; i++)
+	{
+	    if (!XmIsTearOffButton(wlist[i]) && 
+	         (XmIsPushButton(wlist[i]) ||
+	          XmIsPushButtonGadget(wlist[i]) ||
+	          XmIsToggleButton(wlist[i]) ||
+	          XmIsToggleButtonGadget(wlist[i]) ||
+	          XmIsCascadeButton(wlist[i]) ||
+	          XmIsCascadeButtonGadget(wlist[i])))
+	    {
+		if (count == data.post_from_button)
+		{
+		    break;
+		}
+		count++;
+	    }
+	}
+	if (i != num_children)
+	{
+	    XtVaSetValues(wlist[i], XmNsubMenuId, rc, NULL);
+	}
+	else
+	{
+	}
+    }
+#endif
 
     /* create the buttons in the menu */
 
@@ -625,10 +702,12 @@ XmCreateSimpleRadioBox(Widget parent,
     XmSimpleMenuRec data;
     Widget tgl;
 
+#ifdef	SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     bzero((void *)&data, sizeof(XmSimpleMenuRec));
 
