@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/Text.c,v 1.120 2000/03/11 13:44:26 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/Text.c,v 1.123 2000/05/20 15:46:14 hennessy Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Text.c,v 1.120 2000/03/11 13:44:26 rwscott Exp $";
+static const char rcsid[] = "$Id: Text.c,v 1.123 2000/05/20 15:46:14 hennessy Exp $";
 
 #undef VERBOSE
 
@@ -986,8 +986,7 @@ Redisplay(XmTextWidget w)
     Text_InRedisplay(w) = False;
 }
 
-static
-Cardinal
+static Cardinal
 GetSecResData(WidgetClass wc, XmSecondaryResourceData **resdata)
 {
     return 0;
@@ -1586,7 +1585,8 @@ _XmTextInsert(Widget w, XmTextPosition position, char *string, XEvent *evp)
     block.ptr = string;
     block.length = strlen(string);
     block.format = XmFMT_8_BIT;
-
+    if ( position > Text_LastPos(w) )
+	position = Text_LastPos(w) ;                                                                                                                                                                                                                                      
     startret = position;
     endret = position;
 
@@ -2578,7 +2578,7 @@ XmTextSetHighlight(Widget w, XmTextPosition left, XmTextPosition right, XmHighli
     if (XtIsSubclass(w, xmTextWidgetClass))
     {
 	int		i, j, k, l;
-	XmHighlightMode m;
+	XmHighlightMode lm, rm;
 
 #if 0
 	fprintf(stderr,
@@ -2605,11 +2605,24 @@ XmTextSetHighlight(Widget w, XmTextPosition left, XmTextPosition right, XmHighli
 		;
 	if (i == 0)
 	{
-	    m = XmHIGHLIGHT_NORMAL;
+	    /* assert(left==0); */
+	    lm = XmHIGHLIGHT_NORMAL;
 	}
 	else
 	{
-	    m = Text_Highlight(w).list[i-1].mode;
+	    lm = Text_Highlight(w).list[i-1].mode;
+	}
+	for (j=i; j < Text_Highlight(w).number &&
+			Text_Highlight(w).list[j].position < right; j++)
+		;
+	if (j == 0)
+	{
+	    /* assert(right==0); */
+	    rm = XmHIGHLIGHT_NORMAL;
+	}
+	else
+	{
+	    rm = Text_Highlight(w).list[j-1].mode;
 	}
 	if (Text_Highlight(w).list[i].position == left) 
 	{
@@ -2623,7 +2636,7 @@ XmTextSetHighlight(Widget w, XmTextPosition left, XmTextPosition right, XmHighli
 	else 
 	{
 	    /* Insert an entry for the left */
-	    if (Text_Highlight(w).list[i].mode != mode) 
+	    if (lm != mode) 
 	    {
 		/* Need one more entry */
 		/* Make sure there's an empty space */
@@ -2661,7 +2674,7 @@ XmTextSetHighlight(Widget w, XmTextPosition left, XmTextPosition right, XmHighli
 	}
 	/* Now this is hilarious - insert one again */
 	i++;
-	if (Text_Highlight(w).list[i].mode != mode) 
+	if (rm != mode) 
 	{
 	    if (Text_Highlight(w).number == Text_Highlight(w).maximum) 
 	    {
@@ -2675,7 +2688,7 @@ XmTextSetHighlight(Widget w, XmTextPosition left, XmTextPosition right, XmHighli
 		Text_Highlight(w).list[k] = Text_Highlight(w).list[k-1];
 	    }
 	    Text_Highlight(w).number++;
-	    Text_Highlight(w).list[i].mode = m;
+	    Text_Highlight(w).list[i].mode = rm;
 	    Text_Highlight(w).list[i].position = right;
 	}
 	/* End Highlighting */
