@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.56 2000/04/08 14:54:51 hennessy Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.57 2000/04/18 19:35:40 hennessy Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.56 2000/04/08 14:54:51 hennessy Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.57 2000/04/18 19:35:40 hennessy Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -850,7 +850,7 @@ DoScanType(XmTextWidget w, XEvent *ev, XmTextPosition pos)
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 			"DoScanType: XmSELECT_ALL (left %d pos %d right %d)\n",
 			  left, pos, right));
-
+ 
 	XmTextSetSelection((Widget)w, left, right, ev->xbutton.time);
 
 	break;
@@ -862,7 +862,13 @@ DoScanType(XmTextWidget w, XEvent *ev, XmTextPosition pos)
     }
 }
 
-
+static void DoUnselectionBecauseOfKeyNavigation(Widget w, XEvent *ev)
+{
+    if ( _XmStringSourceHasSelection(GetSrc(w)) )
+    {
+	XmTextSetSelection(w,-1,-1,ev->xkey.time);
+    }
+}
 
 /* Action Routines -------------------------------------------------------- */
 
@@ -902,7 +908,7 @@ static void
 MoveBackwardChar(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveBackwardChar\n"));
-
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     if (Text_CursorPos(w) > 0)
     {
 	_XmTextSetCursorPosition((Widget)w, Text_CursorPos(w) - 1);
@@ -915,6 +921,7 @@ MoveBackwardParagraph(Widget w, XEvent *ev,
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveBackwardParagraph"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     if (Text_CursorPos(w) > 0)
     {
 	XmTextPosition pos;
@@ -934,6 +941,7 @@ MoveBackwardWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MoweBackwardWord"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     if (Text_CursorPos(w) > 0)
     {
 	XmTextPosition pos;
@@ -960,6 +968,7 @@ MoveBeginningOfFile(Widget w, XEvent *ev, String *params,
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveBeginningOfFile"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     if (Text_CursorPos(w) > 0)
     {
 	_XmTextSetCursorPosition(w, 0);
@@ -974,6 +983,7 @@ MoveToLineStart(Widget w, XEvent *ev, String *params,
 
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveToLineStart"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
 				   XmSELECT_LINE, XmsdLeft, 1, False);
 
@@ -1312,6 +1322,7 @@ MoveToLineEnd(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveToLineEnd"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
 				   XmSELECT_LINE, XmsdRight, 1, False);
     _XmTextSetCursorPosition(w, pos);
@@ -1324,6 +1335,7 @@ MoveEndOfFile(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveToEndOfFile"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     pos = Text_LastPos(w);
 
     _XmTextSetCursorPosition(w, pos);
@@ -1457,6 +1469,7 @@ MoveForwardChar(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveForwardChar\n"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     if (Text_CursorPos(w) < Text_LastPos(w))
     {
         _XmTextSetCursorPosition(w, Text_CursorPos(w) + 1);
@@ -1471,6 +1484,7 @@ MoveForwardParagraph(Widget w, XEvent *ev, String *params,
 
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveForwardParagraph"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
 				   XmSELECT_PARAGRAPH, XmsdRight, 1, False);
 
@@ -1489,6 +1503,7 @@ MoveForwardWord(Widget w, XEvent *ev, String *params,
 
     DEBUGOUT(XdbDebug(__FILE__, w, "MoveForwardWord"));
 
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
 				   XmSELECT_WORD, XmsdRight, 1, False);
 
@@ -1844,6 +1859,7 @@ MoveNextLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
     XmTextPosition pos = Text_CursorPos(w);
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MoveNextLine\n"));
+    DoUnselectionBecauseOfKeyNavigation(aw,ev);
 
     index = _XmTextGetTableIndex(w, pos);
     if (index == (maxindex = Text_TotalLines(w) - 1))
@@ -1886,6 +1902,7 @@ MoveNextPage(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 #endif
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MoveNextPage"));
+    DoUnselectionBecauseOfKeyNavigation(aw,ev);
 
     index = _XmTextGetTableIndex(w, pos);
     maxindex = Text_TotalLines(w) - 1;
@@ -1982,12 +1999,14 @@ static void
 MovePageLeft(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MovePageLeft\n"));
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
 }
 
 static void
 MovePageRight(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "MovePageRight\n"));
+    DoUnselectionBecauseOfKeyNavigation(w,ev);
 }
 
 static void
@@ -2015,6 +2034,7 @@ MovePreviousLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MovePreviousLine\n"));
 
+    DoUnselectionBecauseOfKeyNavigation(aw,ev);
     index = _XmTextGetTableIndex(w, pos);
     if (index == 0)
     {
@@ -2051,6 +2071,7 @@ MovePreviousPage(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 
     DEBUGOUT(XdbDebug(__FILE__, aw, "MovePreviousPage"));
 
+    DoUnselectionBecauseOfKeyNavigation(aw,ev);
     index = _XmTextGetTableIndex(w, pos);
     if (index < delta)
     {
