@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.81 2000/04/08 14:54:51 hennessy Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.83 2000/05/24 23:08:54 hennessy Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.81 2000/04/08 14:54:51 hennessy Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.83 2000/05/24 23:08:54 hennessy Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -31,7 +31,9 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v
 #include <Xm/TextOutP.h>
 #include <Xm/ScrolledWP.h>
 #include <Xm/ScrollBarP.h>
+#include <Xm/AtomMgr.h>
 #include <XmI/XmI.h>
+#include <XmI/AtomMgrI.h>
 #include <X11/Xfuncs.h>
 #include <limits.h>		/* for INT_MAX */
 #ifdef HAVE_STRING_H
@@ -129,6 +131,30 @@ static XtResource output_resources[] =
 /*
  * Common initialization procedures -------------------------------------------
  */
+int  _XmFontCalculateAverageCharacterWidth(Widget w, XFontStruct *fs)
+{
+	/*CP: The following is more or less what the XFree86 people have suggested. */
+    unsigned long width;
+    Atom atom;
+
+	/* this is copied from a similar function in Screen.c */
+    if ( (atom = XmInternAtom(XtDisplay(w), _XA_AVERAGE_WIDTH, True) )
+	   && XGetFontProperty(fs, atom, &width) && width )
+	{
+	width =  (width + 5) / 10; /*CP: just to round it up slightly */
+	}
+    else if ( ( ! XGetFontProperty( fs, XA_QUAD_WIDTH, &width)) || width == 0)
+	{
+		/*CP: Yes, even if the property is available it could be 0 - as per XF86 people */
+		/*So we will will use the character "x" */
+	if ( fs->per_char &&  fs->min_char_or_byte2 <= 'x' &&
+			fs->max_char_or_byte2 >= 'x') /* i.e. we have data for x */
+	    width = fs->per_char['a' - fs->min_char_or_byte2].width;
+	else
+	    width = fs->max_bounds.width / 2 ;                                          
+	}
+    return width;
+}
 
 /* FIX ME - this thing is clearly overly simplistic. */
 /* I think it should :
@@ -220,13 +246,7 @@ FontInit(XmTextWidget w)
     {
 	fs = (XFontStruct *)entry->font;
 
-	Out_Font(o) = fs;
-	Out_FontHeight(o) = Out_FontAscent(o) + Out_FontDescent(o);
-	Out_CursorHeight(o) = Out_FontHeight(o);
-	Out_FontAverageWidth(o) =
-	    (fs->max_bounds.width + fs->min_bounds.width) / 2;
-
-	Out_TabWidth(o) = 8 * Out_FontAverageWidth(o);
+	Out_UseFontSet(o) = False;
     }
     else
     {				/* XmFONT_IS_FONTSET */
@@ -250,16 +270,23 @@ FontInit(XmTextWidget w)
 	}
 
 	fs = fsl[0];
-	Out_Font(o) = fs;
-	Out_FontHeight(o) = Out_FontAscent(o) + Out_FontDescent(o);
-	Out_FontAverageWidth(o) =
+/*	Out_UseFontSet(o) = True;
+*/    }
+
+    Out_Font(o) = fs;
+
+    Out_FontAverageWidth(o) =
 	    (fs->max_bounds.width + fs->min_bounds.width) / 2;
 
-	Out_TabWidth(o) = 8 * Out_FontAverageWidth(o);
-    }
-
+    if ( Out_FontAverageWidth(o) <= 0 )
+	{
+	Out_FontAverageWidth(o) = _XmFontCalculateAverageCharacterWidth((Widget)w, fs);
+	}
+    Out_TabWidth(o) = 8 * Out_FontAverageWidth(o);
     Out_Font_Ascent(o) = Out_FontAscent(o);
     Out_Font_Descent(o) = Out_FontDescent(o);
+    Out_FontHeight(o) = Out_FontAscent(o) + Out_FontDescent(o);
+    Out_CursorHeight(o) = Out_FontHeight(o);
     DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 		      "font: ascent=%d descent=%d height=%d avewidth=%d\n",
 		      Out_FontAscent(o), Out_FontDescent(o),
@@ -543,7 +570,24 @@ DrawAll(XmTextWidget w)
 	    np = -1;
 	}
     }
-
+    while ( np != -1 && Text_Line(w)[0].start > np )
+	{
+	highlight = Text_Highlight(w).list[hi].mode;
+	hi++;
+	if (hi < mh)
+	    {
+	    np = Text_Highlight(w).list[hi].position;
+	    }
+	else
+	    {
+	    np = -1;
+	    }
+	}
+    if ( np < Text_Line(w)[0].start )
+	{
+	np = -1;
+	}
+ 
     for (i = 0; i < (Text_LineCount(w) < Out_Rows(o) ?
 		     Text_LineCount(w) : Out_Rows(o)); i++)
     {
@@ -2421,6 +2465,7 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
 
     Out_XOffset(o) = 0;
 
+    /*CP: debugging */
     XtAddEventHandler(aw, FocusChangeMask, False, HandleFocusEvents, NULL);
 
     CursorInit(w);
