@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ToggleBG.c,v 1.69 1999/11/11 13:43:26 rwscott Exp $
+ * $Id: ToggleBG.c,v 1.70 2000/01/16 17:12:21 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ToggleBG.c,v 1.69 1999/11/11 13:43:26 rwscott Exp $";
+static const char rcsid[] = "$Id: ToggleBG.c,v 1.70 2000/01/16 17:12:21 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -40,11 +40,7 @@ static const char rcsid[] = "$Id: ToggleBG.c,v 1.69 1999/11/11 13:43:26 rwscott
 #include <X11/ShellP.h>
 #include <Xm/ToggleBP.h>
 
-#if STDC_HEADERS
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
 #include <XmI/DebugUtil.h>
 
@@ -1686,7 +1682,6 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
     }
 }
 
-#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
@@ -1696,19 +1691,6 @@ MenuProcEntry(int proc, Widget w,...)
     */
 
     va_start(arg_list, w);
-#else
-MenuProcEntry(proc, w, va_alist)
-     int proc;
-     Widget w;
-     va_dcl
-{
-    va_list arglist;
-    /*
-    Cardinal num_params = 0;
-    */
-
-    va_start(arglist);
-#endif
 
     switch (proc)
     {
