@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Traversal.c,v 1.28 1998/10/01 02:23:09 rwscott Exp $
+ * $Id: Traversal.c,v 1.35 2000/04/30 22:38:24 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,12 +22,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Traversal.c,v 1.28 1998/10/01 02:23:09 rwscott Exp $";
+static const char rcsid[] = "$Id: Traversal.c,v 1.35 2000/04/30 22:38:24 amai Exp $";
+
+#include <LTconfig.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <LTconfig.h>
 #include <XmI/XmI.h>
 #include <XmI/TraversalI.h>
 
@@ -56,6 +57,16 @@ Boolean _XmNewTravGraph(XmTravTree tree, Widget shell, Widget first);
 Boolean _XmIsViewable(Widget w);
 Widget _XmIsScrollableClipWidget(Widget w, XRectangle *rect);
 Boolean _XmIntersectionOf(XRectangle *a, XRectangle *b, XRectangle *dest);
+void _XmTravGraphRemove(XmTravTree tree, Widget w);
+void _XmTravGraphAdd(XmTravTree tree, Widget w);
+void _XmTravGraphUpdate(XmTravTree tree, Widget w);
+Boolean _XmGetEffectiveView(Widget w, XRectangle *rect);
+Boolean _XmSetInitialOfTabGraph(XmTravTree tree, Widget tab, Widget first);
+void _XmTabListAdd(XmTravTree tree, Widget w);
+void _XmTabListDelete(XmTravTree tree, Widget w);
+Widget _XmGetClippingAncestor(Widget w, XRectangle *rect);
+Widget _XmTraverse(XmTravTree tree, XmTraversalDirection dir, Widget w);
+
 
 /************************** Tree Manip functions ***************************/
 
@@ -164,7 +175,7 @@ DumpFocusData(XmFocusData fd)
 
 #endif
 
-XmTravTreeNode
+static XmTravTreeNode
 GetNodeOfWidget(XmTravTree tree, Widget w)
 {
     int i;
@@ -186,7 +197,7 @@ GetNodeOfWidget(XmTravTree tree, Widget w)
     return NULL;
 }
 
-XmTravTreeNode
+static XmTravTreeNode
 GetNodeFromGraph(XmTravTreeNode node, Widget first)
 {
     XmTravTreeNode tmp;
@@ -225,7 +236,7 @@ GetNodeFromGraph(XmTravTreeNode node, Widget first)
     }
 }
 
-XmTravTreeNode
+static XmTravTreeNode
 AllocListEntry(XmTravTree tree)
 {
     XmTravTreeNode node;
@@ -256,7 +267,7 @@ AllocListEntry(XmTravTree tree)
     return node;
 }
 
-Boolean
+static Boolean
 GetChildList(Widget w, WidgetList *children, Cardinal *nchildren)
 {
     XmManagerClassExt *mce;
@@ -273,7 +284,7 @@ GetChildList(Widget w, WidgetList *children, Cardinal *nchildren)
     return False;
 }
 
-void
+static void
 GetNodeList(Widget w, XRectangle *rect, XmTravTree tree,
 	    int toffset, int coffset)
 {
@@ -412,7 +423,7 @@ GetNodeList(Widget w, XRectangle *rect, XmTravTree tree,
     }
 }
 
-void
+static void
 GetRectRelativeToShell(Widget w, XRectangle *rect)
 {
     Position cx, cy;
@@ -435,13 +446,13 @@ GetRectRelativeToShell(Widget w, XRectangle *rect)
     rect->y = cy;
 }
 
-int
+static int
 CompareExcls(XmConst void *a, XmConst void *b)
 {
     return 0;
 }
 
-int
+static int
 CompareNodesVert(XmConst void *a, XmConst void *b)
 {
     XmTravTreeNode nodea = *(XmTravTreeNode *)a, nodeb = *(XmTravTreeNode *)b;
@@ -472,7 +483,7 @@ CompareNodesVert(XmConst void *a, XmConst void *b)
     }
 }
 
-int
+static int
 CompareNodesHoriz(XmConst void *a, XmConst void *b)
 {
     XmTravTreeNode nodea = *(XmTravTreeNode *)a, nodeb = *(XmTravTreeNode *)b;
@@ -503,7 +514,7 @@ CompareNodesHoriz(XmConst void *a, XmConst void *b)
     }
 }
 
-void
+static void
 SortGraph(XmTravTreeNode node, Boolean have_excls)
 {
     XmTravTreeNode *block, tmp, *ptr;
@@ -587,7 +598,7 @@ SortGraph(XmTravTreeNode node, Boolean have_excls)
     XtFree((char *)block);
 }
 
-XmTravTreeNode
+static XmTravTreeNode
 GetNextNearestNode(XmTravTreeNode node, XRectangle *rect)
 {
     XmTravTreeNode tmp, *block;
@@ -654,7 +665,7 @@ GetNextNearestNode(XmTravTreeNode node, XRectangle *rect)
     return tmp;
 }
 
-void
+static void
 LinkNodeList(XmTravTree tree)
 {
     unsigned short i;
@@ -703,7 +714,7 @@ LinkNodeList(XmTravTree tree)
     DEBUGOUT(DumpTree(tree));
 }
 
-void
+static void
 SortNodeList(XmTravTree tree)
 {
     int i;
@@ -733,7 +744,7 @@ SortNodeList(XmTravTree tree)
     DEBUGOUT(DumpTree(tree));
 }
 
-Boolean
+static Boolean
 NodeIsTraversable(XmTravTreeNode node)
 {
     if (node == NULL)
@@ -754,7 +765,7 @@ NodeIsTraversable(XmTravTreeNode node)
     return XmIsTraversable(node->widget);
 }
 
-XmTravTreeNode
+static XmTravTreeNode
 NextControl(XmTravTreeNode node)
 {
     XmTravTreeNode last, next, ptr, start;
@@ -790,7 +801,7 @@ NextControl(XmTravTreeNode node)
     return next;
 }
 
-XmTravTreeNode
+static XmTravTreeNode
 PrevControl(XmTravTreeNode node)
 {
     XmTravTreeNode last, prev, ptr, start;
@@ -826,7 +837,7 @@ PrevControl(XmTravTreeNode node)
     return prev;
 }
 
-XmTravTreeNode
+static XmTravTreeNode
 TraverseControl(XmTravTreeNode node, XmTraversalDirection dir)
 {
     XmTravTreeNode tmp, last;
@@ -985,7 +996,7 @@ TraverseControl(XmTravTreeNode node, XmTraversalDirection dir)
     return tmp;
 }
 
-XmTravTreeNode
+static XmTravTreeNode
 TraverseTab(XmTravTreeNode node, XmTraversalDirection dir)
 {
     XmTravTreeNode tmp, last;
@@ -1180,7 +1191,7 @@ TraverseTab(XmTravTreeNode node, XmTraversalDirection dir)
     return tmp;
 }
 
-Boolean
+static Boolean
 EffectiveView(Widget w, XRectangle *src, XRectangle *rect)
 {
     Boolean no_scr_par = True;
@@ -1261,7 +1272,7 @@ EffectiveView(Widget w, XRectangle *src, XRectangle *rect)
     return True;
 }
 
-Widget
+static Widget
 FindFirstManaged(Widget w)
 {
     int i;
@@ -1281,7 +1292,7 @@ FindFirstManaged(Widget w)
     return NULL;
 }
 
-Boolean
+static Boolean
 SetInitialNode(XmTravTreeNode root, XmTravTreeNode leaf)
 {
     DEBUGOUT(XdbDebug(__FILE__, root->widget, "SetInitialNode\n"));
@@ -1317,7 +1328,7 @@ SetInitialNode(XmTravTreeNode root, XmTravTreeNode leaf)
     return True;
 }
 
-int
+static int
 SearchTabList(XmTravTree tree, Widget w)
 {
     int i;
@@ -1334,7 +1345,7 @@ SearchTabList(XmTravTree tree, Widget w)
     return -1;
 }
 
-void
+static void
 DeleteFromTabList(XmTravTree tree, int node)
 {
     DEBUGOUT(XdbDebug(__FILE__, NULL, "DeleteFromTabList\n"));
@@ -1351,7 +1362,7 @@ DeleteFromTabList(XmTravTree tree, int node)
     tree->excl_tabs[tree->num_tab_entries] = NULL;
 }
 
-void
+static void
 SetInitialWidgets(XmTravTree tree)
 {
     XmTravTreeNode tmp, child;
@@ -1391,7 +1402,7 @@ SetInitialWidgets(XmTravTree tree)
     DEBUGOUT(DumpTree(tree));
 }
 
-Boolean
+static Boolean
 InitializeCurrent(XmTravTree tree, Widget w, Boolean check)
 {
     XmTravTreeNode node;
@@ -1482,7 +1493,7 @@ InitializeCurrent(XmTravTree tree, Widget w, Boolean check)
     return True;
 }
 
-Widget
+static Widget
 FindFirstFocus(Widget w)
 {
     Widget shell = _XmFindTopMostShell(w);
@@ -1683,7 +1694,7 @@ _XmGetFocusPolicy(Widget w)
 }
 
 XmFocusData
-_XmCreateFocusData()
+_XmCreateFocusData(void)
 {
     DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmCreateFocusData\n"));
 
@@ -1719,7 +1730,8 @@ _XmGetFocusData(Widget wid)
 	wid = XtParent(wid);
     }
 
-	DEBUGOUT(XdbDebug2(__FILE__, orig, wid, "_XmGetFocusData: Found shell\n"));
+    DEBUGOUT(XdbDebug2("RWS", wid, orig, "_XmGetFocusData: Found shell\n"));
+    DEBUGOUT(XdbDebug2(__FILE__, orig, wid, "_XmGetFocusData: Found shell\n"));
 
     if (!wid)
     {
@@ -1850,14 +1862,17 @@ _XmIsTraversable(Widget w, Boolean vischeck)
     XRectangle rect;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmIsTraversable\n"));
+    DEBUGOUT(XdbDebug("MENU", w, "_XmIsTraversable\n"));
 
     if (w == NULL || !XtIsManaged(w))
     {
+	DEBUGOUT(XdbDebug0("MENU", w, "\tNot-managed\n"));
 	return False;
     }
 
     if (!_XmIsNavigable(w))
     {
+	DEBUGOUT(XdbDebug0("MENU", w, "\tNot-navigable\n"));
 	return False;
     }
 
@@ -1880,6 +1895,7 @@ _XmIsNavigable(Widget w)
     XmNavigability nav;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmIsNavigable()\n"));
+    DEBUGOUT(XdbDebug("MENU", w, "_XmIsNavigable()\n"));
 
     if (!w)
     {
@@ -1889,7 +1905,8 @@ _XmIsNavigable(Widget w)
     nav = _XmGetNavigability(w);
     if (nav != XmTAB_NAVIGABLE && nav != XmCONTROL_NAVIGABLE)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w, " returns False 1: %d\n", nav));
+	DEBUGOUT(XdbDebug(__FILE__, w, " returns False 1: %s\n", XdbNavigability2String(nav)));
+	DEBUGOUT(XdbDebug0("MENU", w, " returns False 1: %s\n", XdbNavigability2String(nav)));
 
 	return False;
     }
@@ -1906,7 +1923,8 @@ _XmIsNavigable(Widget w)
 	nav = _XmGetNavigability(w);
 	if (nav == XmNOT_NAVIGABLE)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, " returns False 2: %d\n", nav));
+	    DEBUGOUT(XdbDebug(__FILE__, w, " returns False 2: %s\n", XdbNavigability2String(nav)));
+	    DEBUGOUT(XdbDebug("MENU", w, " returns False 2: %s\n", XdbNavigability2String(nav)));
 	    return False;
 	}
 
@@ -1914,6 +1932,7 @@ _XmIsNavigable(Widget w)
     }
 
     DEBUGOUT(XdbDebug(__FILE__, w, " returns True\n"));
+    DEBUGOUT(XdbDebug("MENU", w, " returns True\n"));
 
     return True;
 }
@@ -2493,6 +2512,9 @@ _XmCallFocusMoved(Widget old,
     {
 	tsh = _XmFindTopMostShell(new_wid);
     }
+    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmCallFocusMoved(): tsh %s\n",
+		      tsh ? XtName(tsh) : "(null)"
+		      ));
 
     if (!XtIsSubclass(tsh, vendorShellWidgetClass))
     {
@@ -2522,7 +2544,7 @@ _XmCallFocusMoved(Widget old,
     return cbs.cont;
 }
 
-Boolean
+static Boolean
 _XmCallTraverseObscured(Widget w, XmTraversalDirection dir)
 {
     XRectangle rect;
@@ -2844,6 +2866,7 @@ _XmMgrTraversal(Widget widget, XmTraversalDirection direction)
     static Boolean in_traversal = False;
     Boolean ret = False;
 
+    shell = _XmFindTopMostShell(widget);
     DEBUGOUT(XdbDebug(__FILE__, widget, "_XmMgrTraversal\n"));
 
     if (in_traversal)
@@ -2892,6 +2915,12 @@ _XmMgrTraversal(Widget widget, XmTraversalDirection direction)
     {
 	DEBUGOUT(XdbDebug(__FILE__, widget, "_XmMgrTraversal: _XmTraverse\n"));
 	widget = _XmTraverse(&fd->tree, direction, widget);
+	if (widget && shell != _XmFindTopMostShell(widget))
+	{
+		/* FIX ME */
+		_XmWarning(shell, "%s:_XmMgrTraversal(%d) - I just Traversed into a different shell!!!", __FILE__, __LINE__);
+		widget = NULL;
+	}
     }
 
     if (widget == NULL || widget != cur_focus || fd->old_focus_item == NULL)
@@ -2930,6 +2959,27 @@ _XmMgrTraversal(Widget widget, XmTraversalDirection direction)
 	    fd->focus_item = widget;
 	    fd->old_focus_item = cur_focus;
 
+if (strcmp(XtName(shell), "mgdiff") == 0 &&
+    strcmp(XtName(widget), "Text") == 0)
+{
+Widget tmp = widget;
+
+    printf("%s %d\n", __FILE__, __LINE__);
+    while (tmp != NULL)
+    {
+	    printf("%s %s %s\n", 
+		    XtName(tmp),
+		    XtClass(tmp)->core_class.class_name,
+		    _XmFindTopMostShell(tmp) ? XtName(_XmFindTopMostShell(tmp)) : "NULL"
+		    );
+	    tmp = XtParent(tmp);
+    }
+}
+	    DEBUGOUT(XdbDebug("RWS1", shell,
+			      "KeyboardFocus set to %s %s %p\n",
+			      shell ? XtName(shell) : "(null)",
+			      widget ? XtName(widget) : "(null)",
+			      widget));
 	    DEBUGOUT(XdbDebug(__FILE__, shell,
 			      "KeyboardFocus set to %s %s\n",
 			      shell ? XtName(shell) : "(null)",
@@ -3067,9 +3117,13 @@ _XmNavigInitialize(Widget request,
 		   ArgList args,
 		   Cardinal *num_args)
 {
-    XmFocusData fd = _XmGetFocusData(new_wid);
+    XmFocusData fd;
     XmNavigationType nt = _XmGetNavigationType(new_wid);
 
+    DEBUGOUT(XdbDebug("RWS", new_wid, "_XmNavigInitialize\n"));
+    DEBUGOUT(XdbDebug(__FILE__, new_wid, "_XmNavigInitialize\n"));
+
+    fd = _XmGetFocusData(new_wid);
     if (fd == NULL)
     {
 	DEBUGOUT(XdbDebug(__FILE__, new_wid,
@@ -3405,12 +3459,16 @@ _XmNavigDestroy(Widget wid)
     XmNavigationType nt;
     Widget tmp, shell;
 
+    DEBUGOUT(XdbDebug("RWS", wid, "_XmNavigDestroy\n"));
     DEBUGOUT(XdbDebug(__FILE__, wid, "_XmNavigDestroy\n"));
 
     if (!fd)
     {
 	return;
     }
+    DEBUGOUT(XdbDebug("RWS", wid, "_XmNavigDestroy %s\n",
+    	fd->focus_item ? XtName(fd->focus_item) : "NULL"
+    	));
 
     if (fd->first_focus == wid)
     {
@@ -3457,6 +3515,9 @@ _XmNavigDestroy(Widget wid)
 		fd->focus_item = NULL;
 	    }
 	}
+    DEBUGOUT(XdbDebug("RWS", wid, "_XmNavigDestroy %s\n",
+    	fd->focus_item ? XtName(fd->focus_item) : "NULL"
+    	));
     }
 
     if (fd->tree.num_entries != 0)
@@ -3617,7 +3678,7 @@ XmGetFocusWidget(Widget widget)
 	return NULL;
     }
 
-    if (XmIsManager(w))
+    if (XmIsManager(w) && MGR_HighlightedWidget(w))
     {
 	return MGR_HighlightedWidget(w);
     }
