@@ -1,6 +1,6 @@
 /**
  *
- * $Id: VaSimple.c,v 1.3 1998/09/12 15:05:50 rwscott Exp $
+ * $Id: VaSimple.c,v 1.8 2000/04/19 22:27:27 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -49,7 +49,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: VaSimple.c,v 1.3 1998/09/12 15:05:50 rwscott Exp $";
+static const char rcsid[] = "$Id: VaSimple.c,v 1.8 2000/04/19 22:27:27 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -80,13 +80,6 @@ extern Widget _XtCreateWidget(String,
 			      ArgList, Cardinal,
 			      XtTypedArgList, Cardinal);
 
-extern Widget _XmCreateSimpleGadget(char *,
-				    Widget parent,
-				    int def_type,
-				    XmSimpleMenu data,
-				    int which,
-				    ArgList args, int argc);
-
 /*
  *    Given a nested list, _XmCountNestedList() returns counts of the
  *    total number of attribute-value pairs and the count of those
@@ -692,16 +685,18 @@ XmVaCreateSimpleCheckBox(Widget parent,
 
     DEBUGOUT(XdbDebug(__FILE__, parent, "XmVaCreateSimpleCheckBox();\n"));
 
+#ifdef SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     /* find out how many buttons, how many args, etc... we have 
      * (this also sets the button_strings, button_mnemonics, etc
      * arrays)
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
@@ -726,7 +721,7 @@ XmVaCreateSimpleCheckBox(Widget parent,
      * if it can be set afterward, but it is easier to set it
      * afterward
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
@@ -762,16 +757,18 @@ XmVaCreateSimpleMenuBar(Widget parent,
 
     DEBUGOUT(XdbDebug(__FILE__, parent, "XmVaCreateSimpleMenuBar();\n"));
 
+#ifdef SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     /* find out how many buttons, how many args, etc... we have 
      * (this also sets the button_strings, button_mnemonics, etc
      * arrays)
      */
-    Va_start(var, name);
+    va_start(var, name);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
@@ -787,7 +784,7 @@ XmVaCreateSimpleMenuBar(Widget parent,
      * if it can be set afterward, but it is easier to set it
      * afterward
      */
-    Va_start(var, name);
+    va_start(var, name);
     _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
@@ -825,20 +822,22 @@ XmVaCreateSimpleOptionMenu(Widget parent,
 
     DEBUGOUT(XdbDebug(__FILE__, parent, "XmVaCreateSimpleOptionMenu();\n"));
 
+#ifdef SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     /* find out how many buttons, how many args, etc... we have 
      * (this also sets the button_strings, button_mnemonics, etc
      * arrays)
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
@@ -889,16 +888,24 @@ XmVaCreateSimplePopupMenu(Widget parent,
 
     DEBUGOUT(XdbDebug(__FILE__, parent, "XmVaCreateSimplePopupMenu();\n"));
 
+#ifdef SIMPLE_WALKUP
+	/*
+	 * Jex demonstrates that this shouldn't happen.
+	 * TFM also says
+	 *	"parent - Specifies the widget ID of the parent of the MenuShell"
+	 * which seems to confirm that.
+	 */
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     /* find out how many buttons, how many args, etc... we have 
      * (this also sets the button_strings, button_mnemonics, etc
      * arrays)
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
@@ -918,7 +925,7 @@ XmVaCreateSimplePopupMenu(Widget parent,
      * if it can be set afterward, but it is easier to set it
      * afterward
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
@@ -957,17 +964,19 @@ XmVaCreateSimplePulldownMenu(Widget parent,
 
     DEBUGOUT(XdbDebug(__FILE__, parent, "XmVaCreateSimplePulldownMenu();\n"));
 
+#ifdef SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
 
     /* find out how many buttons, how many args, etc... we have 
      * (this also sets the button_strings, button_mnemonics, etc
      * arrays)
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
@@ -988,7 +997,7 @@ XmVaCreateSimplePulldownMenu(Widget parent,
      * if it can be set afterward, but it is easier to set it
      * afterward
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
@@ -1024,16 +1033,18 @@ XmVaCreateSimpleRadioBox(Widget parent,
 
     DEBUGOUT(XdbDebug(__FILE__, parent, "XmVaCreateSimpleRadioBox();\n"));
 
+#ifdef SIMPLE_WALKUP
     while (parent && !XtIsComposite(parent))
     {
 	parent = XtParent(parent);
     }
+#endif
 
     /* find out how many buttons, how many args, etc... we have 
      * (this also sets the button_strings, button_mnemonics, etc
      * arrays)
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmCountVaList(var, &button_count, &args_count, &typed_count, &total_count);
     va_end(var);
 
@@ -1059,7 +1070,7 @@ XmVaCreateSimpleRadioBox(Widget parent,
      * if it can be set afterward, but it is easier to set it
      * afterward
      */
-    Va_start(var, callback);
+    va_start(var, callback);
     _XmVaToArgList(rc, var, total_count, &arglist, &n);
     va_end(var);
 
