@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Vendor.c,v 1.51 2000/01/26 12:43:53 amai Exp $
+ * $Id: Vendor.c,v 1.58 1994/06/05 13:13:36 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,12 +22,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Vendor.c,v 1.51 2000/01/26 12:43:53 amai Exp $";
+static const char rcsid[] = "$Id: Vendor.c,v 1.58 1994/06/05 13:13:36 amai Exp $";
+
+#include <LTconfig.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <LTconfig.h>
 #include <XmI/XmI.h>
 
 #include <X11/IntrinsicP.h>
@@ -1446,6 +1447,10 @@ LTAddGrab(Widget wid, Boolean exclusive, Boolean spring_loaded,
 
     Display_NumModals(d)++;
 
+    DEBUGOUT(XdbDebug("GRAB", wid, "%s:XtAddGrab(%d) - %s %s\n", 
+    	__FILE__, __LINE__,
+    	exclusive ? "True" : "False",
+    	spring_loaded ? "True" : "False"));
     XtAddGrab(wid, exclusive, spring_loaded);
 
     /*
@@ -1478,6 +1483,10 @@ _XmAddGrab(Widget wid, Boolean exclusive, Boolean spring_loaded)
  * The third parameter "remove_grab" is just a small optimization, so we
  * don't need to remove callbacks and grabs if the widget to be removed
  * from the grab list is already in the phase of being destroyed.
+ * There's an exception though:
+ *  It seems that Xt is cleaning up the grabs okay
+ *  for anything except topLevelShells!
+ *
  * One last reflection about what we're doing here:
  *   Especially during destruction of a widget we must restore right at
  *   this place all those grabs that have been issued after the grab on
@@ -1531,8 +1540,20 @@ LTRemoveGrab(Widget wid, XmVendorShellExtObject ve,
 
     for (i = Display_NumModals(d); i > 0; i--, ps++)
     {
-	if ((ps->wid == wid) && !CoreBeingDestroyed(wid) && remove_grab_physically)
+/* amai: The famous "NEdit freeze" bug (an out of sync problem between
+         the Xt and LT grab list) was cured by changing
+           if ((ps->wid == wid) && !CoreBeingDestroyed(wid) && remove_grab_physically)
+	to
+           if ((ps->wid == wid) && remove_grab_physically)
+	
+	This 'fix' in turn triggered XtRemoveGrabs with bogu arguments, leading
+	to Xt warning messages. See comments at beginning of LTRemoveGrab()!
+*/
+	if ((XtParent(wid) == NULL) || ((ps->wid == wid) && !CoreBeingDestroyed(wid) && remove_grab_physically))
 	{
+	    DEBUGOUT(XdbDebug2("GRAB", XtParent(wid), wid, "%s:XtRemoveGrab(%d) - %s %s\n", __FILE__, __LINE__,
+	    	XtIsShell(wid) ? "True" : "False",
+	    	CoreBeingDestroyed(wid) ? "True" : "False"));
 	    XtRemoveGrab(wid);
 	}
     }
@@ -1579,6 +1600,10 @@ LTRemoveGrab(Widget wid, XmVendorShellExtObject ve,
 	if (pd != ps)
 	{
 	    *pd = *ps;
+	    DEBUGOUT(XdbDebug("GRAB", pd->wid, "%s:XtAddGrab(%d) - %s %s\n", 
+		__FILE__, __LINE__,
+		pd->exclusive ? "True" : "False",
+		pd->springLoaded ? "True" : "False"));
 	    XtAddGrab(pd->wid, pd->exclusive, pd->springLoaded);
 	}
     }
@@ -1599,6 +1624,7 @@ LTRemoveGrabCallback(Widget wid, XtPointer client_data,
 		     XtPointer callback_data)
 {
     DEBUGOUT(XdbDebug(__FILE__, wid, "LTRemoveGrabCallback()\n"));
+    DEBUGOUT(XdbDebug("GRAB", wid, "LTRemoveGrabCallback()\n"));
 #if 0
     /* rws 14 Feb 1998
        This is the cause of the segfault in ddd when pressing OK from
@@ -2039,7 +2065,7 @@ ConfigEventForMe(Display *dpy, XEvent *event, char *arg)
 }
 
 
-static
+static int
 WaitForWM(ShellWidget w, XEvent *event, unsigned long request_num)
 {
     XtAppContext app = XtWidgetToApplicationContext((Widget)w);
@@ -3147,7 +3173,17 @@ _XmGetDefaultDisplay(void)
 unsigned char
 _XmGetAudibleWarning(Widget w)
 {
-    return 0;
+    XmWidgetExtData data;
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmGetAudibleWarning\n"));
+    if ( XmIsVendorShell (w) && ( data = _XmGetWidgetExtData(w, XmSHELL_EXTENSION ) ) )
+	{
+	XmVendorShellExtObject vSE = (XmVendorShellExtObject)data -> widget ;
+	unsigned char audibleWarning = vSE -> vendor.audible_warning ;
+	return audibleWarning ;
+	}
+    
+    _XmWarning(w, "_XmGetAudibleWarning: widget has invalid class");
+    return False;
 }
 
 char *
@@ -3190,10 +3226,38 @@ _LtFindVendorExt(Widget w)
     }
 }
 
-/* This fixes different shared library mechanism in OS/2 
+/*
+From: Martin Simmons <martin@harlequin.co.uk>
+
+  Perhaps LessTif can guard against this common linking problem?  
+  E.g. the following function will check the VendorShell for consistency:
+*/
+
+/* To be used for sanity checking */
+Boolean
+_LtCheckClassOfVendorShell(Widget w)
+{
+    WidgetClass class;
+
+    for (class = XtClass(w); class != NULL; class = class->core_class.superclass)
+    {
+        if (0 == strcmp(class->core_class.class_name, "VendorShell")) {
+            if (class->core_class.extension == (XtPointer)&_XmVendorSCoreClassExtRec)
+                return TRUE;
+            else
+                _XmError(w, "Application not linked correctly: try putting Xm before Xt.");
+        }
+    }
+
+    return FALSE;
+}
+
+/* This corrects for the different shared library loading mechanism in OS/2 
    compared to the standard Unix OS's.
    The routine should be called before any other function,
    done by the _DLL_InitTerm-function under OS/2.
+   On un*x/ELF systems this seems to be ensured by specifying the
+   libs in correct, canonical order on the linker command line
  */
 #ifdef __EMX__
 static void
