@@ -1,6 +1,6 @@
 /*
 
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.46 2000/02/03 17:55:37 amai Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.47 2000/06/14 20:00:12 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.46 2000/02/03 17:55:37 amai Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.47 2000/06/14 20:00:12 danny Exp $";
 
 /*
  * ** Danny had added a flag here, but I think the behavior he added is always
@@ -1233,7 +1233,7 @@ _XmStringFree(_XmString string)
 char *
 _XmStringGetCurrentCharset(void)
 {
-    char *lang;
+    char *lang, *p;
 
     if ((lang = getenv("LANG")) == NULL)
     {
@@ -1248,7 +1248,11 @@ _XmStringGetCurrentCharset(void)
 	return XmSTRING_OS_CHARSET;
     }
 
-    return lang;
+    if ((p = strchr(lang, '.')) != NULL && *(p+1) != 0) {
+	return p+1;
+    }
+
+    return XmFALLBACK_CHARSET;
 }
 
 char *
