@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Xmos.c,v 1.23 1998/10/17 23:40:38 gritton Exp $
+ * $Id: Xmos.c,v 1.28 2000/02/03 17:55:38 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Xmos.c,v 1.23 1998/10/17 23:40:38 gritton Exp $";
+static const char rcsid[] = "$Id: Xmos.c,v 1.28 2000/02/03 17:55:38 amai Exp $";
 
 #include <LTconfig.h>
 
@@ -215,7 +215,7 @@ startsWithTwiddle(String dir)
 static String
 convertTwiddle(char *dir)
 {
-    String ptr;
+    String ptr, ptr1;
     String newdir, home;
 
     for (ptr = dir; *ptr && isspace(*ptr); ptr++)
@@ -223,10 +223,33 @@ convertTwiddle(char *dir)
     }
     ptr++;			/* to get by twiddle */
 
-    home = _XmOSGetHomeDirName();
-    newdir = XtMalloc(strlen(home) + strlen(ptr) + 1);
+    ptr1 = strchr(ptr, '/');
+    if (ptr == ptr1)
+    {
+	home = _XmOSGetHomeDirName();
+    }
+    else
+    {
+	String name, ptr2;
+	struct passwd *pw;
+
+    	name = XtNewString(ptr);
+    	ptr2 = strchr(name, '/');
+    	*ptr2 = '\0';
+    	pw = getpwnam(name);
+    	if (pw)
+    	{
+	    home = XtNewString(pw->pw_dir);
+    	}
+    	else
+    	{
+	    home = XtNewString("/");
+    	}
+    	XtFree(name);
+    }
+    newdir = XtMalloc(strlen(home) + strlen(ptr1) + 1);
     strcpy(newdir, home);
-    strcat(newdir, ptr);
+    strcat(newdir, ptr1);
 
     return newdir;
 }
@@ -300,6 +323,14 @@ _XmOSQualifyFileSpec(String dirSpec,
 	dir = (String)XtRealloc(dir, strlen(dir) + 2);
 	strcat(dir, "/");
     }
+    if (startsWithTwiddle(dir))
+    {
+	char *newdir;
+
+	newdir = convertTwiddle(dir);
+	XtFree(dir);
+	dir = newdir;
+    }
     /* if the dir doesn't start in a '/', add the cwd */
     if (dir[0] != '/')
     {
@@ -316,6 +347,7 @@ _XmOSQualifyFileSpec(String dirSpec,
 	dir = tmp1;
     }
 
+    if (!filterSpec) filterSpec = "";
     /* if the filter starts with an absolute pathname, use it */
     if (*filterSpec == '/')
     {
@@ -430,7 +462,7 @@ _XmOSQualifyFileSpec(String dirSpec,
  * Takes a string and converts /this/dir/. into /this/dir/
  * also converts /this/dir/.. into /this/
  */
-void
+static void
 _XmOSGetDotDot(String s)
 {
     int i, j;
@@ -821,5 +853,30 @@ _XmOSFileCompare(XmConst void *sp1, XmConst void *sp2)
     return strcmp(*((char **)sp1), *((char **)sp2));
 }
 
+
 char _XmSDEFAULT_FONT[] = "fixed";
 char _XmSDEFAULT_BACKGROUND[] = "Blue";
+
+
+/*
+ * Not sure what this thing does but it's rumoured to exist
+   amai: based on a look at some Motif libs I guess it's
+         a 'ghost' induced by typo only ... See below!
+ */
+void
+XmOSInitPath()
+{
+ _XmWarning(NULL, "XmOSInitPath() is not yet implemented!");
+}
+
+
+/*
+ * This one seems to exist in Motif 1.2 and above ...
+   Probably contrary to XmOSInitPath()
+ */
+String
+_XmOSInitPath(String file_name, String env_pathname, Boolean *user_path)
+{
+ _XmWarning(NULL, "_XmOSInitPath() is not yet implemented!");
+ return (String)NULL; /* we need a return to keep compiler happy */
+}
