@@ -4,7 +4,6 @@
 
 #include <Xm/BulletinB.h>
 #include <Xm/ComboBoxP.h>
-#include "XdbPrintTree.c"
 #include "XdbPrintResources.c"
 
 #if !defined(CB_List)
@@ -12,6 +11,33 @@
         (((XmComboBoxWidget)(w))->combo_box.list)
 #endif
 
+void
+check_geometry(Widget w)
+{
+   static int result_index = 0;
+
+static XtWidgetGeometry Expected[] = {
+/* result test 0 */
+{  CWWidth | CWHeight            ,   10,   10,  418,   58, 0,0,0 }, /* combo */
+{  CWWidth | CWHeight | CWX | CWY,    6,    6,  370,   46, 0,0,0 }, /* Text */
+{  CWWidth | CWHeight | CWX | CWY,    0,    0,  414,   42, 0,0,0 }, /* GrabShell */
+{  CWWidth | CWHeight | CWX | CWY,    0,    0,  410,   38, 0,0,0 }, /* ListSW */
+{  CWWidth | CWHeight | CWX | CWY,  395,    0,   15,   38, 0,0,0 }, /* VertScrollBar */
+{  CWWidth | CWHeight | CWX | CWY,    0,    0,  395,   38, 0,0,0 }, /* List */
+};
+
+#if 0
+   PrintDetails2(w, NULL);
+#else
+  if (result_index <= 0)
+  {
+     PrintDetails2(w, Expected);
+     fflush(stdout);
+     result_index ++;
+  }
+#endif
+}
+
 int
 main(int argc, char **argv)
 {
@@ -48,15 +74,15 @@ main(int argc, char **argv)
 
   XtRealizeWidget(toplevel);
 
-  #if 1
-  XdbPrintCompleteTree(toplevel);
-
+#if 1
   XdbPrintResources(CB_ListShell(widget));
 
   XdbPrintResources(widget);
-  #endif
+#endif
 
-  XtAppMainLoop(app);
+  check_geometry(widget);
+
+  LessTifTestMainLoop(toplevel);
 
   exit(0);
 }
