@@ -1,8 +1,10 @@
 /*
+ * $Id: test1.c,v 1.2 2000/01/28 18:15:24 amai Exp $
  * Produce a C source file that contains the Motif private error
  * messages.
  */
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
 
@@ -562,7 +564,8 @@ char *copyright =
 " *\n"
 " **/\n";
 
-main()
+int 
+main(int argc, char *argv[])
 {
 	int	i;
 	char	*p;
@@ -597,5 +600,6 @@ main()
 		putchar('\n');
 		putchar('\n');
 	}
+	exit(0);
 }
 
