@@ -3,6 +3,7 @@
  * messages.
  */
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
 
@@ -562,7 +563,8 @@ char *copyright =
 " *\n"
 " **/\n";
 
-main()
+int
+main(int argc, char *argv[])
 {
 	int	i;
 	char	*p;
@@ -587,5 +589,6 @@ main()
 	printf("\n");
 	printf("#endif /* XM_MESSAGES_I_H */\n");
 	printf("\n");
+	exit(0);
 }
 
