@@ -1,7 +1,9 @@
+#include <stdlib.h>
 #include <Xm/PushB.h>
 #include <Xm/FrameP.h>
+#ifdef LESSTIF_VERSION
 #include <XmI/MacrosI.h>
-
+#endif
 Widget toplevel,
        canvas, 
        canvas_frame;
@@ -30,8 +32,20 @@ int main(
 				   canvas_frame, NULL, 0);
 
     XtRealizeWidget(toplevel);
-    printf("PC: %d\n", Frame_ProcessingConstraints(canvas_frame));
-    XtMainLoop();
+#ifdef LESSTIF_VERSION
+    printf("(IGNORE_IN_DIFF) PC: %d\n", Frame_ProcessingConstraints(canvas_frame));
+#endif  
+{
+static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,  506,  322,  500,  500, 0,0,0, /* canvas_frame */
+   CWWidth | CWHeight | CWX | CWY,    1,    1,  498,  498, 0,0,0, /* canvas */
+};
+/* toplevel should be replaced with to correct applicationShell */
+PrintDetails(toplevel, Expected);
+}
+
+  LessTifTestMainLoop(toplevel);
+
     exit(0);
 }
 
