@@ -16,6 +16,7 @@ main(int argc,
 	String quald, qualpat, *entries;
 	unsigned int numents, numalloc, i;
 
+#if NOT_AUTOMATIC
 	entries = NULL;
 	numents = numalloc = 0;
 	for (;;) {
@@ -36,5 +37,6 @@ main(int argc,
 		for (i = 0; i < numents; i++)
 			printf("Entry %d: '%s'\n", i, entries[i]);
 	}
+#endif
 	exit(0);
 }
